@@ -18,11 +18,13 @@
 
 #include "config.h"
 
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 
 #include "gegl-options.h"
+#include <gegl.h>
 
 static GeglOptions *opts_new (void)
 {
@@ -34,6 +36,7 @@ static GeglOptions *opts_new (void)
   o->files    = NULL;
   o->file     = NULL;
   o->rest     = NULL;
+  o->scale    = 1.0;
   return o;
 }
 
@@ -41,11 +44,17 @@ static G_GNUC_NORETURN void
 usage (char *application_name)
 {
     fprintf (stderr,
-"usage: %s [options] <file | -- [op [op] ..]>\n"
+_("usage: %s [options] <file | -- [op [op] ..]>\n"
 "\n"
 "  Options:\n"
 "     -h, --help      this help information\n"
 "\n"
+"     --list-all      list all known operations\n"
+"\n"
+"     --exists        return 0 if the operation(s) exist\n"
+"\n"
+"     --properties    output the properties (name, type, description) of the operation\n"
+"\n"
 "     -i, --file      read xml from named file\n"
 "\n"
 "     -x, --xml       use xml provided in next argument\n"
@@ -58,6 +67,8 @@ usage (char *application_name)
 "     -p              increment frame counters of various elements when\n"
 "                     processing is done.\n"
 "\n"
+"     -s scale, --scale scale  scale output dimensions by this factor.\n"
+"\n"
 "     -X              output the XML that was read in\n"
 "\n"
 "     -v, --verbose   print diagnostics while running\n"
@@ -65,7 +76,7 @@ usage (char *application_name)
 "All parameters following -- are considered ops to be chained together\n"
 "into a small composition instead of using an xml file, this allows for\n"
 "easy testing of filters. Be aware that the default value will be used\n"
-"for all properties.\n"
+"for all properties.\n")
 , application_name);
     exit (0);
 }
@@ -73,7 +84,7 @@ usage (char *application_name)
 #define match(string) (!strcmp (*curr, (string)))
 #define assert_argument() do {\
     if (!curr[1] || curr[1][0]=='-') {\
-        fprintf (stderr, "ERROR: '%s' option expected argument\n", *curr);\
+        fprintf (stderr, _("ERROR: '%s' option expected argument\n"), *curr);\
         exit(-1);\
     }\
 }while(0)
@@ -112,26 +123,27 @@ print_opts (GeglOptions *o)
   switch (o->mode)
     {
       case GEGL_RUN_MODE_DISPLAY:
-        mode_str = "Display on screen"; break;
+        mode_str = _("Display on screen"); break;
       case GEGL_RUN_MODE_XML:
-        mode_str = "Print XML"; break;
+        mode_str = _("Print XML"); break;
       case GEGL_RUN_MODE_OUTPUT:
-        mode_str = "Output in a file"; break;
+        mode_str = _("Output in a file"); break;
       case GEGL_RUN_MODE_HELP:
-        mode_str = "Display help information"; break;
+        mode_str = _("Display help information"); break;
       default:
-        g_warning ("Unknown GeglOption mode: %d", o->mode);
+        g_warning (_("Unknown GeglOption mode: %d"), o->mode);
+        mode_str = _("unknown mode");
         break;
     }
 
     fprintf (stderr,
-"Parsed commandline:\n"
+_("Parsed commandline:\n"
 "\tmode:   %s\n"
 "\tfile:   %s\n"
 "\txml:    %s\n"
 "\toutput: %s\n"
 "\trest:   %s\n"
-"\t\n",
+"\t\n"),
     mode_str,
     o->file==NULL?"(null)":o->file,
     o->xml==NULL?"(null)":o->xml,
@@ -210,6 +222,67 @@ parse_args (int    argc,
             usage (argv[0]);
         }
 
+        else if (match ("--list-all")) {
+            guint   n_operations;
+            gchar **operations = gegl_list_operations (&n_operations);
+            gint    i;
+
+            for (i = 0; i < n_operations; i++)
+              {
+                fprintf (stdout, "%s\n", operations[i]);
+              }
+            g_free (operations);
+
+            exit (0);
+        }
+
+        else if (match ("--exists")) {
+            gchar   *op_name;
+
+            /* The option requires at least one argument. */
+            get_string (op_name);
+            while (op_name)
+              {
+                if (!gegl_has_operation (op_name))
+                  exit (1);
+                get_string_forced (op_name);
+              }
+
+            exit (0);
+        }
+
+        else if (match ("--properties")) {
+            gchar  *op_name;
+            get_string (op_name);
+
+            if (gegl_has_operation (op_name))
+              {
+                gint         i;
+                guint        n_properties;
+                GParamSpec **properties;
+
+                properties = gegl_operation_list_properties (op_name, &n_properties);
+                for (i = 0; i < n_properties; i++)
+                  {
+                    const gchar *property_name;
+                    const gchar *property_blurb;
+
+                    property_name = g_param_spec_get_name (properties[i]);
+                    property_blurb = g_param_spec_get_blurb (properties[i]);
+
+                    fprintf (stdout, "%-30s [%s] %s\n",
+                             property_name,
+                             g_type_name (properties[i]->value_type),
+                             property_blurb);
+                  }
+
+                g_free (properties);
+                exit (0);
+              }
+
+            exit (1);
+        }
+
         else if (match ("--verbose") ||
                  match ("-v")) {
             o->verbose=1;
@@ -242,17 +315,22 @@ parse_args (int    argc,
             o->mode = GEGL_RUN_MODE_OUTPUT;
         }
 
+        else if (match ("--scale") ||
+                 match ("-s")) {
+            get_float (o->scale);
+        }
+
         else if (match ("-X")) {
             o->mode = GEGL_RUN_MODE_XML;
         }
 
         else if (match ("--")) {
-            o->rest = curr+1;
+            o->rest = curr;
             break;
         }
 
         else if (*curr[0]=='-') {
-            fprintf (stderr, "\n\nunknown parameter '%s' giving you help instead\n\n\n", *curr);
+            fprintf (stderr, _("\n\nunknown argument '%s' giving you help instead\n\n\n"), *curr);
             usage (argv[0]);
         }
 
