@@ -1388,7 +1388,8 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
       rect->width == 1 &&
       rect->height == 1)
     {
-      gegl_buffer_get_pixel (buffer, rect->x, rect->y, format, dest_buf, repeat_mode);
+      gegl_buffer_get_pixel (buffer, rect->x, rect->y, format, dest_buf,
+                             repeat_mode);
       return;
     }
 
@@ -1418,19 +1419,18 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
     }
   else
     {
-      gint          level       = 0;
-      gint          buf_width;
-      gint          buf_height;
-      gint          bpp         = babl_format_get_bytes_per_pixel (format);
       GeglRectangle sample_rect;
-      void         *sample_buf;
-      gint          x1 = floorf (rect->x / scale + GEGL_SCALE_EPSILON);
-      gint          x2 = ceil ((rect->x + rect->width) / scale - GEGL_SCALE_EPSILON);
-      gint          y1 = floorf (rect->y / scale + GEGL_SCALE_EPSILON);
-      gint          y2 = ceil ((rect->y + rect->height) / scale - GEGL_SCALE_EPSILON);
-      gint          factor = 1;
-      gint          stride;
-      gint          offset = 0;
+      gint    level       = 0;
+      gint    buf_width;
+      gint    buf_height;
+      gint    bpp         = babl_format_get_bytes_per_pixel (format);
+      void   *sample_buf;
+      gint    x1 = floorf (rect->x / scale + GEGL_SCALE_EPSILON);
+      gint    x2 = ceil ((rect->x + rect->width) / scale - GEGL_SCALE_EPSILON);
+      gint    y1 = floorf (rect->y / scale + GEGL_SCALE_EPSILON);
+      gint    y2 = ceil ((rect->y + rect->height) / scale - GEGL_SCALE_EPSILON);
+      gint    factor = 1;
+      gint    offset = 0;
 
       while (scale <= 0.5)
         {
@@ -1443,36 +1443,38 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
           level++;
         }
 
-      buf_width  = x2 - x1;
-      buf_height = y2 - y1;
-
-      /* ensure we always have some data to sample from */
-      if (scale != 1.0 && scale <= 1.99)
-        {
-          buf_width  += 2;
-          buf_height += 2;
-          offset = (buf_width + 1) * bpp;
-        }
 
       sample_rect.x      = factor * x1;
       sample_rect.y      = factor * y1;
       sample_rect.width  = factor * (x2 - x1);
       sample_rect.height = factor * (y2 - y1);
 
-      sample_buf = scale == 1.0 ? dest_buf : g_malloc0 (buf_height * buf_width * bpp);
-      stride     = scale == 1.0 ? rowstride : buf_width * bpp;
-
-      gegl_buffer_iterate_read_dispatch (buffer, &sample_rect, (guchar*)sample_buf + offset, stride,
-                                         format, level, repeat_mode);
-
       if (scale == 1.0)
-        return;
+        {
+          gegl_buffer_iterate_read_dispatch (buffer, &sample_rect,
+                                             (guchar*)dest_buf, rowstride,
+                                             format, level, repeat_mode);
+          return;
+        }
 
       if (rowstride == GEGL_AUTO_ROWSTRIDE)
         rowstride = rect->width * bpp;
 
+      buf_width  = x2 - x1;
+      buf_height = y2 - y1;
+
       if (scale <= 1.99)
         {
+          buf_width  += 2;
+          buf_height += 2;
+          offset = (buf_width + 1) * bpp;
+          sample_buf = g_malloc0 (buf_height * buf_width * bpp);
+          
+          gegl_buffer_iterate_read_dispatch (buffer, &sample_rect,
+                                         (guchar*)sample_buf + offset,
+                                         buf_width * bpp,
+                                         format, level, repeat_mode);
+
           sample_rect.x      = x1 - 1;
           sample_rect.y      = y1 - 1;
           sample_rect.width  = x2 - x1 + 2;
@@ -1486,9 +1488,17 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
                                    scale,
                                    format,
                                    rowstride);
+          g_free (sample_buf);
         }
       else
         {
+          sample_buf = g_malloc (buf_height * buf_width * bpp);
+          
+          gegl_buffer_iterate_read_dispatch (buffer, &sample_rect,
+                                         (guchar*)sample_buf,
+                                         buf_width * bpp,
+                                         format, level, repeat_mode);
+
           sample_rect.x      = x1;
           sample_rect.y      = y1;
           sample_rect.width  = x2 - x1;
@@ -1502,8 +1512,8 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
                                  scale,
                                  bpp,
                                  rowstride);
+          g_free (sample_buf);
         }
-      g_free (sample_buf);
     }
 }
 
@@ -1549,6 +1559,7 @@ gegl_buffer_sample_cleanup (GeglBuffer *buffer)
 static void
 gegl_buffer_copy2 (GeglBuffer          *src,
                    const GeglRectangle *src_rect,
+                   GeglAbyssPolicy      repeat_mode,
                    GeglBuffer          *dst,
                    const GeglRectangle *dst_rect)
 {
@@ -1578,14 +1589,14 @@ gegl_buffer_copy2 (GeglBuffer          *src,
       dest_rect_r.height = src_rect->height;
 
       i = gegl_buffer_iterator_new (dst, &dest_rect_r, 0, dst->soft_format,
-                                    GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
+                                    GEGL_ACCESS_WRITE, repeat_mode);
       while (gegl_buffer_iterator_next (i))
         {
           GeglRectangle src_rect = i->roi[0];
           src_rect.x += offset_x;
           src_rect.y += offset_y;
           gegl_buffer_iterate_read_dispatch (src, &src_rect, i->data[0], 0,
-                                             dst->soft_format, 0, GEGL_ABYSS_NONE);
+                                             dst->soft_format, 0, repeat_mode);
         }
     }
 }
@@ -1726,31 +1737,31 @@ gegl_buffer_copy (GeglBuffer          *src,
                              GEGL_RECTANGLE (src_rect->x + (top.x-dst_rect->x),
                                              src_rect->y + (top.y-dst_rect->y),
                                  top.width, top.height),
-                             dst, &top);
+                             repeat_mode, dst, &top);
           if (bottom.height)
           gegl_buffer_copy2 (src,
                              GEGL_RECTANGLE (src_rect->x + (bottom.x-dst_rect->x),
                                              src_rect->y + (bottom.y-dst_rect->y),
                                  bottom.width, bottom.height),
-                             dst, &bottom);
+                             repeat_mode, dst, &bottom);
           if (left.width)
           gegl_buffer_copy2 (src,
                              GEGL_RECTANGLE (src_rect->x + (left.x-dst_rect->x),
                                              src_rect->y + (left.y-dst_rect->y),
                                  left.width, left.height),
-                             dst, &left);
+                             repeat_mode, dst, &left);
           if (right.width && right.height)
           gegl_buffer_copy2 (src,
                              GEGL_RECTANGLE (src_rect->x + (right.x-dst_rect->x),
                                              src_rect->y + (right.y-dst_rect->y),
                                  right.width, right.height),
-                             dst, &right);
+                             repeat_mode, dst, &right);
         }
       }
     }
   else
     {
-      gegl_buffer_copy2 (src, src_rect, dst, dst_rect);
+      gegl_buffer_copy2 (src, src_rect, repeat_mode, dst, dst_rect);
     }
 }
 
