@@ -11,12 +11,10 @@
 #include "gegl-buffer-private.h"
 #include "gegl-tile-storage.h"
 #include "gegl-tile-handler-cache.h"
-#include "gegl-utils.h"
 
-static GeglBuffer *
-gegl_buffer_linear_new2 (const GeglRectangle *extent,
-                         const Babl          *format,
-                         gint                 rowstride)
+GeglBuffer *
+gegl_buffer_linear_new (const GeglRectangle *extent,
+                        const Babl          *format)
 {
   GeglBuffer *buffer;
 
@@ -28,9 +26,6 @@ gegl_buffer_linear_new2 (const GeglRectangle *extent,
   if (format==NULL)
     format = babl_format ("RGBA float");
 
-  if (rowstride <= 0)
-    rowstride = extent->width;
-
   /* creating a linear buffer for GeglBuffer is a matter of
    * requesting the correct parameters when creating the
    * buffer
@@ -38,11 +33,11 @@ gegl_buffer_linear_new2 (const GeglRectangle *extent,
   buffer = g_object_new (GEGL_TYPE_BUFFER,
                        "x",          extent->x,
                        "y",          extent->y,
-                       "shift-x",    0,
-                       "shift-y",    0,
+                       "shift-x",    -extent->x,
+                       "shift-y",    -extent->y,
                        "width",      extent->width,
                        "height",     extent->height,
-                       "tile-width", rowstride,
+                       "tile-width", extent->width,
                        "tile-height", extent->height,
                        "format", format,
                        NULL);
@@ -52,13 +47,6 @@ gegl_buffer_linear_new2 (const GeglRectangle *extent,
   return buffer;
 }
 
-GeglBuffer *
-gegl_buffer_linear_new (const GeglRectangle *extent,
-                        const Babl          *format)
-{
-  return gegl_buffer_linear_new2 (extent, format, 0);
-}
-
 /* XXX:
  * this should probably be abstracted into a gegl_buffer_cache_insert_tile */
 void gegl_tile_handler_cache_insert (GeglTileHandlerCache *cache,
@@ -76,36 +64,58 @@ gegl_buffer_linear_new_from_data (const gpointer       data,
                                   gpointer             destroy_fn_data)
 {
   GeglBuffer *buffer;
+  GeglTile   *tile;
+  gint        bpp;
 
-  g_assert (format);
+  g_return_val_if_fail (extent, NULL);
+  g_return_val_if_fail (format, NULL);
 
-  if (rowstride <= 0) /* handle both 0 and negative coordinates as a request
-                       * for a rowstride, negative rowstrides are not supported.
-                       */
-    rowstride = extent->width;
+  bpp = babl_format_get_bytes_per_pixel (format);
+
+  if (rowstride == 0)
+    {
+      rowstride = extent->width;
+    }
   else
-    rowstride = rowstride / babl_format_get_bytes_per_pixel (format);
-    buffer = gegl_buffer_linear_new2 (extent, format, rowstride);
+    {
+      g_return_val_if_fail (rowstride > 0, NULL);
+      g_return_val_if_fail (rowstride % bpp == 0, NULL);
+      rowstride = rowstride / bpp;
+    }
 
-  {
-    GeglTile *tile = gegl_tile_new_bare ();
+  buffer = g_object_new (GEGL_TYPE_BUFFER,
+                         "x",          extent->x,
+                         "y",          extent->y,
+                         "shift-x",    -extent->x,
+                         "shift-y",    -extent->y,
+                         "width",      extent->width,
+                         "height",     extent->height,
+                         "tile-width", rowstride,
+                         "tile-height", extent->height,
+                         "format", format,
+                         "path", "RAM",
+                         NULL);
 
-    tile->tile_storage = buffer->tile_storage;
-    tile->x = 0;
-    tile->y = 0;
-    tile->z = 0;
-    tile->next_shared = tile;
-    tile->prev_shared = tile;
-    gegl_tile_set_data_full (tile,
-                             (gpointer) data,
-                             babl_format_get_bytes_per_pixel (format) * rowstride * extent->height,
-                             destroy_fn,
-                             destroy_fn_data);
+  g_object_set_data (G_OBJECT (buffer), "is-linear", (void*)0xf00);
 
-    if (buffer->tile_storage->cache)
-      gegl_tile_handler_cache_insert (buffer->tile_storage->cache, tile, 0, 0, 0);
-    gegl_tile_unref (tile);
-  }
+  tile = gegl_tile_new_bare ();
+
+  tile->tile_storage = buffer->tile_storage;
+  tile->x = 0;
+  tile->y = 0;
+  tile->z = 0;
+  tile->next_shared = tile;
+  tile->prev_shared = tile;
+  tile->rev = tile->stored_rev + 1;
+  gegl_tile_set_data_full (tile,
+                           (gpointer) data,
+                           bpp * rowstride * extent->height,
+                           destroy_fn,
+                           destroy_fn_data);
+
+  if (buffer->tile_storage->cache)
+    gegl_tile_handler_cache_insert (buffer->tile_storage->cache, tile, 0, 0, 0);
+  gegl_tile_unref (tile);
 
   return buffer;
 }
@@ -124,7 +134,7 @@ typedef struct {
 /* FIXME: make this use direct data access in more cases than the
  * case of the base buffer.
  */
-gpointer *
+gpointer
 gegl_buffer_linear_open (GeglBuffer          *buffer,
                          const GeglRectangle *extent,   /* if NULL, use buf  */
                          gint                *rowstride,/* returns rowstride */
@@ -137,7 +147,7 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
     extent=&buffer->extent;
 
   /*gegl_buffer_lock (buffer);*/
-  g_mutex_lock (&buffer->tile_storage->mutex);
+  g_rec_mutex_lock (&buffer->tile_storage->mutex);
   if (extent->x     == buffer->extent.x &&
       extent->y     == buffer->extent.y &&
       extent->width == buffer->tile_width &&
@@ -248,19 +258,19 @@ gegl_buffer_linear_close (GeglBuffer *buffer,
               linear_buffers = g_list_remove (linear_buffers, info);
               g_object_set_data (G_OBJECT (buffer), "linear-buffers", linear_buffers);
 
-              g_mutex_unlock (&buffer->tile_storage->mutex);
+              g_rec_mutex_unlock (&buffer->tile_storage->mutex);
               /* XXX: potential race */
               gegl_buffer_set (buffer, &info->extent, 0, info->format, info->buf, 0);
 
               gegl_free (info->buf);
               g_free (info);
 
-              g_mutex_lock (&buffer->tile_storage->mutex);
+              g_rec_mutex_lock (&buffer->tile_storage->mutex);
               break;
             }
         }
     }
   /*gegl_buffer_unlock (buffer);*/
-  g_mutex_unlock (&buffer->tile_storage->mutex);
+  g_rec_mutex_unlock (&buffer->tile_storage->mutex);
   return;
 }
