@@ -1,5 +1,4 @@
 /* This file is part of GEGL.
- * ck
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,12 +23,14 @@
 #include "gegl-buffer.h"
 #include "gegl-tile-handler.h"
 #include "gegl-buffer-iterator.h"
+#include "gegl-buffer-cl-iterator.h"
 
 #define GEGL_BUFFER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  GEGL_TYPE_BUFFER, GeglBufferClass))
-#define GEGL_IS_BUFFER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_BUFFER))
 #define GEGL_IS_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  GEGL_TYPE_BUFFER))
 #define GEGL_BUFFER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  GEGL_TYPE_BUFFER, GeglBufferClass))
 
+typedef struct _GeglBufferClass GeglBufferClass;
+
 struct _GeglBuffer
 {
   GeglTileHandler   parent_instance; /* which is a GeglTileHandler which has a
@@ -40,39 +41,41 @@ struct _GeglBuffer
 
   const Babl       *format;  /* the pixel format used for pixels in this
                                 buffer */
+  const Babl  *soft_format;  /* the format the buffer pretends to be, might
+                                be different from format */
 
   gint              shift_x; /* The relative offset of origins compared with */
-  gint              shift_y; /* anchestral tile_storage buffer, during            */
-                             /* construction relative to immediate source  */
+  gint              shift_y; /* anchestral tile_storage buffer, during       */
+                             /* construction relative to immediate source    */
 
   GeglRectangle     abyss;
+  gboolean          abyss_tracks_extent; /* specifies whether the abyss rectangle
+                                            should track any modifications to the
+                                            extent rectangle */
 
-  GeglTile         *hot_tile; /* cached tile for speeding up gegl_buffer_get_pixel
-                                 and gegl_buffer_set_pixel (1x1 sized gets/sets)*/
 
   GeglSampler      *sampler; /* cached sampler for speeding up random
                                 access interpolated fetches from the
                                 buffer */
+  GeglSamplerType   sampler_type;
   const Babl       *sampler_format; /* the format of the cached sampler */
 
   GeglTileStorage  *tile_storage;
 
-  gint              min_x; /* the extent of tile indices that has been */
-  gint              min_y; /* produced by _get_tile for this buffer */
-  gint              max_x; /* this is used in gegl_buffer_void to narrow */
-  gint              max_y; /* down the tiles kill commands are sent for */
-  gint              max_z;
-
   gint              tile_width;
   gint              tile_height;
   gchar            *path;
 
   gint              lock_count;
 
-  gchar            *alloc_stack_trace; /* Stack trace for allocation,
+  gpointer         *alloc_stack_trace; /* Stack trace for allocation,
                                           useful for debugging */
+  gint              alloc_stack_size;
 
-  gpointer          backend;
+  gint              changed_signal_connections; /* to avoid firing changed signals
+                                                   with no listeners */
+
+  GeglTileBackend  *backend;
 };
 
 struct _GeglBufferClass
@@ -81,55 +84,62 @@ struct _GeglBufferClass
 };
 
 
-
 gint              gegl_buffer_leaks       (void);
 
 void              gegl_buffer_stats       (void);
 
-void              gegl_buffer_save        (GeglBuffer          *buffer,
-                                           const gchar         *path,
-                                           const GeglRectangle *roi);
-
-
 const gchar      *gegl_swap_dir           (void);
 
-
 void              gegl_tile_cache_init    (void);
 
 void              gegl_tile_cache_destroy (void);
 
+void              gegl_tile_backend_swap_cleanup (void);
+
 GeglTileBackend * gegl_buffer_backend     (GeglBuffer *buffer);
+GeglTileBackend * gegl_buffer_backend2    (GeglBuffer *buffer); /* non-cached */
 
 gboolean          gegl_buffer_is_shared   (GeglBuffer *buffer);
 
+#define GEGL_BUFFER_DISABLE_LOCKS 1
+
+#ifdef GEGL_BUFFER_DISABLE_LOCKS
+#define           gegl_buffer_try_lock(a)   (TRUE)
+#define           gegl_buffer_lock(a)       do{}while(0)
+#define           gegl_buffer_unlock(a)       do{}while(0)
+#else
 gboolean          gegl_buffer_try_lock    (GeglBuffer *buffer);
 gboolean          gegl_buffer_lock        (GeglBuffer *buffer);
 gboolean          gegl_buffer_unlock      (GeglBuffer *buffer);
+#endif
+
 void              gegl_buffer_set_unlocked (GeglBuffer          *buffer,
                                             const GeglRectangle *rect,
+                                            gint                 level,
                                             const Babl          *format,
-                                            void                *src,
+                                            const void          *src,
                                             gint                 rowstride);
+
+void              gegl_buffer_set_unlocked_no_notify (GeglBuffer          *buffer,
+                                                      const GeglRectangle *rect,
+                                                      gint                 level,
+                                                      const Babl          *format,
+                                                      const void          *src,
+                                                      gint                 rowstride);
+
 void              gegl_buffer_get_unlocked (GeglBuffer          *buffer,
                                             gdouble              scale,
                                             const GeglRectangle *rect,
                                             const Babl          *format,
                                             gpointer             dest_buf,
-                                            gint                 rowstride);
-
-GeglBuffer *
-gegl_buffer_new_ram (const GeglRectangle *extent,
-                     const Babl          *format);
-
-void            gegl_buffer_sampler           (GeglBuffer     *buffer,
-                                               gdouble         x,
-                                               gdouble         y,
-                                               gdouble         scale,
-                                               gpointer        dest,
-                                               const Babl     *format,
-                                               gpointer        sampler);
+                                            gint                 rowstride,
+                                            GeglAbyssPolicy      repeat_mode);
 
+GeglBuffer *      gegl_buffer_new_ram     (const GeglRectangle *extent,
+                                           const Babl          *format);
 
+void              gegl_buffer_emit_changed_signal (GeglBuffer *buffer,
+                                                   const GeglRectangle *rect);
 
 /* the instance size of a GeglTile is a bit large, and should if possible be
  * trimmed down
@@ -154,18 +164,18 @@ struct _GeglTile
   guint            stored_rev;  /* what revision was we when we from tile_storage?
                                    (currently set to 1 when loaded from disk */
 
-  gchar            lock;        /* number of times the tile is write locked
+  gint             lock;        /* number of times the tile is write locked
                                  * should in theory just have the values 0/1
                                  */
-  GMutex          *mutex;
+  gint             is_zero_tile:1;
 
   /* the shared list is a doubly linked circular list */
   GeglTile        *next_shared;
   GeglTile        *prev_shared;
 
   /* called when the tile is about to be destroyed */
-  GeglDestroyNotify destroy_notify;
-  gpointer          destroy_notify_data;
+  GDestroyNotify   destroy_notify;
+  gpointer         destroy_notify_data;
 
   /* called when the tile has been unlocked which typically means tile
    * data has changed
@@ -174,10 +184,22 @@ struct _GeglTile
   gpointer         unlock_notify_data;
 };
 
-#ifndef __GEGL_TILE_C
-#define gegl_tile_get_data(tile)  ((guchar*)((tile)->data))
-#endif // __GEGL_TILE_C
+void _gegl_buffer_drop_hot_tile (GeglBuffer *buffer);
 
+GeglRectangle _gegl_get_required_for_scale (const Babl          *format,
+                                            const GeglRectangle *roi,
+                                            gdouble              scale);
+
+gboolean gegl_buffer_scan_compatible (GeglBuffer *bufferA,
+                                      gint        xA,
+                                      gint        yA,
+                                      GeglBuffer *bufferB,
+                                      gint        xB,
+                                      gint        yB);
+
+#ifndef __GEGL_TILE_H__
+#define gegl_tile_get_data(tile)  ((tile)->data)
+#endif
 
 /* computes the positive integer remainder (also for negative dividends)
  */
