@@ -14,46 +14,15 @@
  * License along with GEGL; if not, see
  * <http://www.gnu.org/licenses/>.
  *
- */
-
-
-/*
- * FAST_PSEUDO_FLOOR is a floor and floorf replacement which has been
- * found to be faster on several linux boxes than the library
- * version. It returns the floor of its argument unless the argument
- * is a negative integer, in which case it returns one less than the
- * floor. For example:
- *
- * FAST_PSEUDO_FLOOR(0.5) = 0
- *
- * FAST_PSEUDO_FLOOR(0.f) = 0
- *
- * FAST_PSEUDO_FLOOR(-.5) = -1
- *
- * as expected, but
- *
- * FAST_PSEUDO_FLOOR(-1.f) = -2
- *
- * The locations of the discontinuities of FAST_PSEUDO_FLOOR are the
- * same as floor and floorf; it is just that at negative integers the
- * function is discontinuous on the right instead of the left.
- */
-#define FAST_PSEUDO_FLOOR(x) ( (int)(x) - ( (x) < 0. ) )
-/*
- * Alternative (if conditional move is fast and correctly identified
- * by the compiler):
- *
- * #define FAST_PSEUDO_FLOOR(x) ( (x)>=0 ? (int)(x) : (int)(x)-1 )
+ * Copyright 2011-2012 Nicolas Robidoux based on earlier code
+ *           2012      Massimo Valentini
  */
 
 #include "config.h"
-#include <glib-object.h>
-#include <glib/gstdio.h>
-#include <glib/gprintf.h>
+#include <math.h>
 
 #include "gegl.h"
 #include "gegl-types-internal.h"
-#include "gegl-buffer-private.h"
 #include "gegl-sampler-linear.h"
 
 enum
@@ -62,20 +31,12 @@ enum
   PROP_LAST
 };
 
-static void gegl_sampler_linear_get (GeglSampler* restrict self,
-                                     const gdouble         x,
-                                     const gdouble         y,
-                                     void*        restrict output);
-
-static void set_property (GObject*      gobject,
-                          guint         property_id,
-                          const GValue* value,
-                          GParamSpec*   pspec);
-
-static void get_property (GObject*    gobject,
-                          guint       property_id,
-                          GValue*     value,
-                          GParamSpec* pspec);
+static void gegl_sampler_linear_get (      GeglSampler* restrict  self,
+                                     const gdouble                absolute_x,
+                                     const gdouble                absolute_y,
+                                           GeglMatrix2           *scale,
+                                           void*        restrict  output,
+                                           GeglAbyssPolicy        repeat_mode);
 
 G_DEFINE_TYPE (GeglSamplerLinear, gegl_sampler_linear, GEGL_TYPE_SAMPLER)
 
@@ -83,56 +44,75 @@ static void
 gegl_sampler_linear_class_init (GeglSamplerLinearClass *klass)
 {
   GeglSamplerClass *sampler_class = GEGL_SAMPLER_CLASS (klass);
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-  object_class->set_property = set_property;
-  object_class->get_property = get_property;
 
   sampler_class->get = gegl_sampler_linear_get;
 }
 
+/*
+ * In principle, x=y=0 and width=height=2 are enough. The following
+ * values are chosen so as to make the context_rect symmetrical
+ * w.r.t. the anchor point. This is so that enough elbow room is added
+ * with transformations that reflect the context rect. If the
+ * context_rect is not symmetrical, the transformation may turn right
+ * into left, and if the context_rect does not stretch far enough on
+ * the left, pixel lookups will fail.
+ *
+ * Additional extra elbow room is added all around. It could be set to
+ * 0 if it is found that round off error never sends things "too far
+ * away". Nicolas would be very surprised if more than 1 is necessary.
+ */
+#define LINEAR_EXTRA_ELBOW_ROOM 0
+
 static void
 gegl_sampler_linear_init (GeglSamplerLinear *self)
 {
-  GEGL_SAMPLER (self)->context_rect.x = 0;
-  GEGL_SAMPLER (self)->context_rect.y = 0;
-  GEGL_SAMPLER (self)->context_rect.width = 2;
-  GEGL_SAMPLER (self)->context_rect.height = 2;
+  GEGL_SAMPLER (self)->level[0].context_rect.x      = -1 -   LINEAR_EXTRA_ELBOW_ROOM;
+  GEGL_SAMPLER (self)->level[0].context_rect.y      = -1 -   LINEAR_EXTRA_ELBOW_ROOM;
+  GEGL_SAMPLER (self)->level[0].context_rect.width  =  3 + 2*LINEAR_EXTRA_ELBOW_ROOM;
+  GEGL_SAMPLER (self)->level[0].context_rect.height =  3 + 2*LINEAR_EXTRA_ELBOW_ROOM;
   GEGL_SAMPLER (self)->interpolate_format = babl_format ("RaGaBaA float");
 }
 
 static void
-gegl_sampler_linear_get (GeglSampler* restrict self,
-                         const gdouble         absolute_x,
-                         const gdouble         absolute_y,
-                         void*        restrict output)
+gegl_sampler_linear_get (      GeglSampler*    restrict  self,
+                         const gdouble                   absolute_x,
+                         const gdouble                   absolute_y,
+                               GeglMatrix2              *scale,
+                               void*           restrict  output,
+                               GeglAbyssPolicy           repeat_mode)
 {
-  const gint pixels_per_buffer_row = 64;
+  const gint pixels_per_buffer_row = GEGL_SAMPLER_MAXIMUM_WIDTH;
   const gint channels = 4;
 
   /*
-   * floor's surrogate FAST_PSEUDO_FLOOR is used to make
-   * sure that the transition through 0 is smooth. If it is known that
-   * negative absolute_x and absolute_y will never be used, plain
-   * cast---that is, const gint ix = absolute_x---is recommended
-   * instead.
+   * The "-1/2"s are there because we want the index of the pixel to
+   * the left and top of the location, and with GIMP's convention the
+   * top left of the top left pixel is located at
+   * (1/2,1/2). Basically, we are converting from a coordinate system
+   * in which the origin is at the top left pixel of the pixel with
+   * index (0,0), to a coordinate system in which the origin is at the
+   * center of the same pixel.
    */
-  const gint ix = FAST_PSEUDO_FLOOR (absolute_x);
-  const gint iy = FAST_PSEUDO_FLOOR (absolute_y);
+  const float iabsolute_x = (float) absolute_x - 0.5;
+  const float iabsolute_y = (float) absolute_y - 0.5;
 
-  /*
-   * x is the x-coordinate of the sampling point relative to the
-   * position of the top left pixel center. Similarly for y. Range of
-   * values: [0,1].
-   */
-  const gfloat x = absolute_x - ix;
-  const gfloat y = absolute_y - iy;
+  const gint ix = floorf (iabsolute_x);
+  const gint iy = floorf (iabsolute_y);
 
   /*
    * Point the data tile pointer to the first channel of the top_left
    * pixel value:
    */
-  const gfloat* restrict in_bptr = gegl_sampler_get_ptr (self, ix, iy);
+  const gfloat* restrict in_bptr =
+    gegl_sampler_get_ptr (self, ix, iy, repeat_mode);
+
+  /*
+   * x is the x-coordinate of the sampling point relative to the
+   * position of the center of the top left pixel. Similarly for
+   * y. Range of values: [0,1].
+   */
+  const gfloat x = iabsolute_x - ix;
+  const gfloat y = iabsolute_y - iy;
 
   /*
    * First bilinear weight:
@@ -177,7 +157,7 @@ gegl_sampler_linear_get (GeglSampler* restrict self,
   /*
    * Last bilinear weight:
    */
-  const gfloat w_times_z = 1.f - ( x + w_times_y );
+  const gfloat w_times_z = (gfloat) 1. - ( x + w_times_y );
 
   gfloat newval[4];
 
@@ -220,22 +200,3 @@ gegl_sampler_linear_get (GeglSampler* restrict self,
     babl_process (self->fish, newval, output, 1);
   }
 }
-
-static void
-set_property (GObject*      gobject,
-              guint         property_id,
-              const GValue* value,
-              GParamSpec*   pspec)
-{
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
-}
-
-static void
-get_property (GObject*    gobject,
-              guint       property_id,
-              GValue*     value,
-              GParamSpec* pspec)
-{
-  G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
-}
-
