@@ -113,20 +113,6 @@ gegl_tile_handler_cache_init (GeglTileHandlerCache *cache)
   gegl_tile_cache_init ();
 }
 
-
-static void
-gegl_tile_handler_cache_reinit_buffer_tiles (gpointer itm,
-                                             gpointer userdata)
-{
-  CacheItem *item;
-  item = itm;
-  if (item->handler == userdata)
-    {
-      GeglTileHandlerCache *cache = userdata;
-      cache->free_list = g_slist_prepend (cache->free_list, item);
-    }
-}
-
 static void
 gegl_tile_handler_cache_reinit (GeglTileHandlerCache *cache)
 {
@@ -143,85 +129,39 @@ gegl_tile_handler_cache_reinit (GeglTileHandlerCache *cache)
     return;
 
   g_mutex_lock (&mutex);
-  /* only throw out items belonging to this cache instance */
-
-  cache->free_list = NULL;
-  g_queue_foreach (cache_queue, gegl_tile_handler_cache_reinit_buffer_tiles, cache);
-  for (iter = cache->free_list; iter; iter = g_slist_next (iter))
+  {
+    for (iter = cache->items; iter; iter = cache->items)
     {
       item = iter->data;
       if (item->tile)
         {
           cache_total -= item->tile->size;
-          gegl_tile_mark_as_stored (item->tile); /* to avoid saving */
+          gegl_tile_mark_as_stored (item->tile); // to avoid saving 
           gegl_tile_unref (item->tile);
           cache->count--;
         }
       g_queue_unlink (cache_queue, &item->link);
+      cache->items = g_slist_remove (cache->items, item);
       g_hash_table_remove (cache_ht, item);
       g_slice_free (CacheItem, item);
     }
-  g_slist_free (cache->free_list);
-  cache->free_list = NULL;
-  g_mutex_unlock (&mutex);
-}
+  }
 
-static void
-gegl_tile_handler_cache_dispose_buffer_tiles (gpointer itm,
-                                              gpointer userdata)
-{
-  CacheItem *item;
-  item = itm;
-  if (item->handler == userdata)
-    {
-      GeglTileHandlerCache *cache = userdata;
-      cache->free_list = g_slist_prepend (cache->free_list, item);
-    }
+  g_mutex_unlock (&mutex);
 }
 
 static void
 gegl_tile_handler_cache_dispose (GObject *object)
 {
-  GeglTileHandlerCache *cache;
-  CacheItem            *item;
-  GSList               *iter;
+  GeglTileHandlerCache *cache = GEGL_TILE_HANDLER_CACHE (object);
 
-  cache = (GeglTileHandlerCache*) (object);
-
-  /* only throw out items belonging to this cache instance */
-
-  cache->free_list = NULL;
-  /* XXX: for optimization this could be delayed,. or collected among multiple
-   * buffer destructions, to avoid the overhead of walking the full queue for
-   * every tiny buffer being destroyed.
-   */
-
-  if (cache->count)
-    {
-      g_mutex_lock (&mutex);
-      g_queue_foreach (cache_queue, gegl_tile_handler_cache_dispose_buffer_tiles, cache);
-      for (iter = cache->free_list; iter; iter = g_slist_next (iter))
-        {
-            item = iter->data;
-            if (item->tile)
-              {
-                cache_total -= item->tile->size;
-                gegl_tile_unref (item->tile);
-                cache->count--;
-              }
-            g_queue_unlink (cache_queue, &item->link);
-            g_hash_table_remove (cache_ht, item);
-            g_slice_free (CacheItem, item);
-        }
-      g_slist_free (cache->free_list);
-      cache->free_list = NULL;
-      g_mutex_unlock (&mutex);
-    }
+  gegl_tile_handler_cache_reinit (cache);
 
   if (cache->count < 0)
     {
       g_warning ("cache-handler tile balance not zero: %i\n", cache->count);
     }
+
   G_OBJECT_CLASS (gegl_tile_handler_cache_parent_class)->dispose (object);
 }
 
@@ -403,6 +343,12 @@ gegl_tile_handler_cache_get_tile (GeglTileHandlerCache *cache,
       g_queue_unlink (cache_queue, &result->link);
       g_queue_push_head_link (cache_queue, &result->link);
       g_mutex_unlock (&mutex);
+      while (result->tile == NULL)
+      {
+        g_printerr ("NULL tile in %s %p %i %i %i %p\n", __FUNCTION__, result, result->x, result->y, result->z,
+                result->tile);
+        return NULL;
+      }
       return gegl_tile_ref (result->tile);
     }
   g_mutex_unlock (&mutex);
@@ -436,10 +382,20 @@ gegl_tile_handler_cache_trim (GeglTileHandlerCache *cache)
   if (link != NULL)
     {
       CacheItem *last_writable = LINK_GET_ITEM (link);
+      GeglTile *tile = last_writable->tile;
+      GeglTileStorage *storage = tile->tile_storage;
 
+      last_writable->handler->items = g_slist_remove (last_writable->handler->items, last_writable);
       g_hash_table_remove (cache_ht, last_writable);
-      cache_total  -= last_writable->tile->size;
-      gegl_tile_unref (last_writable->tile);
+      cache_total -= tile->size;
+
+      if (storage && storage->hot_tile == tile)
+        {
+          storage->hot_tile = NULL;
+          gegl_tile_unref (tile);
+        }
+
+      gegl_tile_unref (tile);
       g_slice_free (CacheItem, last_writable);
       return TRUE;
     }
@@ -463,7 +419,10 @@ gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
       item->tile->tile_storage = NULL;
       gegl_tile_mark_as_stored (item->tile); /* to cheat it out of being stored */
       gegl_tile_unref (item->tile);
+
       g_queue_unlink (cache_queue, &item->link);
+      cache->items = g_slist_remove (cache->items, item);
+
       g_hash_table_remove (cache_ht, item);
       g_slice_free (CacheItem, item);
     }
@@ -483,15 +442,21 @@ gegl_tile_handler_cache_void (GeglTileHandlerCache *cache,
   item = cache_lookup (cache, x, y, z);
   if (item)
     {
-      gegl_tile_void (item->tile);
       cache_total -= item->tile->size;
-      gegl_tile_unref (item->tile);
       g_queue_unlink (cache_queue, &item->link);
+      cache->items = g_slist_remove (cache->items, item);
       g_hash_table_remove (cache_ht, item);
-      g_slice_free (CacheItem, item);
-      cache->count --;
+      cache->count--;
     }
   g_mutex_unlock (&mutex);
+
+  if (item)
+    {
+      gegl_tile_void (item->tile);
+      gegl_tile_unref (item->tile);
+    }
+
+  g_slice_free (CacheItem, item);
 }
 
 void
@@ -512,16 +477,23 @@ gegl_tile_handler_cache_insert (GeglTileHandlerCache *cache,
   item->y         = y;
   item->z         = z;
 
+  tile->x = x;
+  tile->y = y;
+  tile->z = z;
+  tile->tile_storage = cache->tile_storage;
+
   // XXX : remove entry if it already exists
   gegl_tile_handler_cache_void (cache, x, y, z);
 
+  /* XXX: this is a window when the tile is a zero tile during update */
+
   g_mutex_lock (&mutex);
   cache_total  += item->tile->size;
   g_queue_push_head_link (cache_queue, &item->link);
 
   cache->count ++;
 
-
+  cache->items = g_slist_prepend (cache->items, item);
   g_hash_table_insert (cache_ht, item, item);
 
   while (cache_total > gegl_config()->tile_cache_size)
