@@ -24,22 +24,20 @@
 #include <unistd.h>
 #endif
 
-#define __GEGL_TILE_C
-
 #include <glib-object.h>
 
 #include "gegl-types-internal.h"
 
 #include "gegl.h"
 #include "gegl-buffer.h"
-#include "gegl-buffer-private.h"
 #include "gegl-tile.h"
+#include "gegl-buffer-private.h"
 #include "gegl-tile-source.h"
 #include "gegl-tile-storage.h"
 
-#include "gegl-utils.h"
-
-static GStaticMutex cowmutex = G_STATIC_MUTEX_INIT;
+static GMutex cowmutex = { 0, }; /* copy on write is maintained in a doubly linked
+                                  * list, which must be protected by a mutex
+                                  */
 
 GeglTile *gegl_tile_ref (GeglTile *tile)
 {
@@ -58,13 +56,14 @@ void gegl_tile_unref (GeglTile *tile)
    * the in-memory tile is written to disk before we free the memory,
    * otherwise this data will be lost.
    */
-  if (!gegl_tile_is_stored (tile))
-    gegl_tile_store (tile);
+  gegl_tile_store (tile);
 
+  g_mutex_lock (&cowmutex);
   if (tile->data)
     {
       if (tile->next_shared == tile)
         { /* no clones */
+          g_mutex_unlock (&cowmutex);
           if (tile->destroy_notify)
             {
               if (tile->destroy_notify == (void*)&free_data_directly)
@@ -78,16 +77,12 @@ void gegl_tile_unref (GeglTile *tile)
         {
           tile->prev_shared->next_shared = tile->next_shared;
           tile->next_shared->prev_shared = tile->prev_shared;
+          g_mutex_unlock (&cowmutex);
         }
     }
+  else
+    g_mutex_unlock (&cowmutex);
 
-#ifdef GEGL_USE_TILE_MUTEX
-  if (tile->mutex)
-    {
-      g_mutex_free (tile->mutex);
-      tile->mutex = NULL;
-    }
-#endif
   g_slice_free (GeglTile, tile);
 }
 
@@ -95,13 +90,13 @@ void gegl_tile_unref (GeglTile *tile)
 GeglTile *
 gegl_tile_new_bare (void)
 {
-  GeglTile *tile = g_slice_new0 (GeglTile);
-  tile->ref_count = 1;
+  GeglTile *tile     = g_slice_new0 (GeglTile);
+  tile->ref_count    = 1;
   tile->tile_storage = NULL;
-  tile->stored_rev = 1;
-  tile->rev        = 1;
-  tile->lock       = 0;
-  tile->data       = NULL;
+  tile->stored_rev   = 1;
+  tile->rev          = 1;
+  tile->lock         = 0;
+  tile->data         = NULL;
 
   tile->next_shared = tile;
   tile->prev_shared = tile;
@@ -109,10 +104,6 @@ gegl_tile_new_bare (void)
   tile->destroy_notify = (void*)&free_data_directly;
   tile->destroy_notify_data = NULL;
 
-#ifdef GEGL_USE_TILE_MUTEX
-  tile->mutex = g_mutex_new ();
-#endif
-
   return tile;
 }
 
@@ -121,9 +112,12 @@ gegl_tile_dup (GeglTile *src)
 {
   GeglTile *tile = gegl_tile_new_bare ();
 
-  tile->tile_storage    = src->tile_storage;
-  tile->data       = src->data;
-  tile->size       = src->size;
+  g_mutex_lock (&cowmutex);
+
+  tile->tile_storage = src->tile_storage;
+  tile->data         = src->data;
+  tile->size         = src->size;
+  tile->is_zero_tile = src->is_zero_tile;
 
   tile->destroy_notify      = src->destroy_notify;
   tile->destroy_notify_data = src->destroy_notify_data;
@@ -131,29 +125,9 @@ gegl_tile_dup (GeglTile *src)
   tile->next_shared              = src->next_shared;
   src->next_shared               = tile;
   tile->prev_shared              = src;
-#ifdef GEGL_USE_TILE_MUTEX
-  if (tile->next_shared != src)
-    {
-      g_mutex_lock (tile->next_shared->mutex);
-    }
-#endif
-  if (tile->next_shared != src)
-    {
-      g_static_mutex_lock (&cowmutex);
-    }
-
   tile->next_shared->prev_shared = tile;
 
-  if (tile->next_shared != src)
-    {
-      g_static_mutex_unlock (&cowmutex);
-    }
-#ifdef GEGL_USE_TILE_MUTEX
-  if (tile->next_shared != src)
-    {
-      g_mutex_unlock (tile->next_shared->mutex);
-    }
-#endif
+  g_mutex_unlock (&cowmutex);
 
   return tile;
 }
@@ -181,47 +155,48 @@ gegl_memdup (gpointer src, gsize size)
 static void
 gegl_tile_unclone (GeglTile *tile)
 {
+  g_mutex_lock (&cowmutex);
   if (tile->next_shared != tile)
     {
-      /* the tile data is shared with other tiles,
-       * create a local copy
-       */
-      g_static_mutex_lock (&cowmutex);
-      tile->data                     = gegl_memdup (tile->data, tile->size);
-      tile->destroy_notify           = (void*)&free_data_directly;
-      tile->destroy_notify_data      = NULL;
       tile->prev_shared->next_shared = tile->next_shared;
       tile->next_shared->prev_shared = tile->prev_shared;
       tile->prev_shared              = tile;
       tile->next_shared              = tile;
-      g_static_mutex_unlock (&cowmutex);
+      g_mutex_unlock (&cowmutex);
+
+      /* the tile data is shared with other tiles,
+       * create a local copy
+       */
+      if (tile->is_zero_tile)
+        {
+          tile->data = gegl_calloc (tile->size, 1);
+          tile->is_zero_tile = 0;
+        }
+      else
+        {
+          tile->data = gegl_memdup (tile->data, tile->size);
+        }
+      tile->destroy_notify           = (void*)&free_data_directly;
+      tile->destroy_notify_data      = NULL;
+    }
+  else
+    {
+      g_mutex_unlock (&cowmutex);
     }
 }
-#if 0
-static gint total_locks   = 0;
-static gint total_unlocks = 0;
-#endif
-
-void gegl_bt (void);
 
 void
 gegl_tile_lock (GeglTile *tile)
 {
-#ifdef GEGL_USE_TILE_MUTEX
-  g_mutex_lock (tile->mutex);
-#endif
-
-  if (tile->lock != 0)
+  int slept = 0;
+  while (! (g_atomic_int_compare_and_exchange (&tile->lock, 0, 1)))
+  {
+    if (slept++ == 1000)
     {
-      g_warning ("strange tile lock count: %i", tile->lock);
-      gegl_bt ();
+      g_warning ("blocking when trying to lock tile");
     }
-#if 0
-  total_locks++;
-#endif
-
-  tile->lock++;
-  /*fprintf (stderr, "global tile locking: %i %i\n", locks, unlocks);*/
+    g_usleep (5);
+  }
 
   gegl_tile_unclone (tile);
 }
@@ -256,10 +231,6 @@ gegl_tile_void_pyramid (GeglTile *tile)
 void
 gegl_tile_unlock (GeglTile *tile)
 {
-#if 0
-  total_unlocks++;
-#endif
-
   if (tile->unlock_notify != NULL)
     {
       tile->unlock_notify (tile, tile->unlock_notify_data);
@@ -269,19 +240,17 @@ gegl_tile_unlock (GeglTile *tile)
     {
       g_warning ("unlocked a tile with lock count == 0");
     }
-  tile->lock--;
-  if (tile->lock == 0 &&
-      tile->z == 0)
-    {
-      gegl_tile_void_pyramid (tile);
-    }
-  if (tile->lock==0)
-    tile->rev++;
-#ifdef GEGL_USE_TILE_MUTEX
-  g_mutex_unlock (tile->mutex);
-#endif
-}
+  else if (tile->lock==1)
+  {
+    if (tile->z == 0)
+      {
+        gegl_tile_void_pyramid (tile);
+      }
+      tile->rev++;
+  }
 
+  g_atomic_int_add (&tile->lock, -1);
+}
 
 void
 gegl_tile_mark_as_stored (GeglTile *tile)
@@ -298,27 +267,36 @@ gegl_tile_is_stored (GeglTile *tile)
 void
 gegl_tile_void (GeglTile *tile)
 {
-  tile->stored_rev = tile->rev;
-  tile->tile_storage = NULL;
-  if (tile->z==0)
+  g_rec_mutex_lock (&tile->tile_storage->mutex);
+  gegl_tile_mark_as_stored (tile);
+
+  if (tile->z == 0)
     gegl_tile_void_pyramid (tile);
+  g_rec_mutex_unlock (&tile->tile_storage->mutex);
 }
 
 gboolean gegl_tile_store (GeglTile *tile)
 {
+  gboolean ret;
   if (gegl_tile_is_stored (tile))
     return TRUE;
   if (tile->tile_storage == NULL)
     return FALSE;
-  return gegl_tile_source_set_tile (GEGL_TILE_SOURCE (tile->tile_storage),
+  g_rec_mutex_lock (&tile->tile_storage->mutex);
+  if (gegl_tile_is_stored (tile))
+  {
+    g_rec_mutex_unlock (&tile->tile_storage->mutex);
+    return FALSE;
+  }
+  ret = gegl_tile_source_set_tile (GEGL_TILE_SOURCE (tile->tile_storage),
                                     tile->x,
                                     tile->y,
                                     tile->z,
                                     tile);
+  g_rec_mutex_unlock (&tile->tile_storage->mutex);
+  return ret;
 }
 
-/* for internal use, a macro poking directly at the data will be faste
- */
 guchar *gegl_tile_get_data (GeglTile *tile)
 {
   return tile->data;
@@ -344,7 +322,6 @@ void gegl_tile_set_data_full (GeglTile      *tile,
   tile->destroy_notify_data = destroy_notify_data;
 }
 
-
 void         gegl_tile_set_rev        (GeglTile *tile,
                                        guint     rev)
 {
