@@ -346,9 +346,10 @@ gegl_module_register (GTypeModule *module)
 #define gegl_chant_multiline(name, nick, def, blurb)
 #define gegl_chant_object(name,nick,  blurb)
 #define gegl_chant_pointer(name, nick, blurb)
+#define gegl_chant_format(name, nick, blurb)
 #define gegl_chant_color(name, nick, def, blurb)
 #define gegl_chant_curve(name, nick, blurb)
-#define gegl_chant_seed(name, nick, blurb)
+#define gegl_chant_seed(name, rand_name, nick, blurb)
 #define gegl_chant_path(name, nick, blurb)
 #define gegl_chant_register_enum(enum_name) \
 typedef enum {
@@ -398,6 +399,7 @@ GType enum_name ## _get_type (void)               \
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -429,9 +431,11 @@ struct _GeglChantO
 #define gegl_chant_multiline(name, nick, def, blurb)                            gchar             *name;
 #define gegl_chant_object(name,nick,  blurb)                                    GObject           *name;
 #define gegl_chant_pointer(name, nick, blurb)                                   gpointer           name;
+#define gegl_chant_format(name, nick, blurb)                                    gpointer           name;
 #define gegl_chant_color(name, nick, def, blurb)                                GeglColor         *name;
 #define gegl_chant_curve(name, nick, blurb)                                     GeglCurve         *name;
-#define gegl_chant_seed(name, nick, blurb)                                      gint               name;
+#define gegl_chant_seed(name, rand_name, nick, blurb)                           gint               name;\
+                                                                                GeglRandom        *rand_name;
 #define gegl_chant_path(name, nick, blurb)                                      GeglPath          *name;\
                                                                                 gulong path_changed_handler;
 
@@ -448,6 +452,7 @@ struct _GeglChantO
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -470,9 +475,10 @@ enum
 #define gegl_chant_multiline(name, nick, def, blurb)                            PROP_##name,
 #define gegl_chant_object(name, nick, blurb)                                    PROP_##name,
 #define gegl_chant_pointer(name, nick, blurb)                                   PROP_##name,
+#define gegl_chant_format(name, nick, blurb)                                    PROP_##name,
 #define gegl_chant_color(name, nick, def, blurb)                                PROP_##name,
 #define gegl_chant_curve(name, nick, blurb)                                     PROP_##name,
-#define gegl_chant_seed(name, nick, blurb)                                      PROP_##name,
+#define gegl_chant_seed(name, rand_name, nick, blurb)                           PROP_##name,
 #define gegl_chant_path(name, nick, blurb)                                      PROP_##name,
 
 #include GEGL_CHANT_C_FILE
@@ -488,6 +494,7 @@ enum
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -551,6 +558,10 @@ get_property (GObject      *gobject,
     case PROP_##name:                                         \
       g_value_set_pointer (value, properties->name);          \
       break;
+#define gegl_chant_format(name, nick, blurb)                 \
+    case PROP_##name:                                         \
+      g_value_set_pointer (value, properties->name);          \
+      break;
 #define gegl_chant_color(name, nick, def, blurb)              \
     case PROP_##name:                                         \
       g_value_set_object (value, properties->name);           \
@@ -559,7 +570,7 @@ get_property (GObject      *gobject,
     case PROP_##name:                                         \
       g_value_set_object (value, properties->name);           \
       break;
-#define gegl_chant_seed(name, nick, blurb)                    \
+#define gegl_chant_seed(name, rand_name, nick, blurb)         \
     case PROP_##name:                                         \
       g_value_set_int (value, properties->name);              \
       break;
@@ -581,6 +592,7 @@ get_property (GObject      *gobject,
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -628,9 +640,9 @@ set_property (GObject      *gobject,
     case PROP_##name:                                                 \
       if (properties->name)                                           \
         g_free (properties->name);                                    \
-      properties->name = g_strdup (g_value_get_string (value));       \
+      properties->name = g_value_dup_string (value);                  \
       break;
-#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)          \
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)      \
     case PROP_##name:                                                 \
       properties->name = g_value_get_enum (value);                    \
       break;
@@ -638,13 +650,13 @@ set_property (GObject      *gobject,
     case PROP_##name:                                                 \
       if (properties->name)                                           \
         g_free (properties->name);                                    \
-      properties->name = g_strdup (g_value_get_string (value));       \
+      properties->name = g_value_dup_string (value);                  \
       break;
 #define gegl_chant_multiline(name, nick, def, blurb)                  \
     case PROP_##name:                                                 \
       if (properties->name)                                           \
         g_free (properties->name);                                    \
-      properties->name = g_strdup (g_value_get_string (value));       \
+      properties->name = g_value_dup_string (value);                  \
       break;
 #define gegl_chant_object(name, nick, blurb)                          \
     case PROP_##name:                                                 \
@@ -656,6 +668,10 @@ set_property (GObject      *gobject,
     case PROP_##name:                                                 \
       properties->name = g_value_get_pointer (value);                 \
       break;
+#define gegl_chant_format(name, nick, blurb)                          \
+    case PROP_##name:                                                 \
+      properties->name = g_value_get_pointer (value);                 \
+      break;
 #define gegl_chant_color(name, nick, def, blurb)                      \
     case PROP_##name:                                                 \
       if (properties->name != NULL)                                   \
@@ -668,10 +684,14 @@ set_property (GObject      *gobject,
          g_object_unref (properties->name);                           \
       properties->name = g_value_dup_object (value);                  \
       break;
-#define gegl_chant_seed(name, nick, blurb)                            \
+#define gegl_chant_seed(name, rand_name, nick, blurb)                 \
     case PROP_##name:                                                 \
       properties->name = g_value_get_int (value);                     \
-    break;
+      if (!properties->rand_name)                                     \
+        properties->rand_name = gegl_random_new_with_seed (properties->name);\
+      else                                                            \
+        gegl_random_set_seed (properties->rand_name, properties->name);\
+      break;
 #define gegl_chant_path(name, nick, blurb)                            \
     case PROP_##name:                                                 \
       if (properties->name != NULL)                                   \
@@ -689,7 +709,7 @@ set_property (GObject      *gobject,
             g_signal_connect (G_OBJECT (properties->name), "changed", \
                               G_CALLBACK(path_changed), gobject);     \
          }                                                            \
-      break; /*XXX*/
+      break; 
 
 #include GEGL_CHANT_C_FILE
 
@@ -704,6 +724,7 @@ set_property (GObject      *gobject,
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -752,6 +773,7 @@ static void gegl_chant_destroy_notify (gpointer data)
       properties->name = NULL;                      \
     }
 #define gegl_chant_pointer(name, nick, blurb)
+#define gegl_chant_format(name, nick, blurb)
 #define gegl_chant_color(name, nick, def, blurb)    \
   if (properties->name)                             \
     {                                               \
@@ -764,7 +786,12 @@ static void gegl_chant_destroy_notify (gpointer data)
       g_object_unref (properties->name);            \
       properties->name = NULL;                      \
     }
-#define gegl_chant_seed(name, nick, blurb)
+#define gegl_chant_seed(name, rand_name, nick, blurb)\
+  if (properties->rand_name)                         \
+    {                                                \
+      gegl_random_free (properties->rand_name);      \
+      properties->rand_name = NULL;                  \
+    }
 #define gegl_chant_path(name, nick, blurb)          \
   if (properties->name)                             \
     {                                               \
@@ -785,6 +812,7 @@ static void gegl_chant_destroy_notify (gpointer data)
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -818,10 +846,13 @@ gegl_chant_constructor (GType                  type,
 #define gegl_chant_multiline(name, nick, def, blurb)
 #define gegl_chant_object(name, nick, blurb)
 #define gegl_chant_pointer(name, nick, blurb)
+#define gegl_chant_format(name, nick, blurb)
 #define gegl_chant_color(name, nick, def, blurb)              \
     if (properties->name == NULL) \
     {properties->name = gegl_color_new(def?def:"black");}
-#define gegl_chant_seed(name, nick, blurb)
+#define gegl_chant_seed(name, rand_name, nick, blurb)         \
+    if (properties->rand_name == NULL)                        \
+      {properties->rand_name = gegl_random_new_with_seed (0);}
 #define gegl_chant_path(name, nick, blurb)
 #define gegl_chant_curve(name, nick, blurb)
 
@@ -838,6 +869,7 @@ gegl_chant_constructor (GType                  type,
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
@@ -946,6 +978,13 @@ gegl_chant_class_intern_init (gpointer klass)
                                                         G_PARAM_READWRITE |  \
                                                         G_PARAM_CONSTRUCT |  \
                                                         GEGL_PARAM_PAD_INPUT)));
+#define gegl_chant_format(name, nick, blurb)                                 \
+  g_object_class_install_property (object_class, PROP_##name,                \
+                                   gegl_param_spec_format (#name, nick, blurb, \
+                                                          (GParamFlags) (      \
+                                                          G_PARAM_READWRITE |  \
+                                                          G_PARAM_CONSTRUCT |  \
+                                                          GEGL_PARAM_PAD_INPUT)));
 #define gegl_chant_color(name, nick, def, blurb)                             \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    gegl_param_spec_color_from_string (#name, nick, blurb,\
@@ -954,7 +993,7 @@ gegl_chant_class_intern_init (gpointer klass)
                                                           G_PARAM_READWRITE |\
                                                           G_PARAM_CONSTRUCT | \
                                                           GEGL_PARAM_PAD_INPUT)));
-#define gegl_chant_seed(name, nick, blurb)                                   \
+#define gegl_chant_seed(name, rand_name, nick, blurb)                        \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    gegl_param_spec_seed (#name, nick, blurb, \
                                                      (GParamFlags) (         \
@@ -993,6 +1032,7 @@ gegl_chant_class_intern_init (gpointer klass)
 #undef gegl_chant_multiline
 #undef gegl_chant_object
 #undef gegl_chant_pointer
+#undef gegl_chant_format
 #undef gegl_chant_color
 #undef gegl_chant_curve
 #undef gegl_chant_seed
