@@ -14,6 +14,7 @@
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2003-2007 Calvin Williamson, Øyvind Kolås
+ *           2013      Daniel Sabo
  */
 
 #include "config.h"
@@ -59,9 +60,13 @@ static inline gboolean
 pid_is_running (gint pid)
 {
   HANDLE h;
+  DWORD exitcode = 0;
 
-  h = OpenProcess(PROCESS_TERMINATE, FALSE, pid);
-  return (TerminateProcess(h, 0));
+  h = OpenProcess (PROCESS_QUERY_INFORMATION, FALSE, pid);
+  GetExitCodeProcess (h, &exitcode);
+  CloseHandle (h);
+
+  return exitcode == STILL_ACTIVE;
 }
 
 #else
@@ -77,67 +82,28 @@ pid_is_running (gint pid)
 #endif
 
 
-#include <gegl-debug.h>
-
+#include "gegl-debug.h"
 
 guint gegl_debug_flags = 0;
 
+#include "gegl-types.h"
+#include "gegl-types-internal.h"
 #include "gegl-instrument.h"
 #include "gegl-init.h"
+#include "gegl-init-private.h"
 #include "module/geglmodule.h"
 #include "module/geglmoduledb.h"
-#include "gegl-types-internal.h"
 #include "buffer/gegl-buffer.h"
 #include "operation/gegl-operation.h"
 #include "operation/gegl-operations.h"
-#include "operation/gegl-extension-handler.h"
+#include "operation/gegl-extension-handler-private.h"
 #include "buffer/gegl-buffer-private.h"
+#include "buffer/gegl-buffer-iterator-private.h"
+#include "buffer/gegl-tile-backend-ram.h"
+#include "buffer/gegl-tile-backend-file.h"
 #include "gegl-config.h"
-#include "graph/gegl-node.h"
-
-
-/* if this function is made to return NULL swapping is disabled */
-const gchar *
-gegl_swap_dir (void)
-{
-  static gchar *swapdir = "";
-
-  if (swapdir && swapdir[0] == '\0')
-    {
-      if (g_getenv ("GEGL_SWAP"))
-        {
-          if (g_str_equal (g_getenv ("GEGL_SWAP"), "RAM"))
-            swapdir = NULL;
-          else
-            swapdir = g_strdup (g_getenv ("GEGL_SWAP"));
-        }
-      else
-        {
-          swapdir = g_build_filename (g_get_user_cache_dir(),
-                                      GEGL_LIBRARY,
-                                      "swap",
-                                      NULL);
-        }
-
-      /* Fall back to "swapping to RAM" if not able to create swap dir
-       */
-      if (swapdir &&
-          ! g_file_test (swapdir, G_FILE_TEST_IS_DIR) &&
-          g_mkdir_with_parents (swapdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
-        {
-#if 0
-          gchar *name = g_filename_display_name (swapdir);
-          g_warning ("unable to create swapdir '%s': %s",
-                     name, g_strerror (errno));
-          g_free (name);
-#endif
-
-          swapdir = NULL;
-        }
-    }
-
-  return swapdir;
-};
+#include "graph/gegl-node-private.h"
+#include "gegl-random-private.h"
 
 static gboolean  gegl_post_parse_hook (GOptionContext *context,
                                        GOptionGroup   *group,
@@ -147,43 +113,127 @@ static gboolean  gegl_post_parse_hook (GOptionContext *context,
 
 static GeglConfig   *config = NULL;
 
-
 static GeglModuleDB *module_db   = NULL;
 
 static glong         global_time = 0;
 
-static const gchar *makefile (void);
+static gboolean      swap_init_done = FALSE;
+
+static gchar        *swap_dir = NULL;
+
+static void
+gegl_init_swap_dir (void)
+{
+  gchar *swapdir = NULL;
+
+  if (config->swap)
+    {
+      if (g_ascii_strcasecmp (config->swap, "ram") == 0)
+        {
+          swapdir = NULL;
+        }
+      else
+        {
+          swapdir = g_strstrip (g_strdup (config->swap));
+
+          /* Remove any trailing separator, unless the path is only made of a leading separator. */
+          while (strlen (swapdir) > strlen (G_DIR_SEPARATOR_S) && g_str_has_suffix (swapdir, G_DIR_SEPARATOR_S))
+            swapdir[strlen (swapdir) - strlen (G_DIR_SEPARATOR_S)] = '\0';
+        }
+    }
+
+  if (swapdir &&
+      ! g_file_test (swapdir, G_FILE_TEST_IS_DIR) &&
+      g_mkdir_with_parents (swapdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
+    {
+      g_free (swapdir);
+      swapdir = NULL;
+    }
+
+  g_object_set (config, "swap", swapdir, NULL);
+
+  swap_dir = swapdir;
+
+  swap_init_done = TRUE;
+}
+
+/* Return the swap directory, or NULL if swapping is disabled */
+const gchar *
+gegl_swap_dir (void)
+{
+  if (!swap_init_done)
+  {
+    g_critical ("swap parsing not done");
+    gegl_init_swap_dir ();
+  }
+
+  return swap_dir;
+}
+
+static void
+gegl_config_application_license_notify (GObject    *gobject,
+                                        GParamSpec *pspec,
+                                        gpointer    user_data)
+{
+  GeglConfig *cfg = GEGL_CONFIG (gobject);
+
+  gegl_operations_set_licenses_from_string (cfg->application_license);
+}
+
+
+static void
+gegl_config_use_opencl_notify (GObject    *gobject,
+                               GParamSpec *pspec,
+                               gpointer    user_data)
+{
+  GeglConfig *cfg = GEGL_CONFIG (gobject);
+
+  g_signal_handlers_block_by_func (gobject,
+                                   gegl_config_use_opencl_notify,
+                                   NULL);
+
+  if (cfg->use_opencl)
+    {
+       gegl_cl_init (NULL);
+    }
+  else
+    {
+      gegl_cl_disable ();
+    }
+
+  cfg->use_opencl = gegl_cl_is_accelerated();
+
+  g_signal_handlers_unblock_by_func (gobject,
+                                     gegl_config_use_opencl_notify,
+                                     NULL);
+}
+
+static void
+gegl_init_i18n (void)
+{
+  static gboolean i18n_initialized = FALSE;
+
+  if (! i18n_initialized)
+    {
+      bindtextdomain (GETTEXT_PACKAGE, GEGL_LOCALEDIR);
+      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
+      i18n_initialized = TRUE;
+    }
+}
 
-/**
- * gegl_init:
- * @argc: a pointer to the number of command line arguments.
- * @argv: a pointer to the array of command line arguments.
- *
- * Call this function before using any other GEGL functions. It will initialize
- * everything needed to operate GEGL and parses some standard command line
- * options.  @argc and @argv are adjusted accordingly so your own code will
- * never see those standard arguments.
- *
- * Note that there is an alternative ways to initialize GEGL: if you are
- * calling g_option_context_parse() with the option group returned by
- * gegl_get_option_group(), you don't have to call gegl_init().
- **/
 void
 gegl_init (gint    *argc,
            gchar ***argv)
 {
   GOptionContext *context;
   GError         *error = NULL;
-  if (config)
+  static gboolean initialized = FALSE;
+
+  if (initialized)
     return;
 
-  /*  If any command-line actions are ever added to GEGL, then the commented
-   *  out code below should be used.  Until then, we simply call the parse hook
-   *  directly.
-   */
-#if 0
-  gegl_post_parse_hook (NULL, NULL, NULL, NULL);
-#else
+  initialized = TRUE;
 
   context = g_option_context_new (NULL);
   g_option_context_set_ignore_unknown_options (context, TRUE);
@@ -197,28 +247,22 @@ gegl_init (gint    *argc,
     }
 
   g_option_context_free (context);
-#endif
 }
 
-static gchar   *cmd_gegl_swap=NULL;
-static gchar   *cmd_gegl_cache_size=NULL;
-static gchar   *cmd_gegl_chunk_size=NULL;
-static gchar   *cmd_gegl_quality=NULL;
-static gchar   *cmd_gegl_tile_size=NULL;
-static gchar   *cmd_babl_tolerance =NULL;
-static gchar   *cmd_gegl_threads=NULL;
+static gchar    *cmd_gegl_swap           = NULL;
+static gchar    *cmd_gegl_cache_size     = NULL;
+static gchar    *cmd_gegl_chunk_size     = NULL;
+static gchar    *cmd_gegl_quality        = NULL;
+static gchar    *cmd_gegl_tile_size      = NULL;
+static gchar    *cmd_gegl_threads        = NULL;
+static gboolean *cmd_gegl_disable_opencl = NULL;
 
 static const GOptionEntry cmd_entries[]=
 {
-    {
-     "babl-tolerance", 0, 0,
-     G_OPTION_ARG_STRING, &cmd_babl_tolerance,
-     N_("babls error tolerance, a value beteen 0.2 and 0.000000001"), "<float>"
-    },
     {
      "gegl-swap", 0, 0,
      G_OPTION_ARG_STRING, &cmd_gegl_swap,
-     N_("Where GEGL stores it's swap"), "<uri>"
+     N_("Where GEGL stores its swap"), "<uri>"
     },
     {
      "gegl-cache-size", 0, 0,
@@ -245,24 +289,21 @@ static const GOptionEntry cmd_entries[]=
      G_OPTION_ARG_STRING, &cmd_gegl_threads,
      N_("The number of concurrent processing threads to use"), "<threads>"
     },
+    {
+      "gegl-disable-opencl", 0, 0,
+      G_OPTION_ARG_NONE, &cmd_gegl_disable_opencl,
+      N_("Disable OpenCL"), NULL
+    },
     { NULL }
 };
 
-/**
- * gegl_get_option_group:
- *
- * Returns a #GOptionGroup for the commandline arguments recognized
- * by GEGL. You should add this group to your #GOptionContext
- * with g_option_context_add_group(), if you are using
- * g_option_context_parse() to parse your commandline arguments.
- *
- * Returns a #GOptionGroup for the commandline arguments recognized by GEGL.
- */
 GOptionGroup *
 gegl_get_option_group (void)
 {
   GOptionGroup *group;
 
+  gegl_init_i18n ();
+
   group = g_option_group_new ("gegl", "GEGL Options", "Show GEGL Options",
                               NULL, NULL);
   g_option_group_add_entries (group, cmd_entries);
@@ -272,52 +313,94 @@ gegl_get_option_group (void)
   return group;
 }
 
+static void gegl_config_set_defaults (GeglConfig *config)
+{
+  gchar *swapdir = g_build_filename (g_get_user_cache_dir(),
+                                     GEGL_LIBRARY,
+                                     "swap",
+                                     NULL);
+  g_object_set (config,
+                "swap", swapdir,
+                NULL);
+
+  g_free (swapdir);
+}
+
+static void gegl_config_parse_env (GeglConfig *config)
+{
+  if (g_getenv ("GEGL_QUALITY"))
+    {
+      const gchar *quality = g_getenv ("GEGL_QUALITY");
+
+      if (g_str_equal (quality, "fast"))
+        g_object_set (config, "quality", 0.0, NULL);
+      else if (g_str_equal (quality, "good"))
+        g_object_set (config, "quality", 0.5, NULL);
+      else if (g_str_equal (quality, "best"))
+        g_object_set (config, "quality", 1.0, NULL);
+      else
+        g_object_set (config, "quality", atof (quality), NULL);
+    }
+
+  if (g_getenv ("GEGL_CACHE_SIZE"))
+    config->tile_cache_size = atoll(g_getenv("GEGL_CACHE_SIZE"))* 1024*1024;
+
+  if (g_getenv ("GEGL_CHUNK_SIZE"))
+    config->chunk_size = atoi(g_getenv("GEGL_CHUNK_SIZE"));
+
+  if (g_getenv ("GEGL_TILE_SIZE"))
+    {
+      const gchar *str = g_getenv ("GEGL_TILE_SIZE");
+      config->tile_width = atoi(str);
+      str = strchr (str, 'x');
+      if (str)
+        config->tile_height = atoi(str+1);
+    }
+
+  if (g_getenv ("GEGL_THREADS"))
+    {
+      _gegl_threads = atoi(g_getenv("GEGL_THREADS"));
+
+      if (_gegl_threads > GEGL_MAX_THREADS)
+        {
+          g_warning ("Tried to use %i threads, max is %i",
+                     _gegl_threads, GEGL_MAX_THREADS);
+          _gegl_threads = GEGL_MAX_THREADS;
+        }
+    }
+
+  if (g_getenv ("GEGL_USE_OPENCL"))
+    {
+      const char *opencl_env = g_getenv ("GEGL_USE_OPENCL");
+
+      if (g_ascii_strcasecmp (opencl_env, "yes") == 0)
+        ;
+      else if (g_ascii_strcasecmp (opencl_env, "no") == 0)
+        gegl_cl_hard_disable ();
+      else if (g_ascii_strcasecmp (opencl_env, "cpu") == 0)
+        gegl_cl_set_default_device_type (CL_DEVICE_TYPE_CPU);
+      else if (g_ascii_strcasecmp (opencl_env, "gpu") == 0)
+        gegl_cl_set_default_device_type (CL_DEVICE_TYPE_GPU);
+      else if (g_ascii_strcasecmp (opencl_env, "accelerator") == 0)
+        gegl_cl_set_default_device_type (CL_DEVICE_TYPE_ACCELERATOR);
+      else
+        g_warning ("Unknown value for GEGL_USE_OPENCL: %s", opencl_env);
+    }
+
+  if (g_getenv ("GEGL_SWAP"))
+    g_object_set (config, "swap", g_getenv ("GEGL_SWAP"), NULL);
+}
+
 GeglConfig *gegl_config (void)
 {
   if (!config)
     {
       config = g_object_new (GEGL_TYPE_CONFIG, NULL);
-      if (g_getenv ("GEGL_QUALITY"))
-        config->quality = atof(g_getenv("GEGL_QUALITY"));
-      if (g_getenv ("GEGL_CACHE_SIZE"))
-        config->cache_size = atoi(g_getenv("GEGL_CACHE_SIZE"))* 1024*1024;
-      if (g_getenv ("GEGL_CHUNK_SIZE"))
-        config->chunk_size = atoi(g_getenv("GEGL_CHUNK_SIZE"));
-      if (g_getenv ("GEGL_TILE_SIZE"))
-        {
-          const gchar *str = g_getenv ("GEGL_TILE_SIZE");
-          config->tile_width = atoi(str);
-          str = strchr (str, 'x');
-          if (str)
-            config->tile_height = atoi(str+1);
-        }
-      if (g_getenv ("GEGL_THREADS"))
-        {
-          config->threads = atoi(g_getenv("GEGL_THREADS"));
-          if (config->threads > GEGL_MAX_THREADS)
-            {
-              g_warning ("Tried to use %i threads max is %i",
-                         config->threads, GEGL_MAX_THREADS);
-              config->threads = GEGL_MAX_THREADS;
-            }
-        }
-
-      if (g_getenv ("GEGL_USE_OPENCL") != NULL && strcmp(g_getenv ("GEGL_USE_OPENCL"), "yes") == 0)
-        config->use_opencl = TRUE;
-      else
-        config->use_opencl = FALSE;
-
-      if (gegl_swap_dir())
-        config->swap = g_strdup(gegl_swap_dir ());
+      gegl_config_set_defaults (config);
     }
-  return GEGL_CONFIG (config);
+  return config;
 }
 
-void gegl_tile_backend_ram_stats (void);
-void gegl_tile_backend_tiledir_stats (void);
-void gegl_tile_backend_file_stats (void);
-
-
 static void swap_clean (void)
 {
   const gchar  *swap_dir = gegl_swap_dir ();
@@ -355,17 +438,27 @@ static void swap_clean (void)
     }
 }
 
-void gegl_tile_storage_cache_cleanup (void);
+void gegl_temp_buffer_free (void);
 
 void
 gegl_exit (void)
 {
-  glong timing = gegl_ticks ();
+  if (!config)
+    {
+      g_warning("gegl_exit() called without matching call to gegl_init()");
+      return;
+    }
 
-  gegl_tile_storage_cache_cleanup ();
+  GEGL_INSTRUMENT_START()
+
+  gegl_tile_backend_swap_cleanup ();
   gegl_tile_cache_destroy ();
   gegl_operation_gtype_cleanup ();
   gegl_extension_handler_cleanup ();
+  gegl_random_cleanup ();
+  gegl_cl_cleanup ();
+
+  gegl_temp_buffer_free ();
 
   if (module_db != NULL)
     {
@@ -375,8 +468,7 @@ gegl_exit (void)
 
   babl_exit ();
 
-  timing = gegl_ticks () - timing;
-  gegl_instrument ("gegl", "gegl_exit", timing);
+  GEGL_INSTRUMENT_END ("gegl", "gegl_exit")
 
   /* used when tracking buffer and tile leaks */
   if (g_getenv ("GEGL_DEBUG_BUFS") != NULL)
@@ -384,18 +476,24 @@ gegl_exit (void)
       gegl_buffer_stats ();
       gegl_tile_backend_ram_stats ();
       gegl_tile_backend_file_stats ();
-      gegl_tile_backend_tiledir_stats ();
     }
   global_time = gegl_ticks () - global_time;
   gegl_instrument ("gegl", "gegl", global_time);
 
-  if (g_getenv ("GEGL_DEBUG_TIME") != NULL)
+  if (gegl_instrument_enabled)
     {
       g_printf ("\n%s", gegl_instrument_utf8 ());
     }
 
   if (gegl_buffer_leaks ())
-    g_printf ("EEEEeEeek! %i GeglBuffers leaked\n", gegl_buffer_leaks ());
+    {
+      g_printf ("EEEEeEeek! %i GeglBuffers leaked\n", gegl_buffer_leaks ());
+#ifdef GEGL_ENABLE_DEBUG
+      if (!(gegl_debug_flags & GEGL_DEBUG_BUFFER_ALLOC))
+        g_printerr ("To debug GeglBuffer leaks, set the environment "
+                    "variable GEGL_DEBUG to \"buffer-alloc\"\n");
+#endif
+    }
   gegl_tile_cache_destroy ();
 
   if (gegl_swap_dir ())
@@ -432,18 +530,11 @@ gegl_exit (void)
     }
   g_object_unref (config);
   config = NULL;
+  global_time = 0;
 }
 
 static void swap_clean (void);
 
-static void
-gegl_init_i18n (void)
-{
-  setlocale (LC_ALL, "");
-  bindtextdomain (GETTEXT_PACKAGE, GEGL_LOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-}
-
 void
 gegl_get_version (int *major,
                   int *minor,
@@ -459,6 +550,59 @@ gegl_get_version (int *major,
     *micro = GEGL_MICRO_VERSION;
 }
 
+void
+gegl_load_module_directory (const gchar *path)
+{
+  g_return_if_fail (g_file_test (path, G_FILE_TEST_IS_DIR));
+
+  gegl_module_db_load (module_db, path);
+}
+
+
+GSList *
+gegl_get_default_module_paths(void)
+{
+  GSList *list = NULL;
+  gchar *module_path = NULL;
+
+  // GEGL_PATH
+  const gchar *gegl_path = g_getenv ("GEGL_PATH");
+  if (gegl_path)
+    {
+      list = g_slist_append (list, g_strdup (gegl_path));
+      return list;
+    }
+
+  // System library dir
+#ifdef G_OS_WIN32
+  {
+    gchar *prefix;
+    prefix = g_win32_get_package_installation_directory_of_module ( hLibGeglModule );
+    module_path = g_build_filename (prefix, "lib", GEGL_LIBRARY, NULL);
+    g_free(prefix);
+  }
+#else
+  module_path = g_build_filename (LIBDIR, GEGL_LIBRARY, NULL);
+#endif
+  list = g_slist_append (list, module_path);
+
+  /* User data dir
+   * ~/.local/share/gegl-x.y/plug-ins */
+  module_path = g_build_filename (g_get_user_data_dir (),
+                                  GEGL_LIBRARY,
+                                  "plug-ins",
+                                  NULL);
+  g_mkdir_with_parents (module_path, S_IRUSR | S_IWUSR | S_IXUSR);
+  list = g_slist_append (list, module_path);
+
+  return list;
+}
+
+static void
+load_module_path(gchar *path, GeglModuleDB *db)
+{
+  gegl_module_db_load (db, path);
+}
 
 static gboolean
 gegl_post_parse_hook (GOptionContext *context,
@@ -466,39 +610,21 @@ gegl_post_parse_hook (GOptionContext *context,
                       gpointer        data,
                       GError        **error)
 {
-  glong time;
-
-  if (config)
-    return TRUE;
-
+  GeglConfig *config;
 
   g_assert (global_time == 0);
   global_time = gegl_ticks ();
-  g_type_init ();
+
+  if (g_getenv ("GEGL_DEBUG_TIME") != NULL)
+    gegl_instrument_enable ();
+
   gegl_instrument ("gegl", "gegl_init", 0);
 
-  config = (void*)gegl_config ();
+  config = gegl_config ();
 
-  if (cmd_gegl_swap)
-    g_object_set (config, "swap", cmd_gegl_swap, NULL);
-  if (cmd_gegl_quality)
-    config->quality = atof (cmd_gegl_quality);
-  if (cmd_gegl_cache_size)
-    config->cache_size = atoi (cmd_gegl_cache_size)*1024*1024;
-  if (cmd_gegl_chunk_size)
-    config->chunk_size = atoi (cmd_gegl_chunk_size);
-  if (cmd_gegl_tile_size)
-    {
-      const gchar *str = cmd_gegl_tile_size;
-      config->tile_width = atoi(str);
-      str = strchr (str, 'x');
-      if (str)
-        config->tile_height = atoi(str+1);
-    }
-  if (cmd_gegl_threads)
-    config->threads = atoi (cmd_gegl_threads);
-  if (cmd_babl_tolerance)
-    g_object_set (config, "babl-tolerance", atof(cmd_babl_tolerance), NULL);
+  gegl_config_parse_env (config);
+
+  babl_init ();
 
 #ifdef GEGL_ENABLE_DEBUG
   {
@@ -515,131 +641,70 @@ gegl_post_parse_hook (GOptionContext *context,
   }
 #endif /* GEGL_ENABLE_DEBUG */
 
-  time = gegl_ticks ();
+  if (cmd_gegl_swap)
+    g_object_set (config, "swap", cmd_gegl_swap, NULL);
+  if (cmd_gegl_quality)
+    config->quality = atof (cmd_gegl_quality);
+  if (cmd_gegl_cache_size)
+    config->tile_cache_size = atoll (cmd_gegl_cache_size)*1024*1024;
+  if (cmd_gegl_chunk_size)
+    config->chunk_size = atoi (cmd_gegl_chunk_size);
+  if (cmd_gegl_tile_size)
+    {
+      const gchar *str = cmd_gegl_tile_size;
+      config->tile_width = atoi(str);
+      str = strchr (str, 'x');
+      if (str)
+        config->tile_height = atoi(str+1);
+    }
+  if (cmd_gegl_threads)
+    {
+      _gegl_threads = atoi (cmd_gegl_threads);
+      if (_gegl_threads > GEGL_MAX_THREADS)
+        {
+          g_warning ("Tried to use %i threads, max is %i",
+                     _gegl_threads, GEGL_MAX_THREADS);
+          _gegl_threads = GEGL_MAX_THREADS;
+        }
+    }
+  if (cmd_gegl_disable_opencl)
+    gegl_cl_hard_disable ();
 
-  babl_init ();
-  gegl_instrument ("gegl_init", "babl_init", gegl_ticks () - time);
+  gegl_init_swap_dir ();
 
-  gegl_init_i18n ();
+  GEGL_INSTRUMENT_START();
+
+  gegl_operation_gtype_init ();
 
-  time = gegl_ticks ();
   if (!module_db)
     {
-      const gchar *gegl_path = g_getenv ("GEGL_PATH");
-
+      GSList *paths = gegl_get_default_module_paths ();
       module_db = gegl_module_db_new (FALSE);
-
-      if (gegl_path)
-        {
-          gegl_module_db_load (module_db, gegl_path);
-        }
-      else
-        {
-          gchar *module_path;
-
-#ifdef G_OS_WIN32
-          {
-            gchar *prefix;
-            prefix = g_win32_get_package_installation_directory_of_module ( hLibGeglModule );
-            module_path = g_build_filename (prefix, "lib", GEGL_LIBRARY, NULL);
-            g_free(prefix);
-          }
-#else
-          module_path = g_build_filename (LIBDIR, GEGL_LIBRARY, NULL);
-#endif
-
-          gegl_module_db_load (module_db, module_path);
-          g_free (module_path);
-
-          /* also load plug-ins from ~/.local/share/gegl-0.0/plugins */
-          module_path = g_build_filename (g_get_user_data_dir (),
-                                          GEGL_LIBRARY,
-                                          "plug-ins",
-                                          NULL);
-
-          if (g_mkdir_with_parents (module_path,
-                                    S_IRUSR | S_IWUSR | S_IXUSR) == 0)
-            {
-              gchar *makefile_path = g_build_filename (module_path,
-                                                       "Makefile",
-                                                       NULL);
-
-              if (! g_file_test (makefile_path, G_FILE_TEST_EXISTS))
-                g_file_set_contents (makefile_path, makefile (), -1, NULL);
-
-              g_free (makefile_path);
-            }
-
-          gegl_module_db_load (module_db, module_path);
-          g_free (module_path);
-        }
-
-      gegl_instrument ("gegl_init", "load modules", gegl_ticks () - time);
+      g_slist_foreach(paths, (GFunc)load_module_path, module_db);
+      g_slist_free_full (paths, g_free);
     }
 
+  GEGL_INSTRUMENT_END ("gegl_init", "load modules");
+
   gegl_instrument ("gegl", "gegl_init", gegl_ticks () - global_time);
 
-  if (g_getenv ("GEGL_SWAP"))
-    g_object_set (config, "swap", g_getenv ("GEGL_SWAP"), NULL);
-  if (g_getenv ("GEGL_QUALITY"))
-    {
-      const gchar *quality = g_getenv ("GEGL_QUALITY");
-      if (g_str_equal (quality, "fast"))
-        g_object_set (config, "quality", 0.0, NULL);
-      if (g_str_equal (quality, "good"))
-        g_object_set (config, "quality", 0.5, NULL);
-      if (g_str_equal (quality, "best"))
-        g_object_set (config, "quality", 1.0, NULL);
-    }
-
   swap_clean ();
 
-  /* Initialize OpenCL if wanted and possible */
-  if (gegl_config()->use_opencl)
-    gegl_cl_init (NULL);
+  g_signal_connect (G_OBJECT (config),
+                   "notify::use-opencl",
+                   G_CALLBACK (gegl_config_use_opencl_notify),
+                   NULL);
+  g_object_set (config, "use-opencl", config->use_opencl, NULL);
 
-  GEGL_NOTE (GEGL_DEBUG_OPENCL, "Using OpenCL: %d\n", gegl_config()->use_opencl && gegl_cl_is_accelerated ());
+  g_signal_connect (G_OBJECT (config),
+                   "notify::application-license",
+                   G_CALLBACK (gegl_config_application_license_notify),
+                   NULL);
+  gegl_operations_set_licenses_from_string (config->application_license);
 
   return TRUE;
 }
 
-
-
-#ifdef GEGL_ENABLE_DEBUG
-#if 0
-static gboolean
-gegl_arg_debug_cb (const char *key,
-                   const char *value,
-                   gpointer    user_data)
-{
-  gegl_debug_flags |=
-    g_parse_debug_string (value,
-                          gegl_debug_keys,
-                          G_N_ELEMENTS (gegl_debug_keys));
-  return TRUE;
-}
-
-static gboolean
-gegl_arg_no_debug_cb (const char *key,
-                      const char *value,
-                      gpointer    user_data)
-{
-  gegl_debug_flags &=
-    ~g_parse_debug_string (value,
-                           gegl_debug_keys,
-                           G_N_ELEMENTS (gegl_debug_keys));
-  return TRUE;
-}
-#endif
-#endif
-
-/*
- * gegl_get_debug_enabled:
- *
- * Check if gegl has debugging turned on.
- *
- * Return value: TRUE if debugging is turned on, FALSE otherwise.
- */
 gboolean
 gegl_get_debug_enabled (void)
 {
@@ -649,27 +714,3 @@ gegl_get_debug_enabled (void)
   return FALSE;
 #endif
 }
-
-
-static const gchar *makefile (void)
-{
-  return
-    "# This is a generic makefile for GEGL operations. Just add .c files,\n"
-    "# rename mentions of the filename and opname to the new name, and it should \n"
-    "# compile. Operations in this dir should be loaded by GEGL by default\n"
-    "# If the operation being written depends on extra libraries, you'd better\n"
-    "# add a dedicated target with the extra bits linked in.\n"
-    "\n\n"
-    "CFLAGS  += `pkg-config gegl --cflags`  -I. -fPIC\n"
-    "LDFLAGS += `pkg-config gegl --libs` -shared\n"
-    "SHREXT=.so\n"
-    "CFILES = $(wildcard ./*.c)\n"
-    "SOBJS  = $(subst ./,,$(CFILES:.c=$(SHREXT)))\n"
-    "all: $(SOBJS)\n"
-    "%$(SHREXT): %.c $(GEGLHEADERS)\n"
-    "	@echo $@; $(CC) $(CFLAGS) $(LDFLAGS) -o $@ $< $(LDADD)\n"
-    "clean:\n"
-    "	rm -f *$(SHREXT) $(OFILES)\n";
-}
-
-
