@@ -22,25 +22,12 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GeglCRVisitor        GeglCRVisitor;
-typedef struct _GeglDebugRectVisitor GeglDebugRectVisitor;
-typedef struct _GeglEvalMgr          GeglEvalMgr;
-typedef struct _GeglEvalVisitor      GeglEvalVisitor;
-typedef struct _GeglFinishVisitor    GeglFinishVisitor;
-typedef struct _GeglGraph            GeglGraph;
-typedef struct _GeglHaveVisitor      GeglHaveVisitor;
-typedef struct _GeglNeedVisitor      GeglNeedVisitor;
-typedef struct _GeglDotVisitor       GeglDotVisitor;
-#ifndef __GEGL_PLUGIN_H__
 typedef struct _GeglPad              GeglPad;
-typedef struct _GeglOperation        GeglOperation;
-typedef struct _GeglOperationContext GeglOperationContext;
 typedef struct _GeglConnection       GeglConnection;
-#endif
-typedef struct _GeglPrepareVisitor   GeglPrepareVisitor;
-typedef struct _GeglLockVisitor      GeglLockVisitor;
-typedef struct _GeglUnlockVisitor    GeglUnlockVisitor;
-typedef struct _GeglVisitable        GeglVisitable; /* dummy typedef */
+
+typedef struct _GeglEvalManager      GeglEvalManager;
+typedef struct _GeglDotVisitor       GeglDotVisitor;
+typedef struct _GeglVisitable        GeglVisitable;
 typedef struct _GeglVisitor          GeglVisitor;
 
 typedef struct _GeglPoint            GeglPoint;
@@ -59,6 +46,30 @@ struct _GeglDimension
 };
 
 
+static inline int gegl_level_from_scale (gfloat scale)
+{
+  gint level = 0;
+  //gint factor = 1;
+
+  while (scale <= 0.500001)
+  {
+    scale *= 2;
+   // factor *= 2;
+    level++;
+#if 0
+    if (rect)
+    {
+      rect->x = 0 < rect->x ? rect->x/2 : (rect->x - 1) / 2;
+      rect->y = 0 < rect->y ? rect->y/2 : (rect->y - 1) / 2;
+      rect->width = 0 < rect->width ? rect->width/2 : (rect->width  - 1) / 2;
+      rect->height = 0 < rect->height ? rect->height/2 : (rect->height  - 1) / 2;
+    }
+#endif
+  }
+
+  return level;
+}
+
 G_END_DECLS
 
 #endif /* __GEGL_TYPES_INTERNAL_H__ */
