@@ -25,13 +25,6 @@
 #include <limits.h>
 
 #include "gegl.h"
-#include "gegl-types-internal.h"
-#include "graph/gegl-node.h"
-#include "graph/gegl-pad.h"
-#include "operation/gegl-operation.h"
-#include "property-types/gegl-color.h"
-#include "property-types/gegl-curve.h"
-#include "property-types/gegl-path.h"
 #include "property-types/gegl-paramspecs.h"
 #include "gegl-instrument.h"
 #include "gegl-xml.h"
@@ -108,17 +101,15 @@ static void param_set (ParseData   *pd,
   else if (strcmp (param_name, "operation") &&
            strcmp (param_name, "type"))
     {
-      GeglOperation *operation;
       GParamSpec    *paramspec;
 set_clone_prop_as_well:
 
-      operation = new->operation;
-      paramspec = g_object_class_find_property (G_OBJECT_GET_CLASS (operation), param_name);
+      paramspec = gegl_node_find_property (new, param_name);
 
       if (!paramspec)
         {
           g_warning ("property %s not found for %s",
-                     param_name, gegl_node_get_debug_name (new));
+                     param_name, gegl_node_get_operation (new));
         }
       else if (g_type_is_a (G_PARAM_SPEC_TYPE (paramspec),
                             GEGL_TYPE_PARAM_FILE_PATH))
@@ -132,6 +123,7 @@ set_clone_prop_as_well:
           else
             {
               gchar * absolute_path;
+              gchar   temp_path[PATH_MAX];
               if (pd->path_root)
                 {
                   buf = g_strdup_printf ("%s/%s", pd->path_root, param_value);
@@ -141,17 +133,17 @@ set_clone_prop_as_well:
                   buf = g_strdup_printf ("./%s", param_value);
                 }
 
-              absolute_path = realpath (buf, NULL);
+              absolute_path = realpath (buf, temp_path);
               g_free (buf);
               if (absolute_path)
                 {
                   gegl_node_set (new, param_name, absolute_path, NULL);
-                  free (absolute_path);
                 }
               else
                 {
-                  g_warning ("Unable to obtain absolute path for paramater %s\n",
-                             param_name);
+                  g_warning ("Unable to obtain absolute path for parameter %s\n", param_name);
+                  // Attempt to set raw value, useful for '-' meaning stdin
+                  gegl_node_set (new, param_name, param_value, NULL);
                 }
             }
         }
@@ -190,7 +182,33 @@ set_clone_prop_as_well:
         {
           GEnumClass *eclass = g_type_class_peek (paramspec->value_type);
           GEnumValue *evalue = g_enum_get_value_by_nick (eclass, param_value);
-          gegl_node_set (new, param_name, evalue->value, NULL);
+          if (evalue)
+            {
+              gegl_node_set (new, param_name, evalue->value, NULL);
+            }
+          else
+            {
+              /* warn, but try to get a valid nick out of the old-style
+               * value name
+               */
+              gchar *nick;
+              gchar *c;
+              g_printerr ("gegl-xml (param_set %s): enum %s has no value '%s'\n",
+                          paramspec->name,
+                          g_type_name (paramspec->value_type),
+                          param_value);
+              nick = g_strdup (param_value);
+              for (c = nick; *c; c++)
+                {
+                  *c = g_ascii_tolower (*c);
+                  if (*c == ' ')
+                    *c = '-';
+                }
+              evalue = g_enum_get_value_by_nick (eclass, nick);
+              if (evalue)
+                gegl_node_set (new, param_name, evalue->value, NULL);
+              g_free (nick);
+            }
         }
       else if (paramspec->value_type == GEGL_TYPE_COLOR)
         {
@@ -218,6 +236,16 @@ set_clone_prop_as_well:
           gegl_path_parse_string (path, param_value);
           gegl_node_set (new, param_name, path, NULL);
         }
+      else if (paramspec->value_type == G_TYPE_POINTER &&
+               GEGL_IS_PARAM_SPEC_FORMAT (paramspec))
+        {
+          const Babl *format = NULL;
+
+          if (strlen (param_value))
+            format = babl_format (param_value);
+
+          gegl_node_set (new, param_name, format, NULL);
+        }
       else
         {
           g_warning ("operation desired unknown parapspec type for %s",
@@ -272,12 +300,16 @@ static void start_element (GMarkupParseContext *context,
       else
         {
         }
+
       pd->state  = STATE_TREE_NORMAL;
       pd->parent = g_list_prepend (pd->parent, new);
 
-      gegl_node_get_output_proxy (new, "output"); /* creates the pad if it doesn't exist */
       if (pd->iter)
-        gegl_node_connect_from (pd->iter, "input", new, "output");
+	{
+	  gegl_node_get_output_proxy (pd->iter, "output");
+	  gegl_node_connect_from (pd->iter, "input", new, "output");
+	}
+
       pd->iter = gegl_node_get_output_proxy (new, "output");
     }
   else if (!strcmp (element_name, "graph"))
@@ -392,8 +424,10 @@ static void start_element (GMarkupParseContext *context,
         }
       else
         {
-          if (pd->iter)
-            gegl_node_connect_from (pd->iter, "input", new, "output");
+          if (pd->iter && gegl_node_has_pad(new, "output"))
+	    {
+	      gegl_node_connect_from (pd->iter, "input", new, "output");
+	    }
         }
       pd->parent = g_list_prepend (pd->parent, new);
       pd->state  = STATE_TREE_FIRST_CHILD;
@@ -431,7 +465,7 @@ static void end_element (GMarkupParseContext *context,
   else if (!strcmp (element_name, "tree") ||
            !strcmp (element_name, "layers"))
     {
-      if (gegl_node_get_pad (pd->iter, "input"))
+      if (gegl_node_get_producer (pd->iter, "input", NULL))
         {
           gegl_node_connect_from (pd->iter, "input",
                                   gegl_node_get_input_proxy (GEGL_NODE (pd->parent->data), "input"),
@@ -522,13 +556,14 @@ static void each_ref (gpointer value,
 GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
                                   const gchar *path_root)
 {
-  glong                time = gegl_ticks ();
   ParseData            pd   = { 0, };
   GMarkupParseContext *context;
   gboolean             success = FALSE;
 
   g_return_val_if_fail (xmldata != NULL, NULL);
 
+  GEGL_INSTRUMENT_START();
+
   pd.ids       = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   pd.refs      = NULL;
   pd.path_root = path_root;
@@ -558,8 +593,7 @@ GeglNode *gegl_node_new_from_xml (const gchar *xmldata,
   g_markup_parse_context_free (context);
   g_hash_table_destroy (pd.ids);
 
-  time = gegl_ticks () - time;
-  gegl_instrument ("gegl", "gegl_parse_xml", time);
+  GEGL_INSTRUMENT_END ("gegl", "gegl_parse_xml");
 
   return success ? GEGL_NODE (pd.gegl) : NULL;
 }
@@ -762,7 +796,7 @@ serialize_properties (SerializeState *ss,
   gboolean     got_a_param = FALSE;
   gint         i;
 
-  properties = gegl_list_properties (gegl_node_get_operation (node),
+  properties = gegl_operation_list_properties (gegl_node_get_operation (node),
                                      &n_properties);
 
   for (i = 0; i < n_properties; i++)
@@ -885,6 +919,18 @@ serialize_properties (SerializeState *ss,
 
               g_object_unref (path);
             }
+          else if (properties[i]->value_type == G_TYPE_POINTER &&
+                   GEGL_IS_PARAM_SPEC_FORMAT (properties[i]))
+            {
+              const Babl  *format;
+              const gchar *value;
+              gegl_node_get (node, properties[i]->name, &format, NULL);
+              if (format)
+                value = babl_get_name (format);
+              else
+                value = "";
+              xml_param (ss, indent + 2, properties[i]->name, value);
+            }
           else
             {
               g_warning ("%s: serialization of %s properties not implemented",
@@ -969,15 +1015,11 @@ add_stack (SerializeState *ss,
 
       while (iter)
         {
-          GeglPad     *input, *aux;
           const gchar *id = NULL;
           gchar       *class;
           gegl_node_get (iter, "operation", &class, NULL);
 
-          input = gegl_node_get_pad (iter, "input");
-          aux   = gegl_node_get_pad (iter, "aux");
-
-          if (gegl_node_get_num_sinks (iter) > 1)
+          if (gegl_node_get_consumers (iter, "output", NULL, NULL) > 1)
             {
               const gchar *new_id = g_hash_table_lookup (ss->clones, iter);
               if (new_id)
@@ -1007,13 +1049,9 @@ add_stack (SerializeState *ss,
                 }
               else
                 {
-                  if (aux &&
-                      gegl_pad_get_connected_to (aux))
+                  GeglNode *source_node = gegl_node_get_producer (iter, "aux", NULL);
+                  if (source_node)
                     {
-                      GeglPad  *source_pad;
-                      GeglNode *source_node;
-                      source_pad  = gegl_pad_get_connected_to (aux);
-                      source_node = gegl_pad_get_node (source_pad);
                       {
                         GeglNode *graph = g_object_get_data (G_OBJECT (source_node),
                                                              "graph");
@@ -1035,6 +1073,10 @@ add_stack (SerializeState *ss,
                         xml_attr (ss->buf, "operation", class);
                         if (id != NULL)
                           xml_attr (ss->buf, "id", id);
+
+                        if (gegl_node_get_passthrough (iter) == TRUE)
+                          xml_attr (ss->buf, "passthrough", "true");
+
                         g_free (name);
                         g_free (class);
                       }
@@ -1065,6 +1107,10 @@ add_stack (SerializeState *ss,
                             xml_attr (ss->buf, "operation", class);
                             if (id != NULL)
                               xml_attr (ss->buf, "id", id);
+
+                            if (gegl_node_get_passthrough (iter) == TRUE)
+                              xml_attr (ss->buf, "passthrough", "true");
+
                             g_free (name);
                             g_free (class);
                           }
@@ -1077,26 +1123,19 @@ add_stack (SerializeState *ss,
                 }
             }
           id = NULL;
-          if (input)
-            {
-              GeglPad *source_pad;
-              source_pad = gegl_pad_get_connected_to (input);
-              if (source_pad)
-                {
-                  GeglNode *source_node = gegl_pad_get_node (source_pad);
-                  GeglNode *graph       = g_object_get_data (G_OBJECT (source_node),
-                                                             "graph");
-                  if (graph)
-                    source_node = graph;
-                  iter = source_node;
-                }
-              else
-                iter = NULL;
-            }
-          else
-            {
+          {
+            GeglNode *source_node = gegl_node_get_producer (iter, "input", NULL);
+            if (source_node)
+              {
+                GeglNode *graph       = g_object_get_data (G_OBJECT (source_node),
+                                                           "graph");
+                if (graph)
+                  source_node = graph;
+                iter = source_node;
+              }
+            else
               iter = NULL;
-            }
+          }
 
           g_free (class);
         }
@@ -1117,7 +1156,9 @@ gegl_node_to_xml (GeglNode    *gegl,
   ss.clones      = g_hash_table_new (NULL, NULL);
   ss.terse       = FALSE;
 
-  gegl = gegl_node_get_output_proxy (gegl, "output");
+  /* this case is probably only for empty graphs */
+  if (!gegl_node_get_operation (gegl))
+    gegl = gegl_node_get_output_proxy (gegl, "output");
 
   g_string_append (ss.buf, "<?xml version='1.0' encoding='UTF-8'?>\n");
   g_string_append (ss.buf, "<gegl>\n");
