@@ -963,12 +963,15 @@ gegl_node_apply_roi (GeglNode            *self,
     }
 }
 
-static void
-gegl_node_blit_buffer2 (GeglNode            *self,
-                        GeglBuffer          *buffer,
-                        const GeglRectangle *roi,
-                        gint                 level) // XXX: add abyss policy
+void
+gegl_node_blit_buffer (GeglNode            *self,
+                       GeglBuffer          *buffer,
+                       const GeglRectangle *roi,
+                       gint                 level,
+                       GeglAbyssPolicy      abyss_policy) 
 {
+  // XXX: make use of abyss_policy
+
   GeglEvalManager *eval_manager;
   GeglBuffer      *result;
   GeglRectangle    request;
@@ -992,14 +995,6 @@ gegl_node_blit_buffer2 (GeglNode            *self,
     }
 }
 
-void
-gegl_node_blit_buffer (GeglNode            *self,
-                       GeglBuffer          *buffer,
-                       const GeglRectangle *roi)
-{
-  gegl_node_blit_buffer2 (self, buffer, roi, 0);
-}
-
 static inline gboolean gegl_mipmap_rendering_enabled (void)
 {
   static int enabled = -1;
@@ -1056,12 +1051,12 @@ gegl_node_blit (GeglNode            *self,
               const GeglRectangle unscaled_roi = _gegl_get_required_for_scale (format, roi, scale);
               gint  level = gegl_mipmap_rendering_enabled()?gegl_level_from_scale (scale):0;
 
-              gegl_node_blit_buffer2 (self, buffer, &unscaled_roi, level);
+              gegl_node_blit_buffer (self, buffer, &unscaled_roi, level, GEGL_ABYSS_NONE);
               gegl_cache_computed (cache, &unscaled_roi, level);
             }
           else
             {
-              gegl_node_blit_buffer (self, buffer, roi);
+              gegl_node_blit_buffer (self, buffer, roi, 0, GEGL_ABYSS_NONE);
               gegl_cache_computed (cache, roi, 0);
             }
         }
@@ -1836,15 +1831,6 @@ gegl_node_emit_computed (GeglNode *node,
   g_signal_emit (node, gegl_node_signals[COMPUTED], 0, rect, NULL, NULL);
 }
 
-static void
-gegl_node_computed_event (GeglCache *self,
-                          void      *foo,
-                          void      *user_data)
-{
-  GeglNode *node = GEGL_NODE (user_data);
-  gegl_node_emit_computed (node, foo);
-}
-
 GeglCache *
 gegl_node_get_cache (GeglNode *node)
 {
@@ -1893,9 +1879,9 @@ gegl_node_get_cache (GeglNode *node)
       gegl_node_get_bounding_box (node);
       gegl_buffer_set_extent (GEGL_BUFFER (cache), &node->have_rect);
 
-      g_signal_connect (G_OBJECT (cache), "computed",
-                        (GCallback) gegl_node_computed_event,
-                        node);
+      g_signal_connect_swapped (G_OBJECT (cache), "computed",
+                                (GCallback) gegl_node_emit_computed,
+                                node);
       node->cache = cache;
     }
 
@@ -2123,6 +2109,8 @@ gegl_node_get_pad_proxy (GeglNode    *node,
 
       if (!is_graph_input)
         {
+          g_signal_connect_swapped (G_OBJECT (nop), "computed",
+                                    G_CALLBACK (gegl_node_emit_computed), node);
           g_signal_connect (G_OBJECT (nop), "invalidated",
                             G_CALLBACK (graph_source_invalidated), node);
         }
