@@ -35,9 +35,7 @@ typedef struct _GeglVisitorClass GeglVisitorClass;
 struct _GeglVisitor
 {
   GObject     parent_instance;
-  gpointer    context_id;
 
-  GSList     *visits_list;
   GHashTable *hash;
 };
 
@@ -54,7 +52,6 @@ struct _GeglVisitorClass
 
 GType    gegl_visitor_get_type         (void) G_GNUC_CONST;
 
-GSList * gegl_visitor_get_visits_list (GeglVisitor   *self);
 void     gegl_visitor_reset           (GeglVisitor   *self);
 void     gegl_visitor_visit_visitable (GeglVisitor   *self,
                                        GeglVisitable *visitable);
