@@ -22,14 +22,8 @@
 #include <string.h>
 
 #include "gegl.h"
-#include "gegl-types-internal.h"
 #include "gegl-operation-temporal.h"
-#include "gegl-utils.h"
-#include "graph/gegl-node.h"
-#include "graph/gegl-connection.h"
-#include "graph/gegl-pad.h"
-#include "buffer/gegl-region.h"
-#include "buffer/gegl-buffer.h"
+#include "gegl-operation-context.h"
 
 struct _GeglOperationTemporalPrivate
 {
@@ -89,7 +83,8 @@ gegl_operation_temporal_get_frame (GeglOperation *op,
 static gboolean gegl_operation_temporal_process (GeglOperation       *self,
                                                  GeglBuffer          *input,
                                                  GeglBuffer          *output,
-                                                 const GeglRectangle *result)
+                                                 const GeglRectangle *result,
+                                                 gint                 level)
 {
   GeglOperationTemporal *temporal = GEGL_OPERATION_TEMPORAL (self);
   GeglOperationTemporalPrivate *priv = temporal->priv;
@@ -104,7 +99,7 @@ static gboolean gegl_operation_temporal_process (GeglOperation       *self,
    GeglRectangle write_rect = *result;
    write_rect.y = priv->next_to_write * priv->height;
 
-   gegl_buffer_copy (input, result, priv->frame_store, &write_rect);
+   gegl_buffer_copy (input, result, GEGL_ABYSS_NONE, priv->frame_store, &write_rect);
    priv->count++;
    priv->next_to_write++;
    if (priv->next_to_write >= priv->history_length)
@@ -112,7 +107,7 @@ static gboolean gegl_operation_temporal_process (GeglOperation       *self,
   }
 
  if (temporal_class->process)
-   return temporal_class->process (self, input, output, result);
+   return temporal_class->process (self, input, output, result, level);
  return FALSE;
 }
 
