@@ -14,7 +14,7 @@
  * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
  *
  * Copyright 2003      Calvin Williamson
- *           2005-2008 Øyvind Kolås
+ * 2005-2009,2011-2014 Øyvind Kolås
  */
 
 #include "config.h"
@@ -23,14 +23,14 @@
 #include <string.h>
 
 #include "gegl.h"
+#include "gegl-config.h"
 #include "gegl-types-internal.h"
 #include "gegl-operation.h"
-#include "gegl-utils.h"
-#include "graph/gegl-node.h"
+#include "gegl-operation-context.h"
+#include "gegl-operations-util.h"
+#include "graph/gegl-node-private.h"
 #include "graph/gegl-connection.h"
 #include "graph/gegl-pad.h"
-#include "buffer/gegl-region.h"
-#include "buffer/gegl-buffer.h"
 #include "gegl-operations.h"
 
 static void         attach                    (GeglOperation       *self);
@@ -88,14 +88,15 @@ gegl_operation_class_init (GeglOperationClass *klass)
   klass->attach                    = attach;
   klass->prepare                   = NULL;
   klass->no_cache                  = FALSE;
-  klass->opencl_support            = FALSE;
+  klass->threaded                  = FALSE;
   klass->get_bounding_box          = get_bounding_box;
   klass->get_invalidated_by_change = get_invalidated_by_change;
   klass->get_required_for_output   = get_required_for_output;
+  klass->cl_data                   = NULL;
 }
 
 static void
-gegl_operation_base_init  (GeglOperationClass *klass)
+gegl_operation_base_init (GeglOperationClass *klass)
 {
   /* XXX: leaked for now, should replace G_DEFINE_TYPE with the expanded one */
   klass->keys = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -136,9 +137,6 @@ gegl_operation_create_pad (GeglOperation *self,
   gegl_node_add_pad (self->node, pad);
 }
 
-void gegl_node_emit_computed (GeglNode *node, const GeglRectangle *rect);
-
-
 gboolean
 gegl_operation_process (GeglOperation        *operation,
                         GeglOperationContext *context,
@@ -162,6 +160,15 @@ gegl_operation_process (GeglOperation        *operation,
       return TRUE;
     }
 
+  if (operation->node->passthrough)
+  {
+    GeglBuffer *input = GEGL_BUFFER (gegl_operation_context_get_object (context, "input"));
+    gegl_operation_context_take_object (context, output_pad, g_object_ref (G_OBJECT (input)));
+    return TRUE;
+  }
+
+  g_return_val_if_fail (klass->process, FALSE);
+
   return klass->process (operation, context, output_pad, result, level);
 }
 
@@ -174,14 +181,23 @@ gegl_operation_get_bounding_box (GeglOperation *self)
   GeglOperationClass *klass = GEGL_OPERATION_GET_CLASS (self);
   GeglRectangle       rect  = { 0, 0, 0, 0 };
 
-  if (klass->get_bounding_box)
+  if (self->node->passthrough)
+  {
+    GeglRectangle  result = { 0, 0, 0, 0 };
+    GeglRectangle *in_rect;
+    in_rect = gegl_operation_source_get_bounding_box (self, "input");
+    if (in_rect)
+      return *in_rect;
+    return result;
+  }
+  else if (klass->get_bounding_box)
     return klass->get_bounding_box (self);
 
   return rect;
 }
 
 GeglRectangle
-gegl_operation_get_invalidated_by_change (GeglOperation        *self,
+gegl_operation_get_invalidated_by_change (GeglOperation       *self,
                                           const gchar         *input_pad,
                                           const GeglRectangle *input_region)
 {
@@ -192,6 +208,9 @@ gegl_operation_get_invalidated_by_change (GeglOperation        *self,
   g_return_val_if_fail (input_pad != NULL, retval);
   g_return_val_if_fail (input_region != NULL, retval);
 
+  if (self->node && self->node->passthrough)
+    return *input_region;
+
   klass = GEGL_OPERATION_GET_CLASS (self);
 
   if (input_region->width  == 0 ||
@@ -209,6 +228,9 @@ get_required_for_output (GeglOperation        *operation,
                          const gchar         *input_pad,
                          const GeglRectangle *roi)
 {
+  if (operation->node->passthrough)
+    return *roi;
+
   if (operation->node->is_graph)
     {
       return gegl_operation_get_required_for_output (operation, input_pad, roi);
@@ -228,6 +250,9 @@ gegl_operation_get_required_for_output (GeglOperation        *operation,
       roi->height == 0)
     return *roi;
 
+  if (operation->node->passthrough)
+    return *roi;
+
   g_assert (klass->get_required_for_output);
 
   return klass->get_required_for_output (operation, input_pad, roi);
@@ -241,6 +266,9 @@ gegl_operation_get_cached_region (GeglOperation        *operation,
 {
   GeglOperationClass *klass = GEGL_OPERATION_GET_CLASS (operation);
 
+  if (operation->node->passthrough)
+    return *roi;
+
   if (!klass->get_cached_region)
     {
       return *roi;
@@ -281,8 +309,32 @@ gegl_operation_prepare (GeglOperation *self)
 
   g_return_if_fail (GEGL_IS_OPERATION (self));
 
+  if (self->node->passthrough)
+    return;
+
   klass = GEGL_OPERATION_GET_CLASS (self);
 
+  /* build OpenCL kernel */
+  if (!klass->cl_data)
+  {
+    const gchar *cl_source = gegl_operation_class_get_key (klass, "cl-source");
+    if (cl_source)
+      {
+        char *name = strdup (klass->name);
+        const char *kernel_name[] = {name, NULL};
+        char *k;
+        for (k=name; *k; k++)
+          switch (*k)
+            {
+              case ' ': case ':': case '-':
+                *k = '_';
+                break;
+            }
+        klass->cl_data = gegl_cl_compile_and_build (cl_source, kernel_name);
+        free (name);
+      }
+  }
+
   if (klass->prepare)
     klass->prepare (self);
 }
@@ -312,7 +364,7 @@ gegl_operation_get_source_node (GeglOperation *operation,
 }
 
 GeglRectangle *
-gegl_operation_source_get_bounding_box (GeglOperation  *operation,
+gegl_operation_source_get_bounding_box (GeglOperation *operation,
                                         const gchar   *input_pad_name)
 {
   GeglNode *node = gegl_operation_get_source_node (operation, input_pad_name);
@@ -320,9 +372,9 @@ gegl_operation_source_get_bounding_box (GeglOperation  *operation,
   if (node)
     {
       GeglRectangle *ret;
-      g_mutex_lock (node->mutex);
+      g_mutex_lock (&node->mutex);
       ret = &node->have_rect;
-      g_mutex_unlock (node->mutex);
+      g_mutex_unlock (&node->mutex);
       return ret;
     }
 
@@ -355,23 +407,6 @@ get_invalidated_by_change (GeglOperation        *self,
                            const gchar         *input_pad,
                            const GeglRectangle *input_region)
 {
-#if 0
-  /* FIXME: this seems to sometimes go into an infinite loop, the
-   * current workaround of passing the rectangle straight through
-   * isn't even true for unsharp-mask/drop-shadow/difference of gaussians,
-   * but it stops a crasher bug.
-   *
-   * This needs to be revisited as part of the core processing revisit
-   * (perhaps together with a meta-op framework revwrite).
-   */
-  if (self->node->is_graph)
-    {
-      return gegl_operation_get_invalidated_by_change (
-               gegl_node_get_output_proxy (self->node, "output")->operation,
-               input_pad,
-               input_region);
-    }
-#endif
   return *input_region;
 }
 
@@ -457,10 +492,6 @@ gegl_operation_get_format (GeglOperation *self,
 
   pad = gegl_node_get_pad (self->node, pad_name);
 
-  if (pad == NULL || pad->format == NULL)
-    {
-      g_warning ("%s: returns NULL", G_STRFUNC);
-    }
   g_return_val_if_fail (pad != NULL, NULL);
 
   return pad->format;
@@ -494,6 +525,100 @@ gegl_operation_invalidate (GeglOperation       *operation,
   gegl_node_invalidated (node, roi, TRUE);
 }
 
+gboolean
+gegl_operation_cl_set_kernel_args (GeglOperation *operation,
+                                   cl_kernel      kernel,
+                                   gint          *p,
+                                   cl_int        *err)
+{
+  GParamSpec **self;
+  GParamSpec **parent;
+  guint n_self;
+  guint n_parent;
+  gint prop_no;
+
+  self = g_object_class_list_properties (
+            G_OBJECT_CLASS (g_type_class_ref (G_OBJECT_CLASS_TYPE (GEGL_OPERATION_GET_CLASS(operation)))),
+            &n_self);
+
+  parent = g_object_class_list_properties (
+            G_OBJECT_CLASS (g_type_class_ref (GEGL_TYPE_OPERATION)),
+            &n_parent);
+
+  for (prop_no=0;prop_no<n_self;prop_no++)
+    {
+      gint parent_no;
+      gboolean found=FALSE;
+      for (parent_no=0;parent_no<n_parent;parent_no++)
+        if (self[prop_no]==parent[parent_no])
+          found=TRUE;
+      /* only print properties if we are an addition compared to
+       * GeglOperation
+       */
+
+      /* Removing pads */
+      if (!strcmp(g_param_spec_get_name (self[prop_no]), "input") ||
+          !strcmp(g_param_spec_get_name (self[prop_no]), "output") ||
+          !strcmp(g_param_spec_get_name (self[prop_no]), "aux"))
+        continue;
+
+      if (!found)
+        {
+          if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_DOUBLE))
+            {
+              gdouble value;
+              cl_float v;
+
+              g_object_get (G_OBJECT (operation), g_param_spec_get_name (self[prop_no]), &value, NULL);
+
+              v = value;
+              *err = gegl_clSetKernelArg(kernel, (*p)++, sizeof(cl_float), (void*)&v);
+            }
+          else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_FLOAT))
+            {
+              gfloat value;
+              cl_float v;
+
+              g_object_get (G_OBJECT (operation), g_param_spec_get_name (self[prop_no]), &value, NULL);
+
+              v = value;
+              *err = gegl_clSetKernelArg(kernel, (*p)++, sizeof(cl_float), (void*)&v);
+            }
+          else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_INT))
+            {
+              gint value;
+              cl_int v;
+
+              g_object_get (G_OBJECT (operation), g_param_spec_get_name (self[prop_no]), &value, NULL);
+
+              v = value;
+              *err = gegl_clSetKernelArg(kernel, (*p)++, sizeof(cl_int), (void*)&v);
+            }
+          else if (g_type_is_a (G_PARAM_SPEC_VALUE_TYPE (self[prop_no]), G_TYPE_BOOLEAN))
+            {
+              gboolean value;
+              cl_bool v;
+
+              g_object_get (G_OBJECT (operation), g_param_spec_get_name (self[prop_no]), &value, NULL);
+
+              v = value;
+              *err = gegl_clSetKernelArg(kernel, (*p)++, sizeof(cl_bool), (void*)&v);
+            }
+          else
+            {
+              g_error ("Unsupported OpenCL kernel argument");
+              return FALSE;
+            }
+        }
+    }
+
+  if (self)
+    g_free (self);
+  if (parent)
+    g_free (parent);
+
+  return TRUE;
+}
 
 gchar **
 gegl_operation_list_keys (const gchar *operation_name,
@@ -516,7 +641,7 @@ gegl_operation_list_keys (const gchar *operation_name,
   count = g_hash_table_size (GEGL_OPERATION_CLASS (klass)->keys);
   ret = g_malloc0 (sizeof (gpointer) * (count + 1));
   list = g_hash_table_get_keys (GEGL_OPERATION_CLASS (klass)->keys);
-  for (i = 0, l = list; l; l = l->next)
+  for (i = 0, l = list; l; l = l->next, i++)
     {
       ret[i] = l->data;
     }
@@ -532,21 +657,42 @@ gegl_operation_class_set_key (GeglOperationClass *klass,
                               const gchar        *key_name,
                               const gchar        *key_value)
 {
+  gchar *key_value_dup;
+
+  g_return_if_fail (key_name != NULL);
+
   if (!key_value)
     {
       g_hash_table_remove (klass->keys, key_name);
       return;
     }
-  else
-    {
-      key_value = g_strdup (key_value);
-      g_hash_table_insert (klass->keys, g_strdup (key_name),
-                           (void*)key_value);
-    }
+
+  key_value_dup = g_strdup (key_value);
+
   if (!strcmp (key_name, "name"))
     {
-      klass->name = key_value;
+      if (klass->name && strcmp (klass->name, key_value))
+        {
+          g_warning ("Cannot change name of operation class 0x%lX from \"%s\" "
+                     "to \"%s\"", (gulong) klass, klass->name, key_value);
+          g_free (key_value_dup);
+          return;
+        }
+      else
+        {
+          klass->name = key_value_dup;
+          gegl_operation_class_register_name (klass, key_value, FALSE);
+        }
     }
+
+  if (!strcmp (key_name, "compat-name"))
+    {
+      klass->compat_name = key_value_dup;
+      gegl_operation_class_register_name (klass, key_value, TRUE);
+    }
+
+  g_hash_table_insert (klass->keys, g_strdup (key_name),
+                       (void*)key_value_dup);
 }
 
 void
@@ -609,3 +755,76 @@ gegl_operation_get_key (const gchar *operation_name,
   g_type_class_unref (klass);
   return ret;
 }
+
+gboolean
+gegl_operation_use_opencl (const GeglOperation *operation)
+{
+  g_return_val_if_fail (operation->node, FALSE);
+  return operation->node->use_opencl && gegl_cl_is_accelerated ();
+}
+
+const Babl *
+gegl_operation_get_source_format (GeglOperation *operation,
+                                  const gchar   *padname)
+{
+  GeglNode *src_node = gegl_operation_get_source_node (operation, padname);
+
+  if (src_node)
+    {
+      GeglOperation *op = src_node->operation;
+      if (op)
+        return gegl_operation_get_format (op, "output");
+            /* XXX: could be a different pad than "output" */
+    }
+  return NULL;
+}
+
+gboolean
+gegl_operation_use_threading (GeglOperation *operation,
+                              const GeglRectangle *roi)
+{
+  gint threads = gegl_config_threads ();
+  if (threads == 1)
+    return FALSE;
+
+  {
+    GeglOperationClass       *op_class;
+    op_class = GEGL_OPERATION_GET_CLASS (operation);
+
+    if (op_class->opencl_support && gegl_cl_is_accelerated ())
+      return FALSE;
+
+    if (op_class->threaded &&
+        roi->width * roi->height > 64*64)
+      return TRUE;
+  }
+  return FALSE;
+}
+
+static guchar *gegl_temp_alloc[GEGL_MAX_THREADS * 4]={NULL,};
+static gint    gegl_temp_size[GEGL_MAX_THREADS * 4]={0,};
+
+guchar *gegl_temp_buffer (int no, int size)
+{
+  if (!gegl_temp_alloc[no] || gegl_temp_size[no] < size)
+  {
+    if (gegl_temp_alloc[no])
+      gegl_free (gegl_temp_alloc[no]);
+    gegl_temp_alloc[no] = gegl_malloc (size);
+    gegl_temp_size[no] = size;
+  }
+  return gegl_temp_alloc[no];
+}
+
+void gegl_temp_buffer_free (void);
+void gegl_temp_buffer_free (void)
+{
+  int no;
+  for (no = 0; no < GEGL_MAX_THREADS * 4; no++)
+    if (gegl_temp_alloc[no])
+    {
+      gegl_free (gegl_temp_alloc[no]);
+      gegl_temp_alloc[no] = NULL;
+      gegl_temp_size[no] = 0;
+    }
+}
