@@ -21,17 +21,24 @@
 #include <glib-object.h>
 
 #include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-debug.h"
 #include "buffer/gegl-region.h"
-#include "graph/gegl-node.h"
+#include "graph/gegl-node-private.h"
 
+#include "operation/gegl-operation-context.h"
+#include "operation/gegl-operation-context-private.h"
 #include "operation/gegl-operation-sink.h"
 
 #include "gegl-config.h"
 #include "gegl-processor.h"
-#include "gegl-types-internal.h"
-#include "gegl-utils.h"
+#include "gegl-processor-private.h"
 
+#include "graph/gegl-visitor.h"
+#include "graph/gegl-visitable.h"
+#include "process/gegl-list-visitor.h"
+
+#include "opencl/gegl-cl.h"
 
 enum
 {
@@ -56,9 +63,7 @@ static void      gegl_processor_get_property (GObject               *gobject,
                                               GParamSpec            *pspec);
 static void      gegl_processor_set_node     (GeglProcessor         *processor,
                                               GeglNode              *node);
-static GObject * gegl_processor_constructor  (GType                  type,
-                                              guint                  n_params,
-                                              GObjectConstructParam *params);
+static void      gegl_processor_constructed  (GObject               *object);
 static gdouble   gegl_processor_progress     (GeglProcessor         *processor);
 static gint      gegl_processor_get_band_size(gint                   size) G_GNUC_CONST;
 
@@ -69,6 +74,7 @@ struct _GeglProcessor
   GeglNode        *node;
   GeglRectangle    rectangle;
   GeglNode        *input;
+  gint             level;
   GeglOperationContext *context;
 
   GeglRegion      *valid_region;     /* used when doing unbuffered rendering */
@@ -89,7 +95,7 @@ gegl_processor_class_init (GeglProcessorClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize     = gegl_processor_finalize;
-  gobject_class->constructor  = gegl_processor_constructor;
+  gobject_class->constructed  = gegl_processor_constructed;
   gobject_class->set_property = gegl_processor_set_property;
   gobject_class->get_property = gegl_processor_get_property;
 
@@ -118,7 +124,7 @@ gegl_processor_class_init (GeglProcessorClass *klass)
                                    g_param_spec_int ("chunksize",
                                                      "chunksize",
                                                      "Size of chunks being rendered (larger chunks need more memory to do the processing).",
-                                                     1, 1024 * 1024, gegl_config()->chunk_size,
+                                                     1, 4096 * 4096, gegl_config()->chunk_size,
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
 }
@@ -126,6 +132,7 @@ gegl_processor_class_init (GeglProcessorClass *klass)
 static void
 gegl_processor_init (GeglProcessor *processor)
 {
+  processor->level            = 0;
   processor->node             = NULL;
   processor->input            = NULL;
   processor->context          = NULL;
@@ -134,23 +141,14 @@ gegl_processor_init (GeglProcessor *processor)
   processor->chunk_size       = 128 * 128;
 }
 
-/* Initialises the fields processor->input, processor->valid_region
- * and processor->queued_region.
- */
-static GObject *
-gegl_processor_constructor (GType                  type,
-                            guint                  n_params,
-                            GObjectConstructParam *params)
+static void
+gegl_processor_constructed (GObject *object)
 {
-  GObject       *object;
-  GeglProcessor *processor;
+  GeglProcessor *processor = GEGL_PROCESSOR (object);
 
-  object    = G_OBJECT_CLASS (gegl_processor_parent_class)->constructor (type, n_params, params);
-  processor = GEGL_PROCESSOR (object);
+  G_OBJECT_CLASS (gegl_processor_parent_class)->constructed (object);
 
   processor->queued_region = gegl_region_new ();
-
-  return object;
 }
 
 static void
@@ -160,8 +158,7 @@ gegl_processor_finalize (GObject *self_object)
 
   if (processor->context)
     {
-      GeglCache *cache = gegl_node_get_cache (processor->input);
-      gegl_node_remove_context (processor->node, cache);
+      gegl_operation_context_destroy (processor->context);
     }
 
   if (processor->node)
@@ -337,6 +334,8 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
 #if 0
       gegl_rectangle_intersect (&processor->rectangle, &processor->rectangle, &bounds);
 #endif
+    }
+    {
 
       /* remove already queued dirty rectangles */
       for (iter = processor->dirty_rectangles; iter; iter = g_slist_next (iter))
@@ -355,18 +354,15 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
       gegl_operation_sink_needs_full (processor->node->operation))
     {
       GeglCache *cache;
-      GValue     value = { 0, };
 
       cache = gegl_node_get_cache (processor->input);
 
-      if (!gegl_node_get_context (processor->node, cache))
-        processor->context = gegl_node_add_context (processor->node, cache);
-
-      g_value_init (&value, GEGL_TYPE_BUFFER);
-      g_value_set_object (&value, cache);
-      gegl_operation_context_set_property (processor->context, "input", &value);
-      g_value_unset (&value);
+      if (!processor->context)
+        {
+          processor->context = gegl_operation_context_new (processor->node->operation);
+        }
 
+      gegl_operation_context_set_object (processor->context, "input", G_OBJECT (cache));
 
       gegl_operation_context_set_result_rect (processor->context,
                                               &processor->rectangle);
@@ -374,6 +370,12 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
                                               &processor->rectangle);
     }
 
+  if (processor->valid_region)
+    {
+      gegl_region_destroy (processor->valid_region);
+      processor->valid_region = gegl_region_new ();
+    }
+
   g_object_notify (G_OBJECT (processor), "rectangle");
 }
 
@@ -387,7 +389,12 @@ gegl_processor_get_band_size (gint size)
 
   /* try to make the rects generated match better with potential 2^n sized
    * tiles, XXX: should be improved to make the next slice fit as well. */
-  if (band_size <= 256)
+  if (band_size <= 128)
+    {
+      band_size = MIN(band_size, 64); /* prefer a band_size of 128,
+                                          hoping to hit tiles */
+    }
+  else if (band_size <= 256)
     {
       band_size = MIN(band_size, 128); /* prefer a band_size of 128,
                                           hoping to hit tiles */
@@ -411,10 +418,11 @@ gegl_processor_get_band_size (gint size)
 static gboolean
 render_rectangle (GeglProcessor *processor)
 {
-  gboolean   buffered;
-  const gint max_area = processor->chunk_size;
-  GeglCache *cache    = NULL;
-  gint       pxsize;
+  gboolean    buffered;
+  const gint  max_area = processor->chunk_size * (1<<processor->level) * (1<<processor->level);
+  GeglCache  *cache    = NULL;
+  const Babl *format   = NULL;
+  gint        pxsize;
 
   /* Retreive the cache if the processor's node is not buffered if it's
    * operation is a sink and it doesn't use the full area  */
@@ -423,7 +431,8 @@ render_rectangle (GeglProcessor *processor)
   if (buffered)
     {
       cache = gegl_node_get_cache (processor->input);
-      g_object_get (cache, "px-size", &pxsize, NULL);
+      format = gegl_buffer_get_format ((GeglBuffer *)cache);
+      pxsize = babl_format_get_bytes_per_pixel (format);
     }
 
   if (processor->dirty_rectangles)
@@ -474,29 +483,39 @@ render_rectangle (GeglProcessor *processor)
 
       if (buffered)
         {
-          /* only do work if the rectangle is not completely inside the valid
-           * region of the cache */
-          if (gegl_region_rect_in (cache->valid_region, dr) !=
-              GEGL_OVERLAP_RECTANGLE_IN)
+          gboolean found_full = FALSE;
+          for (gint level = processor->level; level >= 0; level--)
+          {
+            if (gegl_region_rect_in (cache->valid_region[level], dr) == GEGL_OVERLAP_RECTANGLE_IN)
+            {
+              found_full = TRUE;
+              break;
+            }
+            /* XXX: dr should be adjusted to be the bounding box of not-found
+             * in cache if there is partial hits
+             */
+          }
+
+          if (!found_full)
             {
               /* create a buffer and initialise it */
               guchar *buf;
 
-              gegl_region_union_with_rect (cache->valid_region, dr);
               buf = g_malloc (dr->width * dr->height * pxsize);
               g_assert (buf);
 
+              /* FIXME: Check if the node caches naturaly, if so the buffer_set call isn't needed */
+
               /* do the image calculations using the buffer */
-              gegl_node_blit (cache->node, 1.0, dr, cache->format, buf,
+              gegl_node_blit (processor->input, 1.0/(1<<processor->level),
+                              dr, format, buf,
                               GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
 
-
               /* copy the buffer data into the cache */
-              gegl_buffer_set (GEGL_BUFFER (cache), dr, cache->format, buf,
-                               GEGL_AUTO_ROWSTRIDE);
+              gegl_buffer_set (GEGL_BUFFER (cache), dr, processor->level, format, buf, GEGL_AUTO_ROWSTRIDE);
 
               /* tells the cache that the rectangle (dr) has been computed */
-              gegl_cache_computed (cache, dr);
+              gegl_cache_computed (cache, dr, processor->level);
 
               /* release the buffer */
               g_free (buf);
@@ -505,7 +524,8 @@ render_rectangle (GeglProcessor *processor)
         }
       else
         {
-           gegl_node_blit (processor->node, 1.0, dr, NULL, NULL,
+           gegl_node_blit (processor->node, 1.0/(1<<processor->level),
+                           dr, NULL, NULL,
                            GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
            gegl_region_union_with_rect (processor->valid_region, dr);
            g_slice_free (GeglRectangle, dr);
@@ -583,7 +603,7 @@ gegl_processor_progress (GeglProcessor *processor)
     }
   else
     {
-      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+      valid_region = gegl_node_get_cache (processor->input)->valid_region[processor->level];
     }
 
   wanted = rect_area (&(processor->rectangle));
@@ -623,7 +643,7 @@ gegl_processor_render (GeglProcessor *processor,
   else
     {
       g_return_val_if_fail (processor->input != NULL, FALSE);
-      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+      valid_region = gegl_node_get_cache (processor->input)->valid_region[processor->level];
     }
 
   {
@@ -735,7 +755,32 @@ gegl_processor_work (GeglProcessor *processor,
                      gdouble       *progress)
 {
   gboolean   more_work = FALSE;
-  GeglCache *cache     = gegl_node_get_cache (processor->input);
+
+  if (gegl_config()->use_opencl)
+    {
+      if (gegl_cl_is_accelerated ()
+          && processor->chunk_size != GEGL_CL_CHUNK_SIZE)
+        {
+          GeglListVisitor *visitor = g_object_new (GEGL_TYPE_LIST_VISITOR, NULL);
+          GList *iterator = NULL;
+          GList *visits_list = NULL;
+          visits_list = gegl_list_visitor_get_dfs_path (visitor, GEGL_VISITABLE (processor->node));
+
+          for (iterator = visits_list; iterator; iterator = iterator->next)
+            {
+              GeglNode *node = (GeglNode*) iterator->data;
+              if (GEGL_OPERATION_GET_CLASS(node->operation)->cl_data
+                  || GEGL_OPERATION_GET_CLASS(node->operation)->opencl_support)
+                {
+                  processor->chunk_size = GEGL_CL_CHUNK_SIZE;
+                  break;
+                }
+            }
+
+          g_list_free (visits_list);
+          g_object_unref (visitor);
+        }
+    }
 
   more_work = gegl_processor_render (processor, &processor->rectangle, progress);
   if (more_work)
@@ -754,9 +799,8 @@ gegl_processor_work (GeglProcessor *processor,
       gegl_operation_process (processor->node->operation,
                               processor->context,
                               "output"  /* ignored output_pad */,
-                              &processor->context->result_rect
-                              );
-      gegl_node_remove_context (processor->node, cache);
+                              &processor->context->result_rect, processor->context->level);
+      gegl_operation_context_destroy (processor->context);
       processor->context = NULL;
 
       return TRUE;
@@ -765,17 +809,6 @@ gegl_processor_work (GeglProcessor *processor,
   return FALSE;
 }
 
-void
-gegl_processor_destroy (GeglProcessor *processor)
-{
-  /* The only thing this function is doing is shielding users of the
-   * library from GLib. If you are writing a language wrapper you can
-   * use the normal GObject unreffing mechanisms. This function will
-   * never do more than unreffing the processor.
-   */
-  g_object_unref (processor);
-}
-
 GeglProcessor *
 gegl_node_new_processor (GeglNode            *node,
                          const GeglRectangle *rectangle)
@@ -787,3 +820,14 @@ gegl_node_new_processor (GeglNode            *node,
                        "rectangle", rectangle,
                        NULL);
 }
+
+void gegl_processor_set_level (GeglProcessor *processor,
+                               gint           level)
+{
+  processor->level = level;
+}
+void gegl_processor_set_scale (GeglProcessor *processor,
+                               gdouble        scale)
+{
+  processor->level = gegl_level_from_scale (scale);
+}
