@@ -53,6 +53,15 @@ GType        gegl_color_get_type               (void) G_GNUC_CONST;
  * GeglColor is an object containing a color at the moment only RGB colors
  * are supported, in the future a GeglColor might also indicate other
  * enumerated or natively in other color representations colors.
+ *
+ * GeglColor accepts a subset of format string as defined by the CSS color specification:
+ *  http://dev.w3.org/csswg/css-color/
+ *
+ * - RGB hexadecimal notation: #rrggbb[aa] / #rgb[a]
+ * - Named colors, limited to the 16 specified in HTML4
+ * 
+ * To specify linear-light floating-point RGB, use: rgb[a](0.40, 0.44, 0.92 [, a])
+ * The normal bounds are [0.0 1.0], unlike CSS which is [0 255]. Out-of-bounds values are allowed.
  */
 
 /**
@@ -65,13 +74,23 @@ GType        gegl_color_get_type               (void) G_GNUC_CONST;
  */
 GeglColor *  gegl_color_new                    (const gchar *string);
 
+/**
+ * gegl_color_duplicate:
+ * @color: the color to duplicate.
+ *
+ * Creates a copy of @color.
+ *
+ * Return value: (transfer full): A new copy of @color.
+ */
+GeglColor *  gegl_color_duplicate              (GeglColor   *color);
+
 /**
  * gegl_color_get_rgba:
  * @color: a #GeglColor
- * @red: red return location.
- * @green: green return location.
- * @blue: blue return location.
- * @alpha: alpha return location.
+ * @red: (out): red return location.
+ * @green: (out): green return location.
+ * @blue: (out): blue return location.
+ * @alpha: (out): alpha return location.
  *
  * Retrieves the current set color as linear light non premultipled RGBA data,
  * any of the return pointers can be omitted.
@@ -82,17 +101,6 @@ void         gegl_color_get_rgba               (GeglColor   *color,
                                                 gdouble     *blue,
                                                 gdouble     *alpha);
 
-/**
- * gegl_color_get_rgba4f:
- * @color: a #GeglColor
- * @buf: pointer to a buffer for a single "RGBA float" pixel.
- *
- * Retrieves the current set color as linear light non premultipled RGBA data,
- * and stores it at the memory location in @buf
- */
-void         gegl_color_get_rgba4f             (GeglColor   *color,
-                                                gfloat      *buf);
-
 /**
  * gegl_color_set_rgba:
  * @color: a #GeglColor
@@ -108,6 +116,28 @@ void         gegl_color_set_rgba               (GeglColor   *color,
                                                 gdouble      green,
                                                 gdouble      blue,
                                                 gdouble      alpha);
+/**
+ * gegl_color_set_pixel: (skip)
+ * @color: a #GeglColor
+ * @format: a babl pixel format
+ * @pixel: pointer to a pixel
+ *
+ * Set a GeglColor from a pointer to a pixel and it's babl format.
+ */
+void         gegl_color_set_pixel              (GeglColor   *color,
+                                                const Babl  *format,
+                                                const void  *pixel);
+/**
+ * gegl_color_get_pixel: (skip)
+ * @color: a #GeglColor
+ * @format: a babl pixel format
+ * @pixel: pointer to a pixel
+ *
+ * Store the color in a pixel in the given format.
+ */
+void         gegl_color_get_pixel              (GeglColor   *color,
+                                                const Babl  *format,
+                                                void        *pixel);
 
 /***
  */
@@ -117,17 +147,51 @@ void         gegl_color_set_rgba               (GeglColor   *color,
 
 GType        gegl_param_color_get_type         (void) G_GNUC_CONST;
 
+/**
+ * gegl_param_spec_color:
+ * @name: canonical name of the property specified
+ * @nick: nick name for the property specified
+ * @blurb: description of the property specified
+ * @default_color: the default value for the property specified
+ * @flags: flags for the property specified
+ *
+ * Creates a new #GParamSpec instance specifying a #GeglColor property.
+ *
+ * Returns: (transfer full): a newly created parameter specification
+ */
 GParamSpec * gegl_param_spec_color             (const gchar *name,
                                                 const gchar *nick,
                                                 const gchar *blurb,
                                                 GeglColor   *default_color,
                                                 GParamFlags  flags);
 
+/**
+ * gegl_param_spec_color_from_string:
+ * @name: canonical name of the property specified
+ * @nick: nick name for the property specified
+ * @blurb: description of the property specified
+ * @default_color_string: the default value for the property specified
+ * @flags: flags for the property specified
+ *
+ * Creates a new #GParamSpec instance specifying a #GeglColor property.
+ *
+ * Returns: (transfer full): a newly created parameter specification
+ */
 GParamSpec * gegl_param_spec_color_from_string (const gchar *name,
                                                 const gchar *nick,
                                                 const gchar *blurb,
                                                 const gchar *default_color_string,
                                                 GParamFlags  flags);
+/**
+ * gegl_param_spec_color_get_default:
+ * @self: a #GeglColor #GParamSpec
+ *
+ * Get the default color value of the param spec
+ *
+ * Returns: (transfer none): the default #GeglColor
+ */
+GeglColor *
+gegl_param_spec_color_get_default (GParamSpec *self);
 
 G_END_DECLS
 
