@@ -19,22 +19,25 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-/* Followed by this #if ... */
-#ifdef GEGL_CHANT_PROPERTIES
-/* ... are the properties of the filter, these are all scalar values
- * (doubles), the the parameters are:
- *       property name,   min,   max, default, "description of property"
+#ifdef GEGL_PROPERTIES
+
+/*  Here in the top of the file the properties of the operation is declared,
+ *  this causes the declaration of a structure for containing the data.
+ *
+ *  The first member of each property_ macro becomes a struct member
+ *  in the GeglProperties struct used when processing.
  */
 
-gegl_chant_double (contrast, _("Contrast"), -5.0, 5.0, 1.0,
-                   _("Range scale factor"))
-gegl_chant_double (brightness, _("Brightness"), -3.0, 3.0, 0.0,
-                   _("Amount to increase brightness"))
+property_double (contrast, _("Contrast"),  1.0)
+   description  (_("Magnitude of contrast scaling >1.0 brighten < 1.0 darken"))
+   value_range  (-5.0, 5.0)
+   ui_range     (0.0, 2.0)
+
+property_double (brightness, _("Brightness"), 0.0)
+   description  (_("Amount to increase brightness"))
+   value_range  (-3.0, 3.0)
+   ui_range     (-1.0, 1.0)
 
-/* this will create the instance structure for our use, and register the
- * property with the given ranges, default values and a comment for the
- * documentation/tooltip.
- */
 #else
 
 /* Specify the base class we're building our operation on, the base
@@ -42,20 +45,20 @@ gegl_chant_double (brightness, _("Brightness"), -3.0, 3.0, 0.0,
  * brightness contrast the best base class is the POINT_FILTER base
  * class.
  */
-#define GEGL_CHANT_TYPE_POINT_FILTER
+
+#define GEGL_OP_POINT_FILTER
 
 /* We specify the file we're in, this is needed to make the code
  * generation for the properties work.
  */
-#define GEGL_CHANT_C_FILE       "brightness-contrast.c"
+#define GEGL_OP_C_SOURCE brightness-contrast.c
 
-/* Including gegl-chant.h creates most of the GObject boiler plate
- * needed, creating a GeglChant instance structure a GeglChantClass
+/* Including gegl-op.h creates most of the GObject boiler plate
+ * needed, creating a GeglOp instance structure a GeglOpClass
  * structure for our operation, as well as the needed code to register
  * our new gobject with GEGL.
  */
-#include "gegl-chant.h"
-
+#include "gegl-op.h"
 
 /* prepare() is called on each operation providing data to a node that
  * is requested to provide a rendered result. When prepare is called
@@ -76,12 +79,13 @@ process (GeglOperation       *op,
          void                *in_buf,
          void                *out_buf,
          glong                n_pixels,
-         const GeglRectangle *roi)
+         const GeglRectangle *roi,
+         gint                 level)
 {
-  /* Retrieve a pointer to GeglChantO structure which contains all the
+  /* Retrieve a pointer to GeglProperties structure which contains all the
    * chanted properties
    */
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (op);
+  GeglProperties *o = GEGL_PROPERTIES (op);
   gfloat     * GEGL_ALIGNED in_pixel;
   gfloat     * GEGL_ALIGNED out_pixel;
   gfloat      brightness, contrast;
@@ -105,15 +109,31 @@ process (GeglOperation       *op,
   return TRUE;
 }
 
+#include "opencl/brightness-contrast.cl.h"
+
 /*
  * The class init function sets up information needed for this operations class
  * (template) in the GObject OO framework.
  */
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
+  gchar                         *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:brightness-contrast'>"
+    "  <params>"
+    "    <param name='contrast'>1.8</param>"
+    "    <param name='brightness'>0.25</param>"
+    "  </params>"
+    "</node>"
+    "<node operation='gegl:load'>"
+    "  <params>"
+    "    <param name='path'>standard-input.png</param>"
+    "  </params>"
+    "</node>"
+    "</gegl>";
 
   operation_class    = GEGL_OPERATION_CLASS (klass);
   point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);
@@ -125,16 +145,14 @@ gegl_chant_class_init (GeglChantClass *klass)
    */
   point_filter_class->process = process;
 
-  /* specify the name this operation is found under in the GUI/when
-   * programming/in XML
-   */
-  operation_class->name        = "gegl:brightness-contrast";
-
-  /* a colon separated list of categories/tags for this operations */
-  operation_class->categories  = "color";
-
-  /* a description of what this operations does */
-  operation_class->description = _("Changes the light level and contrast.");
+  gegl_operation_class_set_keys (operation_class,
+      "name",       "gegl:brightness-contrast",
+      "title",      "Brightness Contrast",
+      "categories", "color",
+      "description", _("Changes the light level and contrast. This operation operates in linear light, 'contrast' is a scale factor around 50%% gray, and 'brightness' a constant offset to apply after contrast scaling."),
+      "cl-source"  , brightness_contrast_cl_source,
+      "reference-composition", composition,
+      NULL);
 }
 
-#endif /* closing #ifdef GEGL_CHANT_PROPERTIES ... else ... */
+#endif /* closing #ifdef GEGL_PROPERTIES ... else ... */
