@@ -20,19 +20,17 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_object(buffer, _("Input buffer"),
-		  _("The GeglBuffer to load into the pipeline"))
+property_object(buffer, _("Input buffer"), GEGL_TYPE_BUFFER)
+	  description (_("The GeglBuffer to load into the pipeline"))
 
 #else
 
-#define GEGL_CHANT_TYPE_SOURCE
-#define GEGL_CHANT_C_FILE       "buffer-source.c"
+#define GEGL_OP_SOURCE
+#define GEGL_OP_C_SOURCE buffer-source.c
 
-#include "gegl-chant.h"
-#include <gegl.h>
-#include "graph/gegl-node.h"
+#include "gegl-op.h"
 
 typedef struct
 {
@@ -40,12 +38,12 @@ typedef struct
 } Priv;
 
 static Priv *
-get_priv (GeglChantO *o)
+get_priv (GeglProperties *o)
 {
-  Priv *priv = (Priv*)o->chant_data;
+  Priv *priv = (Priv*)o->user_data;
   if (priv == NULL) {
     priv = g_new0 (Priv, 1);
-    o->chant_data = (void*) priv;
+    o->user_data = (void*) priv;
 
     priv->buffer_changed_handler = 0;
   }
@@ -64,7 +62,7 @@ static void
 gegl_buffer_source_prepare (GeglOperation *operation)
 {
   const Babl *format = NULL;
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (o->buffer)
     format = gegl_buffer_get_format (GEGL_BUFFER (o->buffer));
@@ -76,7 +74,7 @@ static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
   GeglRectangle result = {0,0,0,0};
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
 
   if (!o->buffer)
     {
@@ -93,7 +91,7 @@ my_set_property (GObject  *gobject,
                           GParamSpec   *pspec)
 {
   GeglOperation *operation = GEGL_OPERATION (gobject);
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   Priv *p = get_priv(o);
   GObject *buffer = NULL;
 
@@ -104,10 +102,11 @@ my_set_property (GObject  *gobject,
         // Invariant: valid buffer should always have valid signal handler
         g_assert(p->buffer_changed_handler > 0);
         g_signal_handler_disconnect (o->buffer, p->buffer_changed_handler);
+        /* XXX: should decrement signal connected count */
       }
       buffer = G_OBJECT (g_value_get_object (value));
       if (buffer) {
-        p->buffer_changed_handler = g_signal_connect (buffer, "changed", G_CALLBACK(buffer_changed), operation);
+        p->buffer_changed_handler = gegl_buffer_signal_connect (GEGL_BUFFER(buffer), "changed", G_CALLBACK(buffer_changed), operation);
       }
       break;
     default:
@@ -126,7 +125,7 @@ process (GeglOperation        *operation,
          const GeglRectangle  *result,
          gint                  level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (o->buffer)
     {
@@ -141,7 +140,7 @@ process (GeglOperation        *operation,
       /* mark that this buffer should not be used for in-place
        * processing.
        */
-      gegl_object_set_has_forked (o->buffer);
+      gegl_object_set_has_forked (G_OBJECT (o->buffer));
     }
   return TRUE;
 }
@@ -149,7 +148,7 @@ process (GeglOperation        *operation,
 static void
 dispose (GObject *object)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (object);
+  GeglProperties *o = GEGL_PROPERTIES (object);
   Priv *p = get_priv(o);
 
   if (o->buffer)
@@ -157,21 +156,22 @@ dispose (GObject *object)
       // Invariant: valid buffer should always have valid signal handler
       g_assert(p->buffer_changed_handler > 0);
       g_signal_handler_disconnect (o->buffer, p->buffer_changed_handler);
+        /* XXX: should decrement signal connected count */
       g_object_unref (o->buffer);
       o->buffer = NULL;
     }
 
   if (p) {
     g_free(p);
-    o->chant_data = NULL;
+    o->user_data = NULL;
   }
 
-  G_OBJECT_CLASS (gegl_chant_parent_class)->dispose (object);
+  G_OBJECT_CLASS (gegl_op_parent_class)->dispose (object);
 }
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
 
@@ -186,8 +186,9 @@ gegl_chant_class_init (GeglChantClass *klass)
 
   gegl_operation_class_set_keys (operation_class,
       "name",       "gegl:buffer-source",
+      "title",      _("Buffer Source"),
       "categories", "programming:input",
-      "description", _("A source that uses an in-memory GeglBuffer, for use internally by GEGL."),
+      "description", _("Use an existing in-memory GeglBuffer as image source."),
       NULL);
 
   operation_class->no_cache = TRUE;
