@@ -131,12 +131,12 @@ static const char* op_c_source =
 "  if (input_format == NULL || babl_format_has_alpha (input_format))           \n"
 "    {                                                                         \n"
 "      mix->has_alpha = TRUE;                                                  \n"
-"      format = babl_format (\"R'G'B'A float\");                               \n"
+"      format = babl_format (\"RGBA float\");                                  \n"
 "    }                                                                         \n"
 "  else                                                                        \n"
 "    {                                                                         \n"
 "      mix->has_alpha = FALSE;                                                 \n"
-"      format = babl_format (\"R'G'B' float\");                                \n"
+"      format = babl_format (\"RGB float\");                                   \n"
 "    }                                                                         \n"
 "                                                                              \n"
 "  gegl_operation_set_format (operation, \"input\", format);                   \n"
@@ -180,7 +180,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "  c *= norm;                                                                  \n"
 "                                                                              \n"
-"  return (gfloat) MIN(1, MAX(0, c));                                          \n"
+"  return (gfloat) c;                                                          \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static inline void                                                            \n"
