@@ -57,8 +57,7 @@ static const char* op_c_source =
 "                                                                              \n"
 "typedef struct                                                                \n"
 "{                                                                             \n"
-"  gfloat color_in[3];                                                         \n"
-"  gfloat color_out[3];                                                        \n"
+"  gfloat color_diff[3];                                                       \n"
 "  gfloat min[3];                                                              \n"
 "  gfloat max[3];                                                              \n"
 "} CeParamsType;                                                               \n"
@@ -70,27 +69,38 @@ static const char* op_c_source =
 "  const Babl     *format = babl_format (\"R'G'B'A float\");                   \n"
 "  const Babl     *colorformat = babl_format (\"R'G'B' float\");               \n"
 "  CeParamsType *params;                                                       \n"
+"  gfloat        color_in[3];                                                  \n"
+"  gfloat        color_out[3];                                                 \n"
+"  gint          chan;                                                         \n"
 "                                                                              \n"
 "  if (o->user_data == NULL)                                                   \n"
 "    o->user_data = g_slice_new0 (CeParamsType);                               \n"
 "                                                                              \n"
 "  params = (CeParamsType*) o->user_data;                                      \n"
 "                                                                              \n"
-"  gegl_color_get_pixel (o->from_color, colorformat, &(params->color_in));     \n"
-"  gegl_color_get_pixel (o->to_color, colorformat, &(params->color_out));      \n"
+"  gegl_color_get_pixel (o->from_color, colorformat, &color_in);               \n"
+"  gegl_color_get_pixel (o->to_color, colorformat, &color_out);                \n"
 "                                                                              \n"
-"  params->min[0] = CLAMP (params->color_in[0] - o->red_threshold,             \n"
-"                          0.0, 1.0);                                          \n"
-"  params->max[0] = CLAMP (params->color_in[0] + o->red_threshold,             \n"
-"                          0.0, 1.0);                                          \n"
-"  params->min[1] = CLAMP (params->color_in[1] - o->green_threshold,           \n"
-"                          0.0, 1.0);                                          \n"
-"  params->max[1] = CLAMP (params->color_in[1] + o->green_threshold,           \n"
-"                          0.0, 1.0);                                          \n"
-"  params->min[2] = CLAMP (params->color_in[2] - o->blue_threshold,            \n"
-"                          0.0, 1.0);                                          \n"
-"  params->max[2] = CLAMP (params->color_in[2] + o->blue_threshold,            \n"
-"                          0.0, 1.0);                                          \n"
+"  params->min[0] = CLAMP (color_in[0] - o->red_threshold,                     \n"
+"                          0.0, 1.0) - GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  params->max[0] = CLAMP (color_in[0] + o->red_threshold,                     \n"
+"                          0.0, 1.0) + GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  params->min[1] = CLAMP (color_in[1] - o->green_threshold,                   \n"
+"                          0.0, 1.0) - GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  params->max[1] = CLAMP (color_in[1] + o->green_threshold,                   \n"
+"                          0.0, 1.0) + GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  params->min[2] = CLAMP (color_in[2] - o->blue_threshold,                    \n"
+"                          0.0, 1.0) - GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  params->max[2] = CLAMP (color_in[2] + o->blue_threshold,                    \n"
+"                          0.0, 1.0) + GEGL_FLOAT_EPSILON;                     \n"
+"                                                                              \n"
+"  for (chan = 0; chan < 3; chan++)                                            \n"
+"    params->color_diff[chan] = color_out[chan] - color_in[chan];              \n"
 "                                                                              \n"
 "  gegl_operation_set_format (operation, \"input\", format);                   \n"
 "  gegl_operation_set_format (operation, \"output\", format);                  \n"
@@ -129,36 +139,18 @@ static const char* op_c_source =
 "                                                                              \n"
 "  while (n_pixels--)                                                          \n"
 "    {                                                                         \n"
-"      if ((GEGL_FLOAT_EQUAL (input[0], params->min[0]) ||                     \n"
-"          (input[0] > params->min[0])) &&                                     \n"
-"          (GEGL_FLOAT_EQUAL (input[0], params->max[0]) ||                     \n"
-"          (input[0] < params->max[0])) &&                                     \n"
-"          (GEGL_FLOAT_EQUAL (input[1], params->min[1]) ||                     \n"
-"          (input[1] > params->min[1])) &&                                     \n"
-"          (GEGL_FLOAT_EQUAL (input[1], params->max[1]) ||                     \n"
-"          (input[1] < params->max[1])) &&                                     \n"
-"          (GEGL_FLOAT_EQUAL (input[2], params->min[2]) ||                     \n"
-"          (input[2] > params->min[2])) &&                                     \n"
-"          (GEGL_FLOAT_EQUAL (input[2], params->max[2]) ||                     \n"
-"          (input[2] < params->max[2])))                                       \n"
+"      if (input[0] > params->min[0] && input[0] < params->max[0] &&           \n"
+"          input[1] > params->min[1] && input[1] < params->max[1] &&           \n"
+"          input[2] > params->min[2] && input[2] < params->max[2])             \n"
 "        {                                                                     \n"
-"          gfloat delta[3];                                                    \n"
-"                                                                              \n"
 "          for (chan = 0; chan < 3 ; chan++)                                   \n"
-"            {                                                                 \n"
-"              delta[chan] = input[chan] > params->color_in[chan] ?            \n"
-"                input[chan] - params->color_in[chan] :                        \n"
-"                params->color_in[chan] - input[chan];                         \n"
-"                                                                              \n"
-"              output[chan] = CLAMP (params->color_out[chan] + delta[chan],    \n"
+"              output[chan] = CLAMP (input[chan] + params->color_diff[chan],   \n"
 "                                    0.0, 1.0);                                \n"
-"            }                                                                 \n"
 "        }                                                                     \n"
 "      else                                                                    \n"
 "        {                                                                     \n"
-"          output[0] = input[0];                                               \n"
-"          output[1] = input[1];                                               \n"
-"          output[2] = input[2];                                               \n"
+"          for (chan = 0; chan < 3 ; chan++)                                   \n"
+"              output[chan] = input[chan];                                     \n"
 "        }                                                                     \n"
 "                                                                              \n"
 "      output[3] = input[3];                                                   \n"
@@ -170,23 +162,82 @@ static const char* op_c_source =
 "  return TRUE;                                                                \n"
 "}                                                                             \n"
 "                                                                              \n"
+"#include \"opencl/gegl-cl.h\"                                                 \n"
+"#include \"opencl/color-exchange.cl.h\"                                       \n"
+"                                                                              \n"
+"static GeglClRunData *cl_data = NULL;                                         \n"
+"                                                                              \n"
+"static gboolean                                                               \n"
+"cl_process (GeglOperation       *operation,                                   \n"
+"            cl_mem              in,                                           \n"
+"            cl_mem              out,                                          \n"
+"            size_t              global_worksize,                              \n"
+"            const GeglRectangle *roi,                                         \n"
+"            gint                level)                                        \n"
+"{                                                                             \n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  CeParamsType   *params = (CeParamsType*) o->user_data;                      \n"
+"  cl_float3   color_diff;                                                     \n"
+"  cl_float3   min;                                                            \n"
+"  cl_float3   max;                                                            \n"
+"  cl_int      cl_err = 0;                                                     \n"
+"  gint        i;                                                              \n"
+"                                                                              \n"
+"  if (!cl_data)                                                               \n"
+"    {                                                                         \n"
+"      const char *kernel_name[] = {\"cl_color_exchange\",                     \n"
+"                                   NULL};                                     \n"
+"      cl_data = gegl_cl_compile_and_build (color_exchange_cl_source, kernel_name);\n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  if (!cl_data)                                                               \n"
+"    return TRUE;                                                              \n"
+"                                                                              \n"
+"  for (i = 0; i < 3; i++)                                                     \n"
+"    {                                                                         \n"
+"      color_diff.s[i] = params->color_diff[i];                                \n"
+"      min.s[i] = params->min[i];                                              \n"
+"      max.s[i] = params->max[i];                                              \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  cl_err = gegl_cl_set_kernel_args (cl_data->kernel[0],                       \n"
+"                                    sizeof(cl_mem),    &in,                   \n"
+"                                    sizeof(cl_mem),    &out,                  \n"
+"                                    sizeof(cl_float3), &color_diff,           \n"
+"                                    sizeof(cl_float3), &min,                  \n"
+"                                    sizeof(cl_float3), &max,                  \n"
+"                                    NULL);                                    \n"
+"  CL_CHECK;                                                                   \n"
+"                                                                              \n"
+"  cl_err = gegl_clEnqueueNDRangeKernel (gegl_cl_get_command_queue (),         \n"
+"                                        cl_data->kernel[0], 1,                \n"
+"                                        NULL, &global_worksize, NULL,         \n"
+"                                        0, NULL, NULL);                       \n"
+"  CL_CHECK;                                                                   \n"
+"                                                                              \n"
+"  return  FALSE;                                                              \n"
+"                                                                              \n"
+"error:                                                                        \n"
+"  return TRUE;                                                                \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
 "  GObjectClass                  *object_class;                                \n"
 "  GeglOperationClass            *operation_class;                             \n"
-"  GeglOperationPointFilterClass *filter_class;                                \n"
+"  GeglOperationPointFilterClass *point_filter_class;                          \n"
 "                                                                              \n"
-"  object_class    = G_OBJECT_CLASS (klass);                                   \n"
-"  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
-"  filter_class    = GEGL_OPERATION_POINT_FILTER_CLASS (klass);                \n"
+"  object_class       = G_OBJECT_CLASS (klass);                                \n"
+"  operation_class    = GEGL_OPERATION_CLASS (klass);                          \n"
+"  point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);             \n"
 "                                                                              \n"
 "  object_class->finalize = finalize;                                          \n"
 "                                                                              \n"
-"  operation_class->prepare = prepare;                                         \n"
-"  operation_class->opencl_support = FALSE;                                    \n"
+"  operation_class->prepare     = prepare;                                     \n"
 "                                                                              \n"
-"  filter_class->process    = process;                                         \n"
+"  point_filter_class->process    = process;                                   \n"
+"  point_filter_class->cl_process = cl_process;                                \n"
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
 "    \"name\",        \"gegl:color-exchange\",                                 \n"
