@@ -22,17 +22,26 @@
 #define LOWEST_TEMPERATURE     1000
 #define HIGHEST_TEMPERATURE   12000
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (original_temperature, "Original temperature", LOWEST_TEMPERATURE, HIGHEST_TEMPERATURE, 6500, "Estimated temperature of the light source in Kelvin the image was taken with.")
-gegl_chant_double (intended_temperature, "Intended temperature", LOWEST_TEMPERATURE, HIGHEST_TEMPERATURE, 6500, "Corrected estimation of the temperature of the light source in Kelvin.")
+property_double (original_temperature, _("Original temperature"), 6500)
+  description(_("Estimated temperature of the light source in Kelvin "
+                "the image was taken with."))
+  value_range (LOWEST_TEMPERATURE, HIGHEST_TEMPERATURE)
+  ui_meta     ("unit", "kelvin")
+
+property_double (intended_temperature, _("Intended temperature"), 6500)
+  description(_("Corrected estimation of the temperature of the light "
+                "source in Kelvin."))
+  value_range (LOWEST_TEMPERATURE, HIGHEST_TEMPERATURE)
+  ui_meta     ("unit", "kelvin")
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_FILTER
-#define GEGL_CHANT_C_FILE       "color-temperature.c"
+#define GEGL_OP_POINT_FILTER
+#define GEGL_OP_C_SOURCE color-temperature.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 static const gfloat rgb_r55[3][12];
 
@@ -73,7 +82,7 @@ convert_k_to_rgb (gfloat  temperature,
 
 static void prepare (GeglOperation *operation)
 {
-  Babl *format = babl_format ("RGBA float");
+  const Babl *format = babl_format ("RGBA float");
   gegl_operation_set_format (operation, "input", format);
   gegl_operation_set_format (operation, "output", format);
 }
@@ -81,15 +90,15 @@ static void prepare (GeglOperation *operation)
 static void
 finalize (GObject *object)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (object);
+  GeglProperties *o = GEGL_PROPERTIES (object);
 
-  if (o->chant_data)
+  if (o->user_data)
     {
-      g_free (o->chant_data);
-      o->chant_data = NULL;
+      g_free (o->user_data);
+      o->user_data = NULL;
     }
 
-  G_OBJECT_CLASS (gegl_chant_parent_class)->finalize (object);
+  G_OBJECT_CLASS (gegl_op_parent_class)->finalize (object);
 }
 
 static void
@@ -99,25 +108,25 @@ notify (GObject    *object,
   if (strcmp (pspec->name, "original-temperature") == 0 ||
       strcmp (pspec->name, "intended-temperature") == 0)
     {
-      GeglChantO *o = GEGL_CHANT_PROPERTIES (object);
+      GeglProperties *o = GEGL_PROPERTIES (object);
 
       /* one of the properties has changed,
        * invalidate the preprocessed coefficients
        */
-      if (o->chant_data)
+      if (o->user_data)
         {
-          g_free (o->chant_data);
-          o->chant_data = NULL;
+          g_free (o->user_data);
+          o->user_data = NULL;
         }
     }
 
-  if (G_OBJECT_CLASS (gegl_chant_parent_class)->notify)
-    G_OBJECT_CLASS (gegl_chant_parent_class)->notify (object, pspec);
+  if (G_OBJECT_CLASS (gegl_op_parent_class)->notify)
+    G_OBJECT_CLASS (gegl_op_parent_class)->notify (object, pspec);
 }
 
 
 static gfloat *
-preprocess (GeglChantO *o)
+preprocess (GeglProperties *o)
 {
   gfloat *coeffs = g_new (gfloat, 3);
   gfloat  original_temperature_rgb[3];
@@ -141,19 +150,20 @@ process (GeglOperation       *op,
          void                *in_buf,
          void                *out_buf,
          glong                n_pixels,
-         const GeglRectangle *roi)
+         const GeglRectangle *roi,
+         gint                 level)
 {
-  GeglChantO   *o         = GEGL_CHANT_PROPERTIES (op);
+  GeglProperties   *o         = GEGL_PROPERTIES (op);
   gfloat       *in_pixel  = in_buf;
   gfloat       *out_pixel = out_buf;
-  const gfloat *coeffs    = o->chant_data;
+  const gfloat *coeffs    = o->user_data;
 
   in_pixel = in_buf;
   out_pixel = out_buf;
 
   if (! coeffs)
     {
-      coeffs = o->chant_data = preprocess (o);
+      coeffs = o->user_data = preprocess (o);
     }
 
   while (n_pixels--)
@@ -170,13 +180,86 @@ process (GeglOperation       *op,
   return TRUE;
 }
 
+#include "opencl/gegl-cl.h"
+#include "opencl/color-temperature.cl.h"
+
+static GeglClRunData *cl_data = NULL;
+
+/* OpenCL processing function */
+static gboolean
+cl_process (GeglOperation       *op,
+            cl_mem               in_tex,
+            cl_mem               out_tex,
+            size_t               global_worksize,
+            const GeglRectangle *roi,
+            int                  level)
+{
+  /* Retrieve a pointer to GeglProperties structure which contains all the
+   * chanted properties
+   */
+
+  GeglProperties   *o         = GEGL_PROPERTIES (op);
+  const gfloat *coeffs    = o->user_data;
+
+  cl_int cl_err = 0;
+
+  if (! coeffs)
+    {
+      coeffs = o->user_data = preprocess (o);
+    }
+
+  if (!cl_data)
+    {
+      const char *kernel_name[] = {"gegl_color_temperature", NULL};
+      cl_data = gegl_cl_compile_and_build (color_temperature_cl_source, kernel_name);
+    }
+
+  if (!cl_data) return 1;
+
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 0, sizeof(cl_mem),    (void*)&in_tex);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 1, sizeof(cl_mem),    (void*)&out_tex);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 2, sizeof(cl_float),  (void*)&coeffs[0]);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 3, sizeof(cl_float),  (void*)&coeffs[1]);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 4, sizeof(cl_float),  (void*)&coeffs[2]);
+  CL_CHECK;
+
+  cl_err = gegl_clEnqueueNDRangeKernel(gegl_cl_get_command_queue (),
+                                        cl_data->kernel[0], 1,
+                                        NULL, &global_worksize, NULL,
+                                        0, NULL, NULL);
+  CL_CHECK;
+
+  return FALSE;
+
+error:
+  return TRUE;
+}
+
+
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GObjectClass                  *object_class;
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
+  gchar                         *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:color-temperature'>"
+    "  <params>"
+    "    <param name='intended-temperature'>12000</param>"
+    "  </params>"
+    "</node>"
+    "<node operation='gegl:load'>"
+    "  <params>"
+    "    <param name='path'>standard-input.png</param>"
+    "  </params>"
+    "</node>"
+    "</gegl>";
 
   object_class       = G_OBJECT_CLASS (klass);
   operation_class    = GEGL_OPERATION_CLASS (klass);
@@ -187,12 +270,18 @@ gegl_chant_class_init (GeglChantClass *klass)
 
   operation_class->prepare = prepare;
 
-  point_filter_class->process = process;
+  point_filter_class->process    = process;
+  point_filter_class->cl_process = cl_process;
 
-  operation_class->name        = "gegl:color-temperature";
-  operation_class->categories  = "color";
-  operation_class->description =
-        _("Allows changing the color temperature of an image.");
+  operation_class->opencl_support = TRUE;
+
+  gegl_operation_class_set_keys (operation_class,
+    "name",        "gegl:color-temperature",
+    "title",       _("Color Temperature"),
+    "categories",  "color",
+    "description", _("Change the color temperature of the image, from an assumed original color temperature to an intended one."),
+    "reference-composition", composition,
+    NULL);
 }
 
 /* Coefficients of rational functions of degree 5 fitted per color channel to
