@@ -232,7 +232,7 @@ static const char* op_c_source =
 "  GeglSampler        *in_sampler;                                             \n"
 "                                                                              \n"
 "  gint     x, y;                                                              \n"
-"  gdouble  cx, cy;                                                            \n"
+"  gdouble  cx = 0, cy = 0;                                                    \n"
 "  gfloat  *in_pixel;                                                          \n"
 "  gint     n_components;                                                      \n"
 "  gint     aux_index, aux2_index;                                             \n"
@@ -259,10 +259,10 @@ static const char* op_c_source =
 "                                           GEGL_ACCESS_READ, o->abyss_policy);\n"
 "                                                                              \n"
 "  if (o->displace_mode == GEGL_DISPLACE_MODE_POLAR)                           \n"
-"  {                                                                           \n"
+"    {                                                                         \n"
 "      cx = gegl_buffer_get_width (input) / 2.0;                               \n"
 "      cy = gegl_buffer_get_height (input) / 2.0;                              \n"
-"  }                                                                           \n"
+"    }                                                                         \n"
 "                                                                              \n"
 "  while (gegl_buffer_iterator_next (iter))                                    \n"
 "    {                                                                         \n"
