@@ -20,16 +20,18 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string(exposures, _("Exposure Values"), "",
-                  _("Relative brightness of each exposure in EV"))
-gegl_chant_int   (steps, _("Discretization Bits"),
-                  8, 32, 12,
-                  _("Log2 of source's discretization steps"))
-gegl_chant_double (sigma, _("Weight Sigma"),
-                  0.0f, 32.0f, 8.0f,
-                  _("Weight distribution sigma controlling response contributions"))
+property_string(exposures, _("Exposure values"), "")
+    description (_("Relative brightness of each exposure in EV"))
+
+property_int (steps, _("Discretization bits"), 13)
+    description (_("Log2 of source's discretization steps"))
+    value_range (8, 32)
+
+property_double (sigma, _("Weight sigma"), 8.0f)
+    description (_("Weight distribution sigma controlling response contributions"))
+    value_range (0.0f, 32.0f)
 
 #else
 
@@ -37,7 +39,7 @@ gegl_chant_double (sigma, _("Weight Sigma"),
 /*#define DEBUG_SAVE_CURVES*/
 
 #include <gegl-plugin.h>
-struct _GeglChant
+struct _GeglOp
 {
   GeglOperationFilter parent_instance;
   gpointer            properties;
@@ -46,11 +48,11 @@ struct _GeglChant
 typedef struct
 {
   GeglOperationFilterClass parent_class;
-} GeglChantClass;
+} GeglOpClass;
 
 
-#define GEGL_CHANT_C_FILE "exp-combine.c"
-#include "gegl-chant.h"
+#define GEGL_OP_C_SOURCE exp-combine.c
+#include "gegl-op.h"
 GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_FILTER)
 
 #include <errno.h>
@@ -58,21 +60,13 @@ GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_FILTER)
 #include <stdio.h>
 
 #include "gegl-debug.h"
-#include "graph/gegl-node.h"
+#include "graph/gegl-node-private.h"
 #include "graph/gegl-pad.h"
 
 
 static const gchar *PAD_FORMAT = "R'G'B' float";
 static const gchar *EXP_PREFIX = "exposure-";
 
-enum
-{
-  PROP_OUTPUT = 1,
-  PROP_INPUT,
-  PROP_AUX
-};
-
-
 /* maximum iterations after algorithm accepts local minima */
 static const guint MAXIT = 500;
 
@@ -786,35 +780,33 @@ gegl_expcombine_is_exposure_pad (GeglPad *p)
 static void
 gegl_expcombine_attach (GeglOperation *operation)
 {
-  GObjectClass  *object_class = G_OBJECT_GET_CLASS (operation);
-  gchar padname[16];
-  guint i;
+  GParamSpec  *pspec;
+  gchar       padname[16];
+  guint       i;
+
+  pspec = g_param_spec_object ("output",
+                               "output",
+                               "Output buffer",
+                               GEGL_TYPE_BUFFER,
+                               G_PARAM_READWRITE |
+                               GEGL_PARAM_PAD_OUTPUT);
+
+  gegl_operation_create_pad (operation, pspec);
+  g_param_spec_sink (pspec);
 
-  g_object_class_install_property (G_OBJECT_GET_CLASS (operation),
-                                   PROP_INPUT,
-                                   g_param_spec_object ("output",
-                                                        "output",
-                                                        "Output buffer",
-                                                        GEGL_TYPE_BUFFER,
-                                                        G_PARAM_READWRITE |
-                                                        GEGL_PARAM_PAD_OUTPUT));
-  gegl_operation_create_pad (operation,
-                             g_object_class_find_property (object_class,
-                                                           "output"));
   for (i = 0; i <= 99; ++i)
     {
       snprintf (padname, G_N_ELEMENTS (padname), "exposure_%u", i);
-      g_object_class_install_property (G_OBJECT_GET_CLASS (operation),
-                                       PROP_INPUT,
-                                       g_param_spec_object (padname,
-                                                            padname,
-                                                            "Exposure input.",
-                                                            GEGL_TYPE_BUFFER,
-                                                            G_PARAM_READWRITE |
-                                                            GEGL_PARAM_PAD_INPUT));
-      gegl_operation_create_pad (operation,
-                                 g_object_class_find_property (object_class,
-                                                               padname));
+
+      pspec = g_param_spec_object (padname,
+                                   padname,
+                                   "Exposure input.",
+                                   GEGL_TYPE_BUFFER,
+                                   G_PARAM_READWRITE |
+                                   GEGL_PARAM_PAD_INPUT);
+
+      gegl_operation_create_pad (operation, pspec);
+      g_param_spec_sink (pspec);
     }
 }
 
@@ -825,24 +817,12 @@ gegl_expcombine_prepare (GeglOperation *operation)
 {
   GSList *inputs = gegl_node_get_input_pads (operation->node);
 
-  /* Set all the pads output formats, and ensure there is an appropriate node
-   * property for each */
+  /* Set all the pads output formats */
   for (; inputs; inputs = inputs->next)
     {
-      GeglPad     *pad     = inputs->data;
-      const gchar *padname = gegl_pad_get_name (pad);
+      GeglPad *pad = inputs->data;
 
       gegl_pad_set_format (pad, babl_format (PAD_FORMAT));
-
-      /* Create properties for all pads that don't have them, allows arbitrary
-       * numbers of pads.
-       */
-      if (g_object_class_find_property (G_OBJECT_GET_CLASS (operation),
-                                        padname))
-          continue;
-
-      g_warning ("Could not find property for pad '%s'",
-                 gegl_pad_get_name (pad));
     }
 
   gegl_operation_set_format (operation, "output", babl_format (PAD_FORMAT));
@@ -890,7 +870,7 @@ gegl_expcombine_get_exposures (GeglOperation        *operation,
                                const GeglRectangle  *full_roi,
                                GeglRectangle        *scaled_roi)
 {
-  GeglChantO    *o          = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties    *o          = GEGL_PROPERTIES (operation);
   gchar         *ev_cursor  = o->exposures;
   GSList        *exposures  = NULL,
                 *inputs,
@@ -1063,7 +1043,7 @@ gegl_expcombine_process (GeglOperation        *operation,
                          const GeglRectangle  *full_roi,
                          gint                  level)
 {
-  GeglChantO *o           = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o           = GEGL_PROPERTIES (operation);
   GeglBuffer *output      = gegl_operation_context_get_target (context,
                                                                output_pad);
 
@@ -1204,7 +1184,6 @@ gegl_expcombine_process (GeglOperation        *operation,
   /* Save the HDR components to the output buffer. */
   gegl_buffer_set (output, full_roi, 0, babl_format (PAD_FORMAT), hdr,
                    GEGL_AUTO_ROWSTRIDE);
-  gegl_cache_computed (gegl_node_get_cache (operation->node), full_roi);
 
   /* Cleanup */
   g_free (hdr);
@@ -1272,7 +1251,7 @@ gegl_expcombine_get_required_for_output (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
 
@@ -1288,9 +1267,10 @@ gegl_chant_class_init (GeglChantClass *klass)
 
   gegl_operation_class_set_keys (operation_class,
   "name"       , "gegl:exp-combine",
+  "title",       _("Combine Exposures"),
   "categories" , "compositors",
   "description",
-      _("Combine multiple scene exposures into one high range buffer"),
+      _("Combine multiple scene exposures into one high dynamic range image."),
       NULL);
 }
 
