@@ -85,14 +85,17 @@ static const char* op_c_source =
 "                          gdouble *b,                                         \n"
 "                          gdouble (*m)[3])                                    \n"
 "{                                                                             \n"
+"  const gdouble K1 = 2.44413;                                                 \n"
+"  const gdouble K2 = 1.4281;                                                  \n"
+"  const gdouble K3 = 0.422205;                                                \n"
 "  const gdouble q = sigma >= 2.5 ?                                            \n"
 "                    0.98711 * sigma - 0.96330 :                               \n"
 "                    3.97156 - 4.14554 * sqrt (1 - 0.26891 * sigma);           \n"
 "                                                                              \n"
-"  const gdouble b0 = 1.57825 + q*(2.44413 + q*(1.4281  + q*0.422205));        \n"
-"  const gdouble b1 =           q*(2.44413 + q*(2.85619 + q*1.26661));         \n"
-"  const gdouble b2 =         - q*           q*(1.4281  + q*1.26661);          \n"
-"  const gdouble b3 =           q*           q*           q*0.422205;          \n"
+"  const gdouble b0 = 1.57825 + q*(K1 + q*(    K2 + q *     K3));              \n"
+"  const gdouble b1 =           q*(K1 + q*(2 * K2 + q * 3 * K3));              \n"
+"  const gdouble b2 =         - q*      q*(    K2 + q * 3 * K3);               \n"
+"  const gdouble b3 =           q*      q*          q *     K3;                \n"
 "                                                                              \n"
 "  const gdouble a1 = b1 / b0;                                                 \n"
 "  const gdouble a2 = b2 / b0;                                                 \n"
