@@ -19,16 +19,17 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_file_path (path, _("File"), "/tmp/gegl-buffer.gegl", _("Target file path to write GeglBuffer to."))
+property_file_path (path, _("File"), "/tmp/gegl-buffer.gegl")
+  description (_("Target file path to write GeglBuffer to."))
 
 #else
 
-#define GEGL_CHANT_TYPE_SINK
-#define GEGL_CHANT_C_FILE       "gegl-buffer-save-op.c"
+#define GEGL_OP_SINK
+#define GEGL_OP_C_SOURCE gegl-buffer-save-op.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 
 static gboolean
@@ -37,7 +38,7 @@ gegl_buffer_save_op_process (GeglOperation       *operation,
                              const GeglRectangle *result,
                              gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   gegl_buffer_save (input, o->path, result);
 
@@ -45,7 +46,7 @@ gegl_buffer_save_op_process (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass     *operation_class;
   GeglOperationSinkClass *sink_class;
