@@ -25,25 +25,34 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string(composite_op, _("Operation"), "gegl:over",
-                  _("Composite operation to use"))
-gegl_chant_double(opacity, _("Opacity"), 0.0, 1.0, 1.0,
-                  _("Opacity"))
-gegl_chant_double(x, _("X"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-                  _("Horizontal position"))
-gegl_chant_double(y, _("Y"), -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
-                  _("Vertical position"))
-gegl_chant_double(scale, _("scale"), -G_MAXDOUBLE, G_MAXDOUBLE, 1.0,
-                  _("Scale 1:1 size"))
-gegl_chant_file_path(src, _("Source"), "",
-                _("Source datafile (png, jpg, raw, svg, bmp, tif, ...)"))
+property_string(composite_op, _("Operation"), "gegl:over")
+    description (_("Composite operation to use"))
+
+property_double(opacity, _("Opacity"), 1.0)
+    value_range (0.0, 1.0)
+
+property_double(x, _("X"), 0.0)
+    description (_("Horizontal position in pixels"))
+    ui_meta     ("axis", "x")
+    ui_meta     ("unit", "pixel-coordinate")
+
+property_double(y, _("Y"), 0.0)
+    description (_("Vertical position in pixels"))
+    ui_meta     ("axis", "y")
+    ui_meta     ("unit", "pixel-coordinate")
+
+property_double(scale, _("Scale"), 1.0)
+    description (_("Scale 1:1 size"))
+
+property_file_path(src, _("Source"), "")
+    description (_("Source image file path (png, jpg, raw, svg, bmp, tif, ...)"))
 
 #else
 
 #include <gegl-plugin.h>
-struct _GeglChant
+struct _GeglOp
 {
   GeglOperationMeta parent_instance;
   gpointer          properties;
@@ -71,19 +80,19 @@ struct _GeglChant
 typedef struct
 {
   GeglOperationMetaClass parent_class;
-} GeglChantClass;
+} GeglOpClass;
 
-#define GEGL_CHANT_C_FILE "layer.c"
-#include "gegl-chant.h"
+#define GEGL_OP_C_SOURCE layer.c
+#include "gegl-op.h"
 GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_META)
 
 #include <glib/gprintf.h>
 
 static void
-prepare (GeglOperation *operation)
+do_setup (GeglOperation *operation)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  GeglChant *self = GEGL_CHANT (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  GeglOp         *self = GEGL_OP (operation);
 
   /* If the src is NULL, and we previously used a source, clear what we have
    * cached and directly link the input and output. We don't need a composite
@@ -94,18 +103,13 @@ prepare (GeglOperation *operation)
       if (self->cached_path != NULL)
         {
           gegl_node_link (self->input, self->output);
-          g_free(self->cached_path);
+          g_free (self->cached_path);
           self->cached_path = NULL;
         }
 
       return;
     }
 
-  /* warning: this might trigger regeneration of the graph,
-   *          for now this is evaded by just ignoring additional
-   *          requests to be made into members of the graph
-   */
-
   /* Check if the composite operation we're using has changed from that which
    * is already in use.
    */
@@ -119,47 +123,19 @@ prepare (GeglOperation *operation)
       self->p_composite_op = g_strdup (o->composite_op);
     }
 
-  g_assert(o->src && o->src[0]);
-
   /* Load a src image, and relink the input/composite/output chain, as it
    * will currently be set to an input/output chain without a composite
    * source.
    */
 
-  /* FIXME:
-   * this reimplements the "load" op, which shouldn't be neccesary, but
-   * currently seems to be neccesary since GEGL doesn't like a meta-op
-   * to be implemented using another meta-op.
-   */
   if (self->cached_path == NULL || strcmp (o->src, self->cached_path))
     {
-      const gchar *extension = strrchr (o->src, '.');
-      const gchar *handler = NULL;
-
-      if (!g_file_test (o->src, G_FILE_TEST_EXISTS))
-      {
-        gchar *name = g_filename_display_name (o->src);
-        gchar *tmp  = g_strdup_printf ("File '%s' does not exist", name);
-        g_free (name);
-        g_warning ("load: %s", tmp);
-        gegl_node_set (self->load,
-            "operation", "gegl:text",
-            "size", 12.0,
-            "string", tmp,
-            NULL);
-        g_free (tmp);
-      }
-      else
-      {
-        if (extension)
-          handler = gegl_extension_handler_get (extension);
-        gegl_node_set (self->load,
-            "operation", handler,
-            NULL);
-        gegl_node_set (self->load,
-            "path",  o->src,
-            NULL);
-      }
+      gegl_node_set (self->load,
+          "operation", "gegl:load",
+          NULL);
+      gegl_node_set (self->load,
+          "path",  o->src,
+          NULL);
 
       /* Currently not using the composite op, reinsert it */
       if (!self->cached_path)
@@ -201,10 +177,27 @@ prepare (GeglOperation *operation)
 
 }
 
+static void
+my_set_property (GObject      *gobject,
+                 guint         property_id,
+                 const GValue *value,
+                 GParamSpec   *pspec)
+{
+  GeglOperation *operation = GEGL_OPERATION (gobject);
+  GeglOp        *self      = GEGL_OP (operation);
+
+  /* The set_property provided by the chant system does the
+   * storing and reffing/unreffing of the input properties */
+  set_property(gobject, property_id, value, pspec);
+
+  if (self->load)
+    do_setup (operation);
+}
+
 static void attach (GeglOperation *operation)
 {
-  GeglChant  *self = GEGL_CHANT (operation);
-  GeglChantO *o    = GEGL_CHANT_PROPERTIES (operation);
+  GeglOp         *self = GEGL_OP (operation);
+  GeglProperties *o    = GEGL_PROPERTIES (operation);
   GeglNode *gegl;
 
   self->self = GEGL_OPERATION (self)->node;
@@ -219,7 +212,7 @@ static void attach (GeglOperation *operation)
                                          NULL);
 
   self->translate = gegl_node_new_child (gegl, "operation", "gegl:translate", NULL);
-  self->scale = gegl_node_new_child (gegl, "operation", "gegl:scale", NULL);
+  self->scale = gegl_node_new_child (gegl, "operation", "gegl:scale-ratio", NULL);
   self->opacity = gegl_node_new_child (gegl, "operation", "gegl:opacity", NULL);
 
   self->load = gegl_node_new_child (gegl,
@@ -231,24 +224,34 @@ static void attach (GeglOperation *operation)
                        NULL);
   gegl_node_link_many (self->input, self->composite_op, self->output, NULL);
   gegl_node_connect_from (self->composite_op, "aux", self->translate, "output");
+
+  gegl_operation_meta_watch_nodes (operation,
+                                   self->composite_op,
+                                   self->translate,
+                                   self->scale,
+                                   self->opacity,
+                                   self->load,
+                                   NULL);
+
+  do_setup (operation);
 }
 
 
 static void
 finalize (GObject *object)
 {
-  GeglChant *self = GEGL_CHANT (object);
+  GeglOp *self = GEGL_OP (object);
 
   if (self->cached_path)
     g_free (self->cached_path);
   if (self->p_composite_op)
     g_free (self->p_composite_op);
 
-  G_OBJECT_CLASS (gegl_chant_parent_class)->finalize (object);
+  G_OBJECT_CLASS (gegl_op_parent_class)->finalize (object);
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GObjectClass       *object_class;
   GeglOperationClass *operation_class;
@@ -256,14 +259,16 @@ gegl_chant_class_init (GeglChantClass *klass)
   object_class    = G_OBJECT_CLASS (klass);
   operation_class = GEGL_OPERATION_CLASS (klass);
 
-  object_class->finalize = finalize;
+  object_class->finalize     = finalize;
+  object_class->set_property = my_set_property;
 
-  operation_class->name        = "gegl:layer";
-  operation_class->categories  = "meta";
-  operation_class->description = _("A layer in the traditional sense.");
   operation_class->attach = attach;
-  operation_class->prepare = prepare;
+
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:layer",
+    "categories" , "meta",
+    "description", _("A layer in the traditional sense"),
+    NULL);
 }
 
-
 #endif
