@@ -182,13 +182,16 @@ my_set_property (GObject      *gobject,
   GeglOp         *self      = GEGL_OP (operation);
   GeglProperties *o         = GEGL_PROPERTIES (operation);
 
-  const gchar *old_path = g_strdup (o->path);
-  const gchar *old_uri = g_strdup (o->uri);
+  gchar *old_path = g_strdup (o->path);
+  gchar *old_uri = g_strdup (o->uri);
+
+  gboolean props_changed;
 
   /* The set_property provided by the chant system does the
-   * storing and reffing/unreffing of the input properties */
-  set_property(gobject, property_id, value, pspec);
-  const gboolean props_changed = g_strcmp0 (o->path, old_path) || g_strcmp0 (o->uri, old_uri);
+   * storing and reffing/unreffing of the input properties
+   */
+  set_property (gobject, property_id, value, pspec);
+  props_changed = g_strcmp0 (o->path, old_path) || g_strcmp0 (o->uri, old_uri);
 
   if (self->load && props_changed)
     do_setup (operation, o->path, o->uri);
