@@ -183,13 +183,16 @@ static const char* op_c_source =
 "  GeglOp         *self      = GEGL_OP (operation);                            \n"
 "  GeglProperties *o         = GEGL_PROPERTIES (operation);                    \n"
 "                                                                              \n"
-"  const gchar *old_path = g_strdup (o->path);                                 \n"
-"  const gchar *old_uri = g_strdup (o->uri);                                   \n"
+"  gchar *old_path = g_strdup (o->path);                                       \n"
+"  gchar *old_uri = g_strdup (o->uri);                                         \n"
+"                                                                              \n"
+"  gboolean props_changed;                                                     \n"
 "                                                                              \n"
 "  /* The set_property provided by the chant system does the                   \n"
-"   * storing and reffing/unreffing of the input properties */                 \n"
-"  set_property(gobject, property_id, value, pspec);                           \n"
-"  const gboolean props_changed = g_strcmp0 (o->path, old_path) || g_strcmp0 (o->uri, old_uri);\n"
+"   * storing and reffing/unreffing of the input properties                    \n"
+"   */                                                                         \n"
+"  set_property (gobject, property_id, value, pspec);                          \n"
+"  props_changed = g_strcmp0 (o->path, old_path) || g_strcmp0 (o->uri, old_uri);\n"
 "                                                                              \n"
 "  if (self->load && props_changed)                                            \n"
 "    do_setup (operation, o->path, o->uri);                                    \n"
