@@ -26,25 +26,26 @@
 #include <math.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (contrast, _("Contrast"),
-                  0.0, 1.0, 0.1,
-                  _("The amount of contrast compression"))
-gegl_chant_double (saturation, _("Saturation"),
-                  0.0, 2.0, 0.8,
-                  _("Global color saturation factor"))
-gegl_chant_double (detail, _("Detail"),
-                  1.0, 99.0, 1.0,
-                  _("Level of emphasis on image gradient details"))
+property_double (contrast, _("Contrast"), 0.1)
+    description (_("The amount of contrast compression"))
+    value_range (0.0, 1.0)
 
+property_double (saturation, _("Saturation"), 0.8)
+    description (("Global color saturation factor"))
+    value_range (0.0, 2.0)
+
+property_double (detail, _("Detail"), 1.0)
+    description (_("Level of emphasis on image gradient details"))
+    value_range (1.0, 99.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "mantiuk06.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE mantiuk06.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -1581,10 +1582,9 @@ mantiuk06_process (GeglOperation       *operation,
                    const GeglRectangle *result,
                    gint                 level)
 {
-  const GeglChantO *o          = GEGL_CHANT_PROPERTIES (operation);
-  const gint        pix_stride = 4; /* RGBA */
-  gfloat           *lum,
-                   *pix;
+  const GeglProperties *o      = GEGL_PROPERTIES (operation);
+  const gint            pix_stride = 4; /* RGBA */
+  gfloat               *lum, *pix;
 
   g_return_val_if_fail (operation, FALSE);
   g_return_val_if_fail (input, FALSE);
@@ -1618,7 +1618,7 @@ mantiuk06_process (GeglOperation       *operation,
 /*
  */
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -1631,9 +1631,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare                 = mantiuk06_prepare;
   operation_class->get_required_for_output = mantiuk06_get_required_for_output;
   operation_class->get_cached_region       = mantiuk06_get_cached_region;
+  operation_class->threaded                = FALSE;
 
   gegl_operation_class_set_keys (operation_class,
-      "name"       , "gegl:mantiuk06",
+      "name",        "gegl:mantiuk06",
+      "title",       _("Mantiuk 2006 Tonemapping"),
       "categories" , "tonemapping",
       "description",
         _("Adapt an image, which may have a high dynamic range, for "
