@@ -18,22 +18,24 @@
  */
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (scaling, _("Scaling"), 0.0, 5000.0, 1.0,
-       _("scaling factor of displacement, indicates how large spatial"
-         " displacement a relative mapping value of 1.0 corresponds to."))
-gegl_chant_enum (sampler_type, _("Sampler"), GeglSamplerType, gegl_sampler_type,
-                 GEGL_SAMPLER_CUBIC, _("Sampler used internally"))
+property_double (scaling, _("Scaling"), 1.0)
+  description   (_("scaling factor of displacement, indicates how large spatial"
+              " displacement a relative mapping value of 1.0 corresponds to."))
+  value_range (0.0, 5000.0)
+
+property_enum (sampler_type, _("Resampling method"),
+    GeglSamplerType, gegl_sampler_type, GEGL_SAMPLER_CUBIC)
 
 #else
 
-#define GEGL_CHANT_TYPE_COMPOSER
-#define GEGL_CHANT_C_FILE       "map-relative.c"
+#define GEGL_OP_COMPOSER
+#define GEGL_OP_C_SOURCE map-relative.c
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 
 static void
@@ -64,7 +66,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *result,
          gint                 level)
 {
-  GeglChantO           *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties       *o = GEGL_PROPERTIES (operation);
   const Babl           *format_io, *format_coords;
   GeglSampler          *sampler;
   GeglBufferIterator   *it;
@@ -73,15 +75,18 @@ process (GeglOperation       *operation,
   format_io = babl_format ("RGBA float");
   format_coords = babl_format_n (babl_type ("float"), 2);
 
-  sampler = gegl_buffer_sampler_new (input, format_io, o->sampler_type);
+  sampler = gegl_buffer_sampler_new_at_level (input, format_io, o->sampler_type, level);
 
   if (aux != NULL)
     {
-      it = gegl_buffer_iterator_new (output, result, level, format_io, GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
+      it = gegl_buffer_iterator_new (output, result, level, format_io,
+                                     GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
       index_out = 0;
 
-      index_coords = gegl_buffer_iterator_add (it, aux, result, level, format_coords, GEGL_BUFFER_READ, GEGL_ABYSS_NONE);
-      index_in = gegl_buffer_iterator_add (it, input, result, level, format_io, GEGL_BUFFER_READ, GEGL_ABYSS_NONE);
+      index_coords = gegl_buffer_iterator_add (it, aux, result, level, format_coords,
+                                               GEGL_ACCESS_READ, GEGL_ABYSS_NONE);
+      index_in = gegl_buffer_iterator_add (it, input, result, level, format_io,
+                                           GEGL_ACCESS_READ, GEGL_ABYSS_NONE);
 
       while (gegl_buffer_iterator_next (it))
         {
@@ -89,7 +94,7 @@ process (GeglOperation       *operation,
           gint        n_pixels = it->length;
           gint        x = it->roi->x; /* initial x                   */
           gint        y = it->roi->y; /*           and y coordinates */
-          gdouble     scaling = GEGL_CHANT_PROPERTIES (operation)->scaling;
+          gdouble     scaling = GEGL_PROPERTIES (operation)->scaling;
           gfloat     *in = it->data[index_in];
           gfloat     *out = it->data[index_out];
           gfloat     *coords = it->data[index_coords];
@@ -130,7 +135,8 @@ process (GeglOperation       *operation,
     }
   else
     {
-      gegl_buffer_copy (input, result, output, result);
+      gegl_buffer_copy (input, result, GEGL_ABYSS_NONE,
+                        output, result);
     }
 
   g_object_unref (sampler);
@@ -139,27 +145,26 @@ process (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
-  gchar                      *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+  gchar                      *composition = 
     "<gegl>"
-    "<node operation='gegl:map-relative'>"
-    "  <params>"
-    "    <param name='scaling'>100</param>"
-    "  </params>"
-    "  <node operation='gegl:load'>"
-    "    <params><param name='path'>standard-aux.png</param></params>"
-    "  </node>"
-    "</node>"
-    "<node operation='gegl:load'>"
-    "  <params>"
-    "    <param name='path'>standard-input.png</param>"
-    "  </params>"
+    "<node operation='gegl:crop' width='200' height='200'/>"
+    "<node operation='gegl:over'>"
+      "<node operation='gegl:map-relative'>"
+      "  <params>"
+      "    <param name='scaling'>30</param>"
+      "  </params>"
+      "  <node operation='gegl:perlin-noise' />"
+      "</node>"
+      "<node operation='gegl:load' path='standard-input.png'/>"
     "</node>"
+    "<node operation='gegl:checkerboard' color1='rgb(0.25,0.25,0.25)' color2='rgb(0.75,0.75,0.75)'/>"
     "</gegl>";
 
+
   operation_class = GEGL_OPERATION_CLASS (klass);
   composer_class  = GEGL_OPERATION_COMPOSER_CLASS (klass);
 
@@ -168,8 +173,9 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_required_for_output = get_required_for_output;
 
   gegl_operation_class_set_keys (operation_class,
-    "name"       , "gegl:map-relative",
-    "categories" , "transform",
+    "name",        "gegl:map-relative",
+    "title",       _("Map Relative"),
+    "categories" , "map",
     "description", _("sample input with an auxiliary buffer that contain relative source coordinates"),
     "reference-composition", composition,
     NULL);
