@@ -25,41 +25,72 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (m_angle, _("Mirror rotation"), 0.0, 180.0, 0.0, _("Rotation applied to the mirrors"))
+property_double (m_angle, _("Mirror rotation"), 0.0)
+    description (_("Rotation applied to the mirrors"))
+    value_range (0.0, 180.0)
+    ui_meta     ("unit", "degree")
 
-gegl_chant_double (r_angle, _("Result rotation"), 0.0, 360.0, 0.0, _("Rotation applied to the result"))
+property_double (r_angle, _("Result rotation"), 0.0)
+    description (_("Rotation applied to the result"))
+    value_range (0.0, 360.0)
+    ui_meta     ("unit", "degree")
 
-gegl_chant_int    (n_segs, _("Mirrors"), 2, 24, 6, _("Number of mirrors to use"))
+property_int    (n_segs, _("Mirrors"), 6)
+    description (_("Number of mirrors to use"))
+    value_range (2, 24)
 
-gegl_chant_double (c_x,  _("X offset"), 0.0, 1.0, 0.5, _("X offset of the result of mirroring"))
+property_double (c_x, _("Offset X"), 0.5)
+    description (_("position of symmetry center in output"))
+    value_range (0.0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
+    ui_meta     ("axis", "x")
 
-gegl_chant_double (c_y,  _("Y offset"), 0.0, 1.0, 0.5, _("Y offset of the result of mirroring"))
+property_double (c_y, _("Offset Y"), 0.5)
+    description (_("position of symmetry center in output"))
+    value_range (0.0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
+    ui_meta     ("axis", "y")
 
-gegl_chant_double (o_x, _("Center X"), -1.0, 1.0, 0.0, _("X axis ratio for the center of mirroring"))
+property_double (o_x, _("Center X"), 0.0)
+    description (_("X axis ratio for the center of mirroring"))
+    value_range (-1.0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
 
-gegl_chant_double (o_y, _("Center Y"), -1.0, 1.0, 0.0, _("Y axis ratio for the center of mirroring"))
+property_double (o_y, _("Center Y"), 0.0)
+    description (_("Y axis ratio for the center of mirroring"))
+    value_range (-1.0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
 
-gegl_chant_double (trim_x, _("Trim X"), 0.0, 0.5, 0.0, _("X axis ratio for trimming mirror expanse"))
+property_double (trim_x, _("Trim X"), 0.0)
+    description (_("X axis ratio for trimming mirror expanse"))
+    value_range (0.0, 0.5)
 
-gegl_chant_double (trim_y, _("Trim Y"), 0.0, 0.5, 0.0, _("Y axis ratio for trimming mirror expanse"))
+property_double (trim_y, _("Trim Y"), 0.0)
+    description (_("Y axis ratio for trimming mirror expanse"))
+    value_range (0.0, 0.5)
 
-gegl_chant_double (input_scale, _("Zoom"), 0.1, 100.0, 100.0, _("Scale factor to make rendering size bigger"))
+property_double (input_scale, _("Zoom"), 100.0)
+    description (_("Scale factor to make rendering size bigger"))
+    value_range (0.1, 100.0)
 
-gegl_chant_double (output_scale, _("Expand"), 0.0, 100.0, 1.0, _("Scale factor to make rendering size bigger"))
+property_double (output_scale, _("Expand"), 1.0)
+    description (_("Scale factor to make rendering size bigger"))
+    value_range (0.0, 100.0)
 
-gegl_chant_boolean (clip, _("Clip result"), TRUE, _("Clip result to input size"))
+property_boolean (clip, _("Clip result to input size"), TRUE)
 
-gegl_chant_boolean (warp, _("Wrap input"), TRUE, _("Fill full output area"))
+property_boolean (warp, _("Wrap input"), TRUE)
+    description (_("Fill full output area"))
 
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "mirrors.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE mirrors.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 
 #if 0
@@ -120,12 +151,13 @@ apply_mirror (double               mirror_angle,
               double               input_scale,
               gboolean             clip,
               gboolean             warp,
-              Babl                *format,
+              const Babl          *format,
               GeglBuffer          *src,
               GeglRectangle       *in_boundary,
               GeglBuffer          *dst,
               GeglRectangle       *boundary,
-              const GeglRectangle *roi)
+              const GeglRectangle *roi,
+              gint                 level)
 {
   gfloat *dst_buf;
   gint    row, col;
@@ -234,7 +266,7 @@ apply_mirror (double               mirror_angle,
 
 
 #ifndef DO_NOT_USE_BUFFER_SAMPLE
-        gegl_buffer_sample (src, cx, cy, NULL, &dst_buf[(row * roi->width + col) * 4], format, GEGL_SAMPLER_LINEAR);
+        gegl_buffer_sample_at_level (src, cx, cy, NULL, &dst_buf[(row * roi->width + col) * 4], format, level, GEGL_SAMPLER_LINEAR, GEGL_ABYSS_NONE);
 #endif
 
 #ifdef DO_NOT_USE_BUFFER_SAMPLE
@@ -250,9 +282,7 @@ apply_mirror (double               mirror_angle,
     gegl_buffer_sample_cleanup(src);
 
   /* Store dst pixels. */
-  gegl_buffer_set (dst, roi, format, dst_buf, GEGL_AUTO_ROWSTRIDE);
-
-  gegl_buffer_flush(dst);
+  gegl_buffer_set (dst, roi, 0, format, dst_buf, GEGL_AUTO_ROWSTRIDE);
 
   /* Free acquired storage. */
 #ifdef DO_NOT_USE_BUFFER_SAMPLE
@@ -269,7 +299,7 @@ get_effective_area (GeglOperation *operation)
 {
   GeglRectangle  result = {0,0,0,0};
   GeglRectangle *in_rect = gegl_operation_source_get_bounding_box (operation, "input");
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gdouble xt = o->trim_x * in_rect->width;
   gdouble yt = o->trim_y * in_rect->height;
 
@@ -292,7 +322,7 @@ get_bounding_box (GeglOperation *operation)
 {
   GeglRectangle  result = {0,0,0,0};
   GeglRectangle *in_rect = gegl_operation_source_get_bounding_box (operation, "input");
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (!in_rect){
         return result;
@@ -349,12 +379,13 @@ static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  GeglRectangle boundary = gegl_operation_get_bounding_box (operation);
-  GeglRectangle  eff_boundary = get_effective_area (operation);
-  Babl *format = babl_format ("RaGaBaA float");
+  GeglProperties *o            = GEGL_PROPERTIES (operation);
+  GeglRectangle   boundary     = gegl_operation_get_bounding_box (operation);
+  GeglRectangle   eff_boundary = get_effective_area (operation);
+  const Babl     *format       = babl_format ("RaGaBaA float");
 
 #ifdef DO_NOT_USE_BUFFER_SAMPLE
  g_warning ("NOT USING BUFFER SAMPLE!");
@@ -374,13 +405,14 @@ process (GeglOperation       *operation,
                 &eff_boundary,
                 output,
                 &boundary,
-                result);
+                result,
+                level);
   return TRUE;
 }
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -393,10 +425,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
 
-  operation_class->name        = "gegl:mirrors";
-  operation_class->categories  = "blur";
-  operation_class->description =
-        _("Applies mirroring effect on the image.");
+  gegl_operation_class_set_keys (operation_class,
+    "name",               "gegl:mirrors",
+    "title",              _("Kaleidoscopic Mirroring"),
+    "position-dependent", "true",
+    "categories",         "blur",
+    "description",        _("Create a kaleidoscope like effect."),
+    NULL);
 }
 
 #endif
