@@ -20,18 +20,18 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_file_path (path, _("File"), "", _("Path of file to save."))
+property_file_path (path, _("File"), "")
+    description(_("Path of file to save."))
 
 #else
 
-#define GEGL_CHANT_C_FILE       "save.c"
+#define GEGL_OP_C_SOURCE save.c
 
 #include "gegl-plugin.h"
-#include "gegl/graph/gegl-node.h"
 
-struct _GeglChant
+struct _GeglOp
 {
   GeglOperationSink  parent_instance;
   gpointer           properties;
@@ -44,9 +44,9 @@ struct _GeglChant
 typedef struct
 {
   GeglOperationSinkClass parent_class;
-} GeglChantClass;
+} GeglOpClass;
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_SINK)
 
 #include <stdio.h>
@@ -55,8 +55,8 @@ GEGL_DEFINE_DYNAMIC_OPERATION(GEGL_TYPE_OPERATION_SINK)
 static void
 gegl_save_set_saver (GeglOperation *operation)
 {
-  GeglChantO  *o    = GEGL_CHANT_PROPERTIES (operation);
-  GeglChant   *self = GEGL_CHANT (operation);
+  GeglProperties  *o    = GEGL_PROPERTIES (operation);
+  GeglOp   *self = GEGL_OP (operation);
   const gchar *extension, *handler;
 
   /* If prepare has already been invoked, bail out early */
@@ -101,9 +101,9 @@ gegl_save_set_saver (GeglOperation *operation)
 static void
 gegl_save_attach (GeglOperation *operation)
 {
-  GeglChant   *self = GEGL_CHANT (operation);
-  const gchar *nodename;
-  gchar       *childname;
+  GeglOp   *self = GEGL_OP (operation);
+  /* const gchar *nodename; */
+  /* gchar       *childname; */
 
   g_assert (!self->input);
   g_assert (!self->save);
@@ -116,7 +116,7 @@ gegl_save_attach (GeglOperation *operation)
                                       NULL);
 
   /* Set some debug names for the child nodes */
-  nodename = gegl_node_get_debug_name (operation->node);
+  /*  nodename = gegl_node_get_debug_name (operation->node);
   g_assert (nodename);
 
   childname = g_strconcat (nodename, "-save", NULL);
@@ -125,7 +125,7 @@ gegl_save_attach (GeglOperation *operation)
 
   childname = g_strconcat (nodename, "-input", NULL);
   gegl_node_set_name (self->input, childname);
-  g_free (childname);
+  g_free (childname);*/
 
   /* Link the saving node and attempt to set an appropriate save operation,
    * might as well at least try to do this before prepare.
@@ -134,11 +134,21 @@ gegl_save_attach (GeglOperation *operation)
   gegl_save_set_saver (operation);
 }
 
-
 static void
-gegl_save_prepare (GeglOperation *operation)
+gegl_save_set_property (GObject      *gobject,
+                        guint         property_id,
+                        const GValue *value,
+                        GParamSpec   *pspec)
 {
-  gegl_save_set_saver (operation);
+  GeglOperation *operation = GEGL_OPERATION (gobject);
+  GeglOp     *self = GEGL_OP (operation);
+
+  /* The set_property provided by the chant system does the
+   * storing and reffing/unreffing of the input properties */
+  set_property(gobject, property_id, value, pspec);
+
+  if (self->save)
+    gegl_save_set_saver (operation);
 }
 
 
@@ -146,50 +156,52 @@ static gboolean
 gegl_save_process (GeglOperation        *operation,
                    GeglOperationContext *context,
                    const gchar          *output_pad,
-                   const GeglRectangle  *roi)
+                   const GeglRectangle  *roi,
+                   gint                  level)
 {
-  GeglChant *self = GEGL_CHANT (operation);
-  gegl_save_set_saver (operation);
+  GeglOp *self = GEGL_OP (operation);
 
-  return gegl_operation_process (self->save->operation,
+  return gegl_operation_process (gegl_node_get_gegl_operation (self->save),
                                  context,
                                  output_pad,
-                                 roi);
+                                 roi,
+                                 level);
 }
 
 static void
 gegl_save_dispose (GObject *object)
 {
-  GeglChant *self = GEGL_CHANT (object);
+  GeglOp *self = GEGL_OP (object);
 
   g_free (self->cached_path);
   self->cached_path = NULL;
 
-  G_OBJECT_CLASS (gegl_chant_parent_class)->dispose (object);
+  G_OBJECT_CLASS (gegl_op_parent_class)->dispose (object);
 }
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GObjectClass           *object_class    = G_OBJECT_CLASS (klass);
   GeglOperationSinkClass *sink_class      = GEGL_OPERATION_SINK_CLASS (klass);
   GeglOperationClass     *operation_class = GEGL_OPERATION_CLASS (klass);
 
-  object_class->dispose = gegl_save_dispose;
+  object_class->dispose      = gegl_save_dispose;
+  object_class->set_property = gegl_save_set_property;
 
   operation_class->attach  = gegl_save_attach;
-  operation_class->prepare = gegl_save_prepare;
-
-  operation_class->process = gegl_operation_process;
   operation_class->process = gegl_save_process;
 
   sink_class->needs_full = TRUE;
 
-  operation_class->name        = "gegl:save";
-  operation_class->categories  = "meta:output";
-  operation_class->description =
-        _("Multipurpose file saver, that uses other native handlers.");
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:save",
+    "title",       _("Save"),
+    "categories" , "meta:output",
+    "description",
+        _("Multipurpose file saver, that uses other native save handlers depending on extension, use the format specific save ops to specify additional parameters."),
+        NULL);
 }
 
 #endif
