@@ -21,19 +21,25 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (radius, _("Radius"), 0.0, 70.0, 8.0,
-  _("Radius of square pixel region, (width and height will be radius*2+1)"))
-gegl_chant_int (pairs, _("Pairs"), 1, 2, 2,
-  _("Number of pairs; higher number preserves more acute features"))
+property_int (radius, _("Radius"), 8)
+    description(_("Radius of square pixel region, (width and height will be radius*2+1)"))
+    value_range (0, 100)
+    ui_range    (0, 40)
+    ui_gamma    (1.5)
+    ui_meta     ("unit", "pixel-distance")
+
+property_int (pairs, _("Pairs"), 2)
+  description(_("Number of pairs; higher number preserves more acute features"))
+  value_range (1, 2)
 
 #else
 
-#define GEGL_CHANT_TYPE_AREA_FILTER
-#define GEGL_CHANT_C_FILE       "snn-mean.c"
+#define GEGL_OP_AREA_FILTER
+#define GEGL_OP_C_SOURCE snn-mean.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 
 static void
@@ -47,25 +53,35 @@ snn_mean (GeglBuffer          *src,
 static void prepare (GeglOperation *operation)
 {
   GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
-  GeglChantO              *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties          *o    = GEGL_PROPERTIES (operation);
 
   area->left = area->right = area->top = area->bottom = ceil (o->radius);
   gegl_operation_set_format (operation, "input", babl_format ("RGBA float"));
   gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
 }
 
+static gboolean
+cl_process (GeglOperation       *operation,
+            GeglBuffer          *input,
+            GeglBuffer          *output,
+            const GeglRectangle *result);
+
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO          *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties      *o = GEGL_PROPERTIES (operation);
   GeglBuffer          *temp_in;
   GeglRectangle        compute;
 
-  compute  = gegl_operation_get_required_for_output (
-                   operation, "input", result);
+  if (gegl_operation_use_opencl (operation))
+    if (cl_process (operation, input, output, result))
+      return TRUE;
+
+  compute = gegl_operation_get_required_for_output (operation, "input", result);
 
   if (o->radius < 1.0)
     {
@@ -82,17 +98,6 @@ process (GeglOperation       *operation,
   return  TRUE;
 }
 
-#define RGB_LUMINANCE_RED    (0.212671)
-#define RGB_LUMINANCE_GREEN  (0.715160)
-#define RGB_LUMINANCE_BLUE   (0.072169)
-
-static inline gfloat rgb2luminance (gfloat *pix)
-{
-  return pix[0] * RGB_LUMINANCE_RED +
-         pix[1] * RGB_LUMINANCE_GREEN +
-         pix[2] * RGB_LUMINANCE_BLUE;
-}
-
 #define POW2(a)((a)*(a))
 
 static inline gfloat colordiff (gfloat *pixA,
@@ -122,7 +127,8 @@ snn_mean (GeglBuffer          *src,
   src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);
   dst_buf = g_new0 (gfloat, dst_rect->width * dst_rect->height * 4);
 
-  gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (src, NULL, 1.0, babl_format ("RGBA float"), src_buf, 
+                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   offset = 0;
 
@@ -136,7 +142,7 @@ snn_mean (GeglBuffer          *src,
         {
           gint u,v;
 
-          gfloat  accumulated[4]={0,};
+          gfloat  accumulated[4]={0.0f, 0.0f, 0.0f, 0.0f};
           gint    count=0;
 
           /* iterate through the upper left quater of pixels */
@@ -201,30 +207,135 @@ snn_mean (GeglBuffer          *src,
           center_pix += 4;
         }
     }
-  gegl_buffer_set (dst, dst_rect, babl_format ("RGBA float"), dst_buf,
+  gegl_buffer_set (dst, dst_rect, 0, babl_format ("RGBA float"), dst_buf,
                    GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
 
 
+#include "opencl/gegl-cl.h"
+#include "buffer/gegl-buffer-cl-iterator.h"
+
+#include "opencl/snn-mean.cl.h"
+
+static GeglClRunData *cl_data = NULL;
+
+static gboolean
+cl_snn_mean (cl_mem                in_tex,
+             cl_mem                out_tex,
+             const GeglRectangle  *src_rect,
+             const GeglRectangle  *roi,
+             gint                  radius,
+             gint                  pairs)
+{
+  cl_int cl_err = 0;
+  size_t global_ws[2];
+
+  if (!cl_data)
+    {
+      const char *kernel_name[] = {"snn_mean", NULL};
+      cl_data = gegl_cl_compile_and_build (snn_mean_cl_source, kernel_name);
+    }
+  if (!cl_data) return TRUE;
+
+
+  global_ws[0] = roi->width;
+  global_ws[1] = roi->height;
+
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 0, sizeof(cl_mem),   (void*)&in_tex);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 1, sizeof(cl_int),   (void*)&src_rect->width);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 2, sizeof(cl_int),   (void*)&src_rect->height);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 3, sizeof(cl_mem),   (void*)&out_tex);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 4, sizeof(cl_int),   (void*)&radius);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 5, sizeof(cl_int),   (void*)&pairs);
+  CL_CHECK;
+
+  cl_err = gegl_clEnqueueNDRangeKernel(gegl_cl_get_command_queue (),
+                                        cl_data->kernel[0], 2,
+                                        NULL, global_ws, NULL,
+                                        0, NULL, NULL);
+  CL_CHECK;
+
+  return FALSE;
+
+error:
+  return TRUE;
+}
+
+static gboolean
+cl_process (GeglOperation       *operation,
+            GeglBuffer          *input,
+            GeglBuffer          *output,
+            const GeglRectangle *result)
+{
+  const Babl *in_format  = gegl_operation_get_format (operation, "input");
+  const Babl *out_format = gegl_operation_get_format (operation, "output");
+  gint err;
+
+  GeglOperationAreaFilter *op_area = GEGL_OPERATION_AREA_FILTER (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+
+  GeglBufferClIterator *i = gegl_buffer_cl_iterator_new (output,
+                                                         result,
+                                                         out_format,
+                                                         GEGL_CL_BUFFER_WRITE);
+
+  gint read = gegl_buffer_cl_iterator_add_2 (i,
+                                             input,
+                                             result,
+                                             in_format,
+                                             GEGL_CL_BUFFER_READ,
+                                             op_area->left,
+                                             op_area->right,
+                                             op_area->top,
+                                             op_area->bottom,
+                                             GEGL_ABYSS_NONE);
+
+  while (gegl_buffer_cl_iterator_next (i, &err))
+    {
+      if (err) return FALSE;
+
+      err = cl_snn_mean(i->tex[read],
+                        i->tex[0],
+                        &i->roi[read],
+                        &i->roi[0],
+                        ceil(o->radius),
+                        o->pairs);
+
+      if (err) return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
 
-  operation_class  = GEGL_OPERATION_CLASS (klass);
-  filter_class     = GEGL_OPERATION_FILTER_CLASS (klass);
+  operation_class = GEGL_OPERATION_CLASS (klass);
+  filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);
 
-  filter_class->process   = process;
+  filter_class->process    = process;
   operation_class->prepare = prepare;
 
-  operation_class->name        = "gegl:snn-mean";
-  operation_class->categories  = "misc";
-  operation_class->description =
-        _("Noise reducing edge enhancing blur filter based "
-          " on Symmetric Nearest Neighbours");
+  operation_class->opencl_support = TRUE;
+
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:snn-mean",
+    "categories" , "enhance:noise-reduction",
+    "title",       _("Symmetric Nearest Neighbour"),
+    "description",
+        _("Noise reducing edge preserving blur filter based "
+          " on Symmetric Nearest Neighbours"),
+        NULL);
 }
 
 #endif
