@@ -41,7 +41,7 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  GeglBufferIterator *gi;                                                     \n"
 "  gint c;                                                                     \n"
-"  gi = gegl_buffer_iterator_new (buffer, NULL, 0, babl_format (\"R'G'B' float\"),\n"
+"  gi = gegl_buffer_iterator_new (buffer, NULL, 0, babl_format (\"RGB float\"),\n"
 "                                 GEGL_ACCESS_READ, GEGL_ABYSS_NONE);          \n"
 "  for (c = 0; c < 3; c++)                                                     \n"
 "    {                                                                         \n"
@@ -90,8 +90,8 @@ static const char* op_c_source =
 "                                                                              \n"
 "static void prepare (GeglOperation *operation)                                \n"
 "{                                                                             \n"
-"  gegl_operation_set_format (operation, \"input\", babl_format (\"R'G'B'A float\"));\n"
-"  gegl_operation_set_format (operation, \"output\", babl_format (\"R'G'B'A float\"));\n"
+"  gegl_operation_set_format (operation, \"input\", babl_format (\"RGBA float\"));\n"
+"  gegl_operation_set_format (operation, \"output\", babl_format (\"RGBA float\"));\n"
 "}                                                                             \n"
 "                                                                              \n"
 "static GeglRectangle                                                          \n"
@@ -483,10 +483,10 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "    }                                                                         \n"
 "                                                                              \n"
-"  gi = gegl_buffer_iterator_new (input, result, 0, babl_format (\"R'G'B'A float\"),\n"
+"  gi = gegl_buffer_iterator_new (input, result, 0, babl_format (\"RGBA float\"),\n"
 "                                 GEGL_ACCESS_READ, GEGL_ABYSS_NONE);          \n"
 "                                                                              \n"
-"  gegl_buffer_iterator_add (gi, output, result, 0, babl_format (\"R'G'B'A float\"),\n"
+"  gegl_buffer_iterator_add (gi, output, result, 0, babl_format (\"RGBA float\"),\n"
 "                            GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);              \n"
 "                                                                              \n"
 "  while (gegl_buffer_iterator_next (gi))                                      \n"
