@@ -19,66 +19,64 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_register_enum (gegl_vignette_shape)
-  enum_value (GEGl_VIGNETTE_SHAPE_CIRCLE,  "Circle")
-  enum_value (GEGl_VIGNETTE_SHAPE_SQUARE,  "Square")
-  enum_value (GEGl_VIGNETTE_SHAPE_DIAMOND, "Diamond")
-gegl_chant_register_enum_end (GeglVignetteShape)
+enum_start (gegl_vignette_shape)
+  enum_value (GEGL_VIGNETTE_SHAPE_CIRCLE,  "circle",  N_("Circle"))
+  enum_value (GEGL_VIGNETTE_SHAPE_SQUARE,  "square",  N_("Square"))
+  enum_value (GEGL_VIGNETTE_SHAPE_DIAMOND, "diamond", N_("Diamond"))
+enum_end (GeglVignetteShape)
 
-gegl_chant_enum   (shape, _("Shape"),
-                   GeglVignetteShape, gegl_vignette_shape,
-                   GEGl_VIGNETTE_SHAPE_CIRCLE,
-                   _("Shape of the vignette"))
+property_enum (shape, _("Vignette shape"),
+    GeglVignetteShape, gegl_vignette_shape,
+    GEGL_VIGNETTE_SHAPE_CIRCLE)
 
-gegl_chant_color  (color, _("Color"),
-                   "black",
-                   _("Defaults to 'black', you can use transparency here "
-                     "to erase portions of an image"))
+property_color (color, _("Color"), "black")
+    description (_("Defaults to 'black', you can use transparency here to erase portions of an image"))
 
-gegl_chant_double (radius, _("Radius"),
-                   0.0, 3.0, 1.5,
-                   _("How far out vignetting goes as portion of half "
-                     "image diagonal"))
+property_double (radius, _("Radius"), 1.2)
+    description (_("How far out vignetting goes as portion of half image diagonal"))
+    value_range (0.0, 3.0)
+    ui_meta     ("unit", "relative-distance")
 
-gegl_chant_double (softness, _("Softness"),
-                   0.0, 1.0, 0.8,
-                   _("Softness"))
+property_double (softness, _("Softness"), 0.8)
+    value_range (0.0, 1.0)
 
-gegl_chant_double (gamma, _("Gamma"),
-                   1.0, 20.0, 2.0,
-                   _("Falloff linearity"))
+property_double (gamma, _("Gamma"), 2.0)
+    description (_("Falloff linearity"))
+    value_range (1.0, 20.0)
 
-gegl_chant_double (proportion, _("Proportion"),
-                   0.0, 1.0, 1.0,
-                   _("How close we are to image proportions"))
+property_double (proportion, _("Proportion"), 1.0)
+    description(_("How close we are to image proportions"))
+    value_range (0.0, 1.0)
 
-gegl_chant_double (squeeze, _("Squeeze"),
-                   -1.0, 1.0, 0.0,
-                   _("Aspect ratio to use, -0.5 = 1:2, 0.0 = 1:1, 0.5 = 2:1, "
-                     "-1.0 = 1:inf 1.0 = inf:1, this is applied after "
-                     "proportion is taken into account, to directly use "
-                     "squeeze factor as proportions, set proportion to 0.0."))
+property_double (squeeze, _("Squeeze"), 0.0)
+    description (("Aspect ratio to use, -0.5 = 1:2, 0.0 = 1:1, 0.5 = 2:1, "
+              "-1.0 = 1:inf 1.0 = inf:1, this is applied after "
+              "proportion is taken into account, to directly use "
+              "squeeze factor as proportions, set proportion to 0.0."))
+    value_range (-1.0, 1.0)
 
-gegl_chant_double (x, _("X"),
-                   -1.0, 2.0, 0.5,
-                   _("Horizontal center of vignetting"))
+property_double (x, _("Center X"), 0.5)
+    ui_range    (0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
+    ui_meta     ("axis", "x")
 
-gegl_chant_double (y, _("Y"),
-                   -1.0, 2.0, 0.5,
-                   _("Vertical center of vignetting"))
+property_double (y, _("Center Y"), 0.5)
+    ui_range    (0, 1.0)
+    ui_meta     ("unit", "relative-coordinate")
+    ui_meta     ("axis", "y")
 
-gegl_chant_double (rotation, _("Rotation"),
-                   0.0, 360.0, 0.0,
-                   _("Rotation angle"))
+property_double (rotation, _("Rotation"), 0.0)
+    value_range (0.0, 360.0)
+    ui_meta     ("unit", "degree") /* XXX: change to radians */
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_FILTER
-#define GEGL_CHANT_C_FILE "vignette.c"
+#define GEGL_OP_POINT_FILTER
+#define GEGL_OP_C_SOURCE vignette.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 #include <math.h>
 
@@ -136,7 +134,7 @@ cl_process (GeglOperation       *operation,
             const GeglRectangle *roi,
             gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gfloat      scale;
   gfloat      radius0, radius1;
   gint        roi_x, roi_y,x;
@@ -249,7 +247,7 @@ process (GeglOperation       *operation,
          const GeglRectangle *roi,
          gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gfloat     *in_pixel =  in_buf;
   gfloat     *out_pixel = out_buf;
   gfloat      scale;
@@ -317,11 +315,11 @@ process (GeglOperation       *operation,
         {
           switch (o->shape)
           {
-            case GEGl_VIGNETTE_SHAPE_CIRCLE:
+            case GEGL_VIGNETTE_SHAPE_CIRCLE:
               strength = hypot ((u-midx) / scale, v-midy);      break;
-            case GEGl_VIGNETTE_SHAPE_SQUARE:
+            case GEGL_VIGNETTE_SHAPE_SQUARE:
               strength = MAX(ABS(u-midx) / scale, ABS(v-midy)); break;
-            case GEGl_VIGNETTE_SHAPE_DIAMOND:
+            case GEGL_VIGNETTE_SHAPE_DIAMOND:
               strength = ABS(u-midx) / scale + ABS(v-midy);     break;
           }
           strength /= length;
@@ -362,7 +360,7 @@ process (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
@@ -378,12 +376,14 @@ gegl_chant_class_init (GeglChantClass *klass)
   point_filter_class->cl_process = cl_process;
 
   gegl_operation_class_set_keys (operation_class,
-    "name",        "gegl:vignette",
-    "categories",  "render",
+    "name",               "gegl:vignette",
+    "title",              _("Vignette"),
+    "position-dependent", "true",
+    "categories",         "render:photo",
     "description", _("Applies a vignette to an image. Simulates the luminance "
                      "fall off at the edge of exposed film, and some other "
                      "fuzzier border effects that can naturally occur with "
-                     "analog photograpy"),
+                     "analog photography"),
   NULL);
 }
 
