@@ -20,35 +20,33 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (ref, _("Reference"), "ID",
-                   _("The reference ID used as input (for use in XML)."))
+property_string (ref, _("Reference"), "ID")
+    description (_("The reference ID used as input (for use in XML)."))
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "clone.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE        clone.c
 
-#include "gegl-chant.h"
-#include "graph/gegl-node.h"
+#include "gegl-op.h"
 #include <math.h>
 #include <string.h>
 
-
 static GeglNode *
 detect (GeglOperation *operation,
         gint           x,
         gint           y)
 {
-  GeglNode *node = operation->node;
+  GeglRectangle have_rect = gegl_node_get_bounding_box (operation->node);
 
-  if (x >= node->have_rect.x &&
-      y >= node->have_rect.y &&
-      x < node->have_rect.width &&
-      y < node->have_rect.height)
+  if (x >= have_rect.x &&
+      y >= have_rect.y &&
+      x < have_rect.width &&
+      y < have_rect.height)
     {
-      return node;
+      return operation->node;
     }
   return NULL;
 }
@@ -69,10 +67,11 @@ get_bounding_box (GeglOperation *operation)
 }
 
 static gboolean
-process (GeglOperation       *operation,
-         GeglOperationContext     *context,
-         const gchar         *output_prop,
-         const GeglRectangle *result)
+process (GeglOperation        *operation,
+         GeglOperationContext *context,
+         const gchar          *output_prop,
+         const GeglRectangle  *result,
+         gint                  level)
 {
   GeglBuffer *input;
 
@@ -88,13 +87,14 @@ process (GeglOperation       *operation,
       g_warning ("clone received NULL input");
       return FALSE;
     }
+
   gegl_operation_context_take_object (context, "output", G_OBJECT (input));
   return TRUE;
 }
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass *operation_class;
 
@@ -102,12 +102,14 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->process = process;
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->detect = detect;
-
-  operation_class->name        = "gegl:clone";
-  operation_class->categories  = "core";
-  operation_class->description = _("Clone a buffer");
-
   operation_class->no_cache = TRUE;
+
+  gegl_operation_class_set_keys (operation_class,
+       "name",          "gegl:clone",
+       "title",         _("Clone"),
+       "description",   _("Clone a buffer"),
+       "categories",    "core",
+       NULL);
 }
 
 #endif
