@@ -20,17 +20,24 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
    /* no properties */
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "nop.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE        nop.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
+static void
+gegl_nop_prepare (GeglOperation *self)
+{
+  const Babl *fmt = gegl_operation_get_source_format (self, "input");
+
+  gegl_operation_set_format (self, "output", fmt);
+}
 
 static gboolean
 gegl_nop_process (GeglOperation        *operation,
@@ -59,15 +66,17 @@ gegl_nop_process (GeglOperation        *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass *operation_class;
 
   operation_class = GEGL_OPERATION_CLASS (klass);
   operation_class->process = gegl_nop_process;
+  operation_class->prepare = gegl_nop_prepare;
 
   gegl_operation_class_set_keys (operation_class,
               "name",        "gegl:nop",
+              "title",       _("No Operation"),
               "categories",  "core",
               "description", _("No operation (can be used as a routing point)"),
               NULL);
