@@ -18,17 +18,18 @@
  * Copyright 2006 Dominik Ernst <dernst@gmx.de>
  */
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (path, "File", "", "Path of file to load.")
+property_file_path (path, "File", "")
+  description ("Path of file to load.")
 
 #else
 
-#define GEGL_CHANT_TYPE_SOURCE
-#define GEGL_CHANT_C_FILE       "exr-load.cpp"
+#define GEGL_OP_SOURCE
+#define GEGL_OP_C_FILE       "exr-load.cpp"
 
 extern "C" {
-#include "gegl-chant.h"
+#include "gegl-op.h"
 }
 
 #include <ImfInputFile.h>
@@ -236,7 +237,8 @@ fix_saturation (GeglBuffer       *buf,
   for (y=0; y<2; y++)
     {
       gegl_rectangle_set (&rect, 0,y, gegl_buffer_get_width (buf), 1);
-      gegl_buffer_get (buf, 1.0, &rect, NULL, row[y+1], GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (buf, &rect, 1.0, NULL, row[y+1], GEGL_AUTO_ROWSTRIDE,
+                       GEGL_ABYSS_NONE);
     }
 
   fix_saturation_row (row[1], row[1], row[2], yw, gegl_buffer_get_width (buf), nc);
@@ -246,11 +248,12 @@ fix_saturation (GeglBuffer       *buf,
       if (y>1)
         {
           gegl_rectangle_set (&rect, 0, y-2, gegl_buffer_get_width (buf), 1);
-          gegl_buffer_set (buf, &rect, NULL, row[0], GEGL_AUTO_ROWSTRIDE);
+          gegl_buffer_set (buf, &rect, 0, NULL, row[0], GEGL_AUTO_ROWSTRIDE);
         }
 
       gegl_rectangle_set (&rect, 0,y+1, gegl_buffer_get_width (buf), 1);
-      gegl_buffer_get (buf, 1.0, &rect, NULL, row[0], GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (buf, &rect, 1.0, NULL, row[0], GEGL_AUTO_ROWSTRIDE,
+                       GEGL_ABYSS_NONE);
 
       tmp = row[0];
       row[0] = row[1];
@@ -265,7 +268,7 @@ fix_saturation (GeglBuffer       *buf,
   for (y=gegl_buffer_get_height (buf)-2; y<gegl_buffer_get_height (buf); y++)
     {
       gegl_rectangle_set (&rect, 0, y, gegl_buffer_get_width (buf), 1);
-      gegl_buffer_set (buf, &rect, NULL, row[y-gegl_buffer_get_height (buf)+2], GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_set (buf, &rect, 0, NULL, row[y-gegl_buffer_get_height (buf)+2], GEGL_AUTO_ROWSTRIDE);
     }
 
   for (y=0; y<3; y++)
@@ -331,19 +334,21 @@ reconstruct_chroma (GeglBuffer *buf,
   for (i=0; i<gegl_buffer_get_height (buf); i+=2)
     {
       gegl_rectangle_set (&rect, 0, i,  gegl_buffer_get_width (buf), 1);
-      gegl_buffer_get (buf, 1.0, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (buf, &rect, 1.0, NULL, pixels, GEGL_AUTO_ROWSTRIDE,
+                       GEGL_ABYSS_NONE);
 
       reconstruct_chroma_row (pixels, gegl_buffer_get_width (buf), has_alpha, tmp);
-      gegl_buffer_set (buf, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_set (buf, &rect, 0, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
     }
 
   for (i=0; i<gegl_buffer_get_width (buf); i++)
     {
       gegl_rectangle_set (&rect, i, 0, 1, gegl_buffer_get_height (buf));
-      gegl_buffer_get (buf, 1.0, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (buf, &rect, 1.0, NULL, pixels, GEGL_AUTO_ROWSTRIDE,
+                       GEGL_ABYSS_NONE);
 
       reconstruct_chroma_row (pixels, gegl_buffer_get_height (buf), has_alpha, tmp);
-      gegl_buffer_set (buf, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_set (buf, &rect, 0, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
     }
 
   g_free (tmp);
@@ -368,7 +373,8 @@ convert_yca_to_rgba (GeglBuffer *buf,
   for (row=0; row<gegl_buffer_get_height (buf); row++)
     {
       gegl_rectangle_set (&rect, 0, row, gegl_buffer_get_width (buf), 1);
-      gegl_buffer_get (buf, 1.0, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (buf, &rect, 1.0, NULL, pixels, GEGL_AUTO_ROWSTRIDE,
+                       GEGL_ABYSS_NONE);
       pxl = (gfloat*) pixels;
 
       for (i=0; i<gegl_buffer_get_width (buf); i++)
@@ -388,7 +394,7 @@ convert_yca_to_rgba (GeglBuffer *buf,
           pxl += dx;
         }
 
-      gegl_buffer_set (buf, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_set (buf, &rect, 0, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
     }
 
   g_free (pixels);
@@ -403,30 +409,39 @@ insert_channels (FrameBuffer  &fb,
                  gint          format_flags,
                  gint          bpp)
 {
-  gint alpha_offset = 12;
+  gint alpha_offset;
   PixelType tp;
+  // bytes per channel - is there a babl mechanism to obtain that from format_flags?
+  gint bpc = 4;
 
   if (format_flags & COLOR_U32)
     tp = UINT;
+  else if (format_flags & COLOR_FP16)
+    {
+      tp = HALF;
+      bpc = 2;
+    }
   else
     tp = FLOAT;
 
+  alpha_offset = bpc*3;
+
   if (format_flags & COLOR_RGB)
     {
-      fb.insert ("R", Slice (tp, base,    bpp, 0, 1,1, 0.0));
-      fb.insert ("G", Slice (tp, base+4,  bpp, 0, 1,1, 0.0));
-      fb.insert ("B", Slice (tp, base+8,  bpp, 0, 1,1, 0.0));
+      fb.insert ("R", Slice (tp, base,          bpp, 0, 1,1, 0.0));
+      fb.insert ("G", Slice (tp, base+bpc,      bpp, 0, 1,1, 0.0));
+      fb.insert ("B", Slice (tp, base+bpc*2,    bpp, 0, 1,1, 0.0));
     }
   else if (format_flags & COLOR_C)
     {
-      fb.insert ("Y",  Slice (tp, base,   bpp,   0, 1,1, 0.5));
-      fb.insert ("RY", Slice (tp, base+4, bpp*2, 0, 2,2, 0.0));
-      fb.insert ("BY", Slice (tp, base+8, bpp*2, 0, 2,2, 0.0));
+      fb.insert ("Y",  Slice (tp, base,         bpp,   0, 1,1, 0.5));
+      fb.insert ("RY", Slice (tp, base+bpc,     bpp*2, 0, 2,2, 0.0));
+      fb.insert ("BY", Slice (tp, base+bpc*2,   bpp*2, 0, 2,2, 0.0));
     }
   else if (format_flags & COLOR_Y)
     {
       fb.insert ("Y",  Slice (tp, base, bpp, 0, 1,1, 0.5));
-      alpha_offset = 4;
+      alpha_offset = bpc;
     }
 
   if (format_flags & COLOR_ALPHA)
@@ -480,7 +495,7 @@ import_exr (GeglBuffer  *gegl_buffer,
           {
             gegl_rectangle_set (&rect, 0, i-dw.min.y,gegl_buffer_get_width (gegl_buffer), 1);
             file.readPixels (i);
-            gegl_buffer_set (gegl_buffer, &rect, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
+            gegl_buffer_set (gegl_buffer, &rect, 0, NULL, pixels, GEGL_AUTO_ROWSTRIDE);
           }
       }
 
@@ -579,6 +594,9 @@ query_exr (const gchar *path,
             strcat (format_string, " u32");
             break;
           case HALF:
+	    format_flags |= COLOR_FP16;
+            strcat (format_string, " half");
+            break;
           case FLOAT:
           default:
             format_flags |= COLOR_FP32;
@@ -593,14 +611,14 @@ query_exr (const gchar *path,
     }
 
   *ff_ptr = format_flags;
-  *format = babl_format (format_string);
+  *format = (void*)babl_format (format_string);
   return TRUE;
 }
 
 static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglRectangle result = {0, 0, 10, 10};
   gint          w, h, ff;
   gpointer      format;
@@ -618,9 +636,10 @@ get_bounding_box (GeglOperation *operation)
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         int                  level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gint        w,h,ff;
   gpointer    format;
   gboolean    ok;
@@ -647,7 +666,7 @@ get_cached_region (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationSourceClass *source_class;
@@ -659,9 +678,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
 
   operation_class->get_cached_region = get_cached_region;
-  operation_class->name        = "gegl:exr-load";
-  operation_class->categories  = "hidden";
-  operation_class->description = "EXR image loader.";
+
+  gegl_operation_class_set_keys (operation_class,
+    "name"        , "gegl:exr-load",
+    "categories"  , "hidden",
+    "description" , "EXR image loader.", NULL);
 
   gegl_extension_handler_register (".exr", "gegl:exr-load");
 }
