@@ -61,6 +61,7 @@ typedef struct
   gint             width;
   gint             height;
   gdouble          fps;
+  gint             codec_delay;
 
   gchar           *loadedfilename; /* to remember which file is "cached"     */
 
@@ -313,13 +314,18 @@ decode_frame (GeglOperation *operation,
     }
 
   decodeframe = frame;
-  if (frame > prevframe + 20 || frame < prevframe )
+  if (frame < 2 || frame > prevframe + 64 || frame < prevframe )
   {
-    int64_t seek_target = av_rescale_q ((frame - 16) / o->frame_rate * AV_TIME_BASE, AV_TIME_BASE_Q, p->video_stream->time_base);
+ 
+    int64_t seek_target = av_rescale_q (((frame) * AV_TIME_BASE * 1.0) / o->frame_rate
+, AV_TIME_BASE_Q, p->video_stream->time_base) / p->video_stream->codec->ticks_per_frame;
+
     if (av_seek_frame (p->video_fcontext, p->video_index, seek_target, (AVSEEK_FLAG_BACKWARD )) < 0)
       fprintf (stderr, "video seek error!\n");
-   else
+    else
       avcodec_flush_buffers (p->video_stream->codec);
+    
+    prevframe = -1;
   }
 
   do
@@ -341,8 +347,9 @@ decode_frame (GeglOperation *operation,
           }
           while (pkt.stream_index != p->video_index);
 
-          decoded_bytes = avcodec_decode_video2 (p->video_stream->codec, p->lavc_frame,
-                                                 &got_picture, &pkt);
+          decoded_bytes = avcodec_decode_video2 (
+                 p->video_stream->codec, p->lavc_frame,
+                 &got_picture, &pkt);
           if (decoded_bytes < 0)
             {
               fprintf (stderr, "avcodec_decode_video failed for %s\n",
@@ -352,8 +359,20 @@ decode_frame (GeglOperation *operation,
 
           if(got_picture)
           {
-             p->prevpts = av_frame_get_best_effort_timestamp (p->lavc_frame) * av_q2d (p->video_stream->time_base);
-             decodeframe = roundf (av_frame_get_best_effort_timestamp (p->lavc_frame) * av_q2d (p->video_stream->time_base) * o->frame_rate) - 1;
+             if ((pkt.dts == pkt.pts) || (p->lavc_frame->key_frame!=0))
+             {
+               p->lavc_frame->pts = 
+(p->video_stream->cur_dts - p->video_stream->first_dts);
+               p->prevpts =
+     av_rescale_q ( p->lavc_frame->pts, p->video_stream->time_base,
+AV_TIME_BASE_Q) * 1.0 / AV_TIME_BASE ;
+               decodeframe = roundf( p->prevpts * o->frame_rate);
+             }
+             else
+             {
+               p->prevpts += 1.0 / o->frame_rate;
+               decodeframe = roundf ( p->prevpts * o->frame_rate);
+             }
           }
 
           if (decoded_bytes != pkt.size)
@@ -362,7 +381,7 @@ decode_frame (GeglOperation *operation,
         }
       while (!got_picture);
     }
-    while (decodeframe < frame);
+    while (decodeframe <= frame + p->codec_delay);
 
   p->prevframe = frame;
   return 0;
@@ -395,12 +414,10 @@ prepare (GeglOperation *operation)
         {
           print_error (o->path, err);
         }
-      fprintf (stderr, "[%lu]\n", p->video_fcontext->start_time_realtime);
       err = avformat_find_stream_info (p->video_fcontext, NULL);
       if (err < 0)
         {
           g_warning ("ff-load: error finding stream info for %s", o->path);
-
           return;
         }
       err = avformat_open_input(&p->audio_fcontext, o->path, NULL, 0);
@@ -412,7 +429,6 @@ prepare (GeglOperation *operation)
       if (err < 0)
         {
           g_warning ("ff-load: error finding stream info for %s", o->path);
-
           return;
         }
 
@@ -460,12 +476,14 @@ prepare (GeglOperation *operation)
 
 #endif
 
+#if 0
       p->video_stream->codec->idct_algo = FF_IDCT_SIMPLEAUTO;
 
       p->video_stream->codec->thread_count = 0;
       p->video_stream->codec->thread_type = FF_THREAD_SLICE;
       /* XXX: permits slice parallell decode, at expense of h264 compliance of output */
       p->video_stream->codec->flags2 = AV_CODEC_FLAG2_FAST;
+#endif
 
       if (p->video_codec == NULL)
           g_warning ("video codec not found");
@@ -522,6 +540,24 @@ prepare (GeglOperation *operation)
       }
 #endif
     }
+
+  /* is there a way to compute this from internals
+   */
+  if (!strcmp (o->video_codec, "mpeg1video"))
+    p->codec_delay = 1;
+  else if (!strcmp (o->video_codec, "mpeg4"))
+    p->codec_delay = 2;
+  else if (!strcmp (o->video_codec, "vp9"))
+    p->codec_delay = 2;
+  else if (!strcmp (o->video_codec, "theora"))
+    p->codec_delay = 2;
+  else if (!strcmp (o->video_codec, "h264"))
+    p->codec_delay = 0;
+  else
+    {
+      p->codec_delay = 0;
+      fprintf (stderr, "[assuming no codec_delay for: %s]", o->video_codec);
+    }
 }
 
 static GeglRectangle
@@ -564,10 +600,9 @@ static void get_sample_data (Priv *p, long sample_no, float *left, float *right)
 {
   int to_remove = 0;
   GList *l;
-  l = p->audio_track;
   if (sample_no < 0)
     return;
-  for (; l; l = l->next)
+  for (l = p->audio_track; l; l = l->next)
   {
     GeglAudioFragment *af = l->data;
     int channels = gegl_audio_fragment_get_channels (af);
