@@ -62,6 +62,7 @@ static const char* op_c_source =
 "  gint             width;                                                     \n"
 "  gint             height;                                                    \n"
 "  gdouble          fps;                                                       \n"
+"  gint             codec_delay;                                               \n"
 "                                                                              \n"
 "  gchar           *loadedfilename; /* to remember which file is \"cached\"     */\n"
 "                                                                              \n"
@@ -314,13 +315,18 @@ static const char* op_c_source =
 "    }                                                                         \n"
 "                                                                              \n"
 "  decodeframe = frame;                                                        \n"
-"  if (frame > prevframe + 20 || frame < prevframe )                           \n"
+"  if (frame < 2 || frame > prevframe + 64 || frame < prevframe )              \n"
 "  {                                                                           \n"
-"    int64_t seek_target = av_rescale_q ((frame - 16) / o->frame_rate * AV_TIME_BASE, AV_TIME_BASE_Q, p->video_stream->time_base);\n"
+"                                                                              \n"
+"    int64_t seek_target = av_rescale_q (((frame) * AV_TIME_BASE * 1.0) / o->frame_rate\n"
+", AV_TIME_BASE_Q, p->video_stream->time_base) / p->video_stream->codec->ticks_per_frame;\n"
+"                                                                              \n"
 "    if (av_seek_frame (p->video_fcontext, p->video_index, seek_target, (AVSEEK_FLAG_BACKWARD )) < 0)\n"
 "      fprintf (stderr, \"video seek error!\\n\");                             \n"
-"   else                                                                       \n"
+"    else                                                                      \n"
 "      avcodec_flush_buffers (p->video_stream->codec);                         \n"
+"                                                                              \n"
+"    prevframe = -1;                                                           \n"
 "  }                                                                           \n"
 "                                                                              \n"
 "  do                                                                          \n"
@@ -342,8 +348,9 @@ static const char* op_c_source =
 "          }                                                                   \n"
 "          while (pkt.stream_index != p->video_index);                         \n"
 "                                                                              \n"
-"          decoded_bytes = avcodec_decode_video2 (p->video_stream->codec, p->lavc_frame,\n"
-"                                                 &got_picture, &pkt);         \n"
+"          decoded_bytes = avcodec_decode_video2 (                             \n"
+"                 p->video_stream->codec, p->lavc_frame,                       \n"
+"                 &got_picture, &pkt);                                         \n"
 "          if (decoded_bytes < 0)                                              \n"
 "            {                                                                 \n"
 "              fprintf (stderr, \"avcodec_decode_video failed for %s\\n\",     \n"
@@ -353,8 +360,20 @@ static const char* op_c_source =
 "                                                                              \n"
 "          if(got_picture)                                                     \n"
 "          {                                                                   \n"
-"             p->prevpts = av_frame_get_best_effort_timestamp (p->lavc_frame) * av_q2d (p->video_stream->time_base);\n"
-"             decodeframe = roundf (av_frame_get_best_effort_timestamp (p->lavc_frame) * av_q2d (p->video_stream->time_base) * o->frame_rate) - 1;\n"
+"             if ((pkt.dts == pkt.pts) || (p->lavc_frame->key_frame!=0))       \n"
+"             {                                                                \n"
+"               p->lavc_frame->pts =                                           \n"
+"(p->video_stream->cur_dts - p->video_stream->first_dts);                      \n"
+"               p->prevpts =                                                   \n"
+"     av_rescale_q ( p->lavc_frame->pts, p->video_stream->time_base,           \n"
+"AV_TIME_BASE_Q) * 1.0 / AV_TIME_BASE ;                                        \n"
+"               decodeframe = roundf( p->prevpts * o->frame_rate);             \n"
+"             }                                                                \n"
+"             else                                                             \n"
+"             {                                                                \n"
+"               p->prevpts += 1.0 / o->frame_rate;                             \n"
+"               decodeframe = roundf ( p->prevpts * o->frame_rate);            \n"
+"             }                                                                \n"
 "          }                                                                   \n"
 "                                                                              \n"
 "          if (decoded_bytes != pkt.size)                                      \n"
@@ -363,7 +382,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "      while (!got_picture);                                                   \n"
 "    }                                                                         \n"
-"    while (decodeframe < frame);                                              \n"
+"    while (decodeframe <= frame + p->codec_delay);                            \n"
 "                                                                              \n"
 "  p->prevframe = frame;                                                       \n"
 "  return 0;                                                                   \n"
@@ -396,12 +415,10 @@ static const char* op_c_source =
 "        {                                                                     \n"
 "          print_error (o->path, err);                                         \n"
 "        }                                                                     \n"
-"      fprintf (stderr, \"[%lu]\\n\", p->video_fcontext->start_time_realtime); \n"
 "      err = avformat_find_stream_info (p->video_fcontext, NULL);              \n"
 "      if (err < 0)                                                            \n"
 "        {                                                                     \n"
 "          g_warning (\"ff-load: error finding stream info for %s\", o->path); \n"
-"                                                                              \n"
 "          return;                                                             \n"
 "        }                                                                     \n"
 "      err = avformat_open_input(&p->audio_fcontext, o->path, NULL, 0);        \n"
@@ -413,7 +430,6 @@ static const char* op_c_source =
 "      if (err < 0)                                                            \n"
 "        {                                                                     \n"
 "          g_warning (\"ff-load: error finding stream info for %s\", o->path); \n"
-"                                                                              \n"
 "          return;                                                             \n"
 "        }                                                                     \n"
 "                                                                              \n"
@@ -461,12 +477,14 @@ static const char* op_c_source =
 "                                                                              \n"
 "#endif                                                                        \n"
 "                                                                              \n"
+"#if 0                                                                         \n"
 "      p->video_stream->codec->idct_algo = FF_IDCT_SIMPLEAUTO;                 \n"
 "                                                                              \n"
 "      p->video_stream->codec->thread_count = 0;                               \n"
 "      p->video_stream->codec->thread_type = FF_THREAD_SLICE;                  \n"
 "      /* XXX: permits slice parallell decode, at expense of h264 compliance of output */\n"
 "      p->video_stream->codec->flags2 = AV_CODEC_FLAG2_FAST;                   \n"
+"#endif                                                                        \n"
 "                                                                              \n"
 "      if (p->video_codec == NULL)                                             \n"
 "          g_warning (\"video codec not found\");                              \n"
@@ -523,6 +541,24 @@ static const char* op_c_source =
 "      }                                                                       \n"
 "#endif                                                                        \n"
 "    }                                                                         \n"
+"                                                                              \n"
+"  /* is there a way to compute this from internals                            \n"
+"   */                                                                         \n"
+"  if (!strcmp (o->video_codec, \"mpeg1video\"))                               \n"
+"    p->codec_delay = 1;                                                       \n"
+"  else if (!strcmp (o->video_codec, \"mpeg4\"))                               \n"
+"    p->codec_delay = 2;                                                       \n"
+"  else if (!strcmp (o->video_codec, \"vp9\"))                                 \n"
+"    p->codec_delay = 2;                                                       \n"
+"  else if (!strcmp (o->video_codec, \"theora\"))                              \n"
+"    p->codec_delay = 2;                                                       \n"
+"  else if (!strcmp (o->video_codec, \"h264\"))                                \n"
+"    p->codec_delay = 0;                                                       \n"
+"  else                                                                        \n"
+"    {                                                                         \n"
+"      p->codec_delay = 0;                                                     \n"
+"      fprintf (stderr, \"[assuming no codec_delay for: %s]\", o->video_codec);\n"
+"    }                                                                         \n"
 "}                                                                             \n"
 "                                                                              \n"
 "static GeglRectangle                                                          \n"
@@ -565,10 +601,9 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  int to_remove = 0;                                                          \n"
 "  GList *l;                                                                   \n"
-"  l = p->audio_track;                                                         \n"
 "  if (sample_no < 0)                                                          \n"
 "    return;                                                                   \n"
-"  for (; l; l = l->next)                                                      \n"
+"  for (l = p->audio_track; l; l = l->next)                                    \n"
 "  {                                                                           \n"
 "    GeglAudioFragment *af = l->data;                                          \n"
 "    int channels = gegl_audio_fragment_get_channels (af);                     \n"
