@@ -97,12 +97,13 @@ static const char* op_c_source =
 "  uint8_t  *buffer;                                                           \n"
 "                                                                              \n"
 "  int       audio_outbuf_size;                                                \n"
-"  int       audio_input_frame_size;                                           \n"
 "  int16_t  *samples;                                                          \n"
 "                                                                              \n"
 "  GList    *audio_track;                                                      \n"
 "  long      audio_pos;                                                        \n"
 "  long      audio_read_pos;                                                   \n"
+"                                                                              \n"
+"  int       next_apts;                                                        \n"
 "} Priv;                                                                       \n"
 "                                                                              \n"
 "static void                                                                   \n"
@@ -120,15 +121,14 @@ static const char* op_c_source =
 "{                                                                             \n"
 "  int to_remove = 0;                                                          \n"
 "  GList *l;                                                                   \n"
-"  l = p->audio_track;                                                         \n"
 "  if (sample_no < 0)                                                          \n"
 "    return;                                                                   \n"
-"  for (; l; l = l->next)                                                      \n"
+"  for (l = p->audio_track; l; l = l->next)                                    \n"
 "  {                                                                           \n"
 "    GeglAudioFragment *af = l->data;                                          \n"
-"    gint pos = gegl_audio_fragment_get_pos (af);                              \n"
-"    gint channels = gegl_audio_fragment_get_channels (af);                    \n"
-"    gint sample_count = gegl_audio_fragment_get_sample_count (af);            \n"
+"    int channels = gegl_audio_fragment_get_channels (af);                     \n"
+"    int pos = gegl_audio_fragment_get_pos (af);                               \n"
+"    int sample_count = gegl_audio_fragment_get_sample_count (af);             \n"
 "    if (sample_no > pos + sample_count)                                       \n"
 "    {                                                                         \n"
 "      to_remove ++;                                                           \n"
@@ -150,8 +150,8 @@ static const char* op_c_source =
 "          for (l = p->audio_track; l; l = l->next)                            \n"
 "          {                                                                   \n"
 "            GeglAudioFragment *af = l->data;                                  \n"
-"            gint pos = gegl_audio_fragment_get_pos (af);                      \n"
-"            gint sample_count = gegl_audio_fragment_get_sample_count (af);    \n"
+"            int pos = gegl_audio_fragment_get_pos (af);                       \n"
+"            int sample_count = gegl_audio_fragment_get_sample_count (af);     \n"
 "            if (sample_no > pos + sample_count)                               \n"
 "            {                                                                 \n"
 "              p->audio_track = g_list_remove (p->audio_track, af);            \n"
@@ -163,7 +163,6 @@ static const char* op_c_source =
 "        return;                                                               \n"
 "      }                                                                       \n"
 "  }                                                                           \n"
-"  fprintf (stderr, \"ff-save didn't find audio sample\\n\");                  \n"
 "  *left  = 0;                                                                 \n"
 "  *right = 0;                                                                 \n"
 "}                                                                             \n"
@@ -325,9 +324,14 @@ static const char* op_c_source =
 "write_audio_frame (GeglProperties *o, AVFormatContext * oc, AVStream * st)    \n"
 "{                                                                             \n"
 "  Priv *p = (Priv*)o->user_data;                                              \n"
-"  AVCodecContext *c;                                                          \n"
-"  AVPacket  pkt = { 0 };                                                      \n"
-"  av_init_packet (&pkt);                                                      \n"
+"  AVCodecContext *c = st->codec;                                              \n"
+"  int sample_count = 100000;                                                  \n"
+"  static AVPacket  pkt = { 0 };                                               \n"
+"                                                                              \n"
+"  if (pkt.size == 0)                                                          \n"
+"  {                                                                           \n"
+"    av_init_packet (&pkt);                                                    \n"
+"  }                                                                           \n"
 "                                                                              \n"
 "  /* first we add incoming frames audio samples */                            \n"
 "  {                                                                           \n"
@@ -337,6 +341,7 @@ static const char* op_c_source =
 "                                                     gegl_audio_fragment_get_channels (o->audio),\n"
 "                                                     gegl_audio_fragment_get_channel_layout (o->audio),\n"
 "                                                     sample_count);           \n"
+"    gegl_audio_fragment_set_sample_count (af, sample_count);                  \n"
 "    for (i = 0; i < sample_count; i++)                                        \n"
 "      {                                                                       \n"
 "        af->data[0][i] = o->audio->data[0][i];                                \n"
@@ -347,20 +352,23 @@ static const char* op_c_source =
 "    p->audio_track = g_list_append (p->audio_track, af);                      \n"
 "  }                                                                           \n"
 "                                                                              \n"
-"  /* then we encode as much as we can in a loop using the codec frame size */ \n"
-"  c = st->codec;                                                              \n"
+"  if (!(c->codec->capabilities & AV_CODEC_CAP_VARIABLE_FRAME_SIZE))           \n"
+"    sample_count = c->frame_size;                                             \n"
 "                                                                              \n"
-"  while (p->audio_pos - p->audio_read_pos > c->frame_size)                    \n"
+"  /* then we encode as much as we can in a loop using the codec frame size */ \n"
+"                                                                              \n"
+"                                                                              \n"
+"  while (p->audio_pos - p->audio_read_pos > sample_count)                     \n"
 "  {                                                                           \n"
 "    long i;                                                                   \n"
 "    int ret;                                                                  \n"
 "    int got_packet = 0;                                                       \n"
 "    AVFrame *frame = alloc_audio_frame (c->sample_fmt, c->channel_layout,     \n"
-"                                        c->sample_rate, c->frame_size);       \n"
+"                                        c->sample_rate, sample_count);        \n"
 "                                                                              \n"
 "    switch (c->sample_fmt) {                                                  \n"
 "      case AV_SAMPLE_FMT_FLT:                                                 \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -369,7 +377,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "        break;                                                                \n"
 "      case AV_SAMPLE_FMT_FLTP:                                                \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -378,7 +386,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "        break;                                                                \n"
 "      case AV_SAMPLE_FMT_S16:                                                 \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -387,7 +395,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "        break;                                                                \n"
 "      case AV_SAMPLE_FMT_S32:                                                 \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -396,7 +404,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "        break;                                                                \n"
 "      case AV_SAMPLE_FMT_S32P:                                                \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -405,7 +413,7 @@ static const char* op_c_source =
 "        }                                                                     \n"
 "        break;                                                                \n"
 "      case AV_SAMPLE_FMT_S16P:                                                \n"
-"        for (i = 0; i < c->frame_size; i++)                                   \n"
+"        for (i = 0; i < sample_count; i++)                                    \n"
 "        {                                                                     \n"
 "          float left = 0, right = 0;                                          \n"
 "          get_sample_data (p, i + p->audio_read_pos, &left, &right);          \n"
@@ -417,9 +425,13 @@ static const char* op_c_source =
 "        fprintf (stderr, \"eeeek unhandled audio format\\n\");                \n"
 "        break;                                                                \n"
 "    }                                                                         \n"
+"    frame->pts = p->next_apts;                                                \n"
+"    p->next_apts += sample_count;                                             \n"
 "                                                                              \n"
 "    av_frame_make_writable (frame);                                           \n"
 "    ret = avcodec_encode_audio2 (c, &pkt, frame, &got_packet);                \n"
+"                                                                              \n"
+"    av_packet_rescale_ts (&pkt, st->codec->time_base, st->time_base);         \n"
 "    if (ret < 0) {                                                            \n"
 "      fprintf (stderr, \"Error encoding audio frame: %s\\n\", av_err2str (ret));\n"
 "    }                                                                         \n"
@@ -428,13 +440,12 @@ static const char* op_c_source =
 "    {                                                                         \n"
 "      pkt.stream_index = st->index;                                           \n"
 "      av_interleaved_write_frame (oc, &pkt);                                  \n"
+"      av_free_packet (&pkt);                                                  \n"
 "    }                                                                         \n"
 "                                                                              \n"
 "    av_frame_free (&frame);                                                   \n"
-"    p->audio_read_pos += c->frame_size;                                       \n"
+"    p->audio_read_pos += sample_count;                                        \n"
 "  }                                                                           \n"
-"                                                                              \n"
-"  p->audio_input_frame_size = c->frame_size;                                  \n"
 "}                                                                             \n"
 "                                                                              \n"
 "void                                                                          \n"
@@ -807,6 +818,53 @@ static const char* op_c_source =
 "  return  TRUE;                                                               \n"
 "}                                                                             \n"
 "                                                                              \n"
+"static void flush_audio (GeglProperties *o)                                   \n"
+"{                                                                             \n"
+"  Priv *p = (Priv*)o->user_data;                                              \n"
+"                                                                              \n"
+"  int got_packet;                                                             \n"
+"  do                                                                          \n"
+"  {                                                                           \n"
+"    AVPacket  pkt = { 0 };                                                    \n"
+"    int ret;                                                                  \n"
+"    got_packet = 0;                                                           \n"
+"    av_init_packet (&pkt);                                                    \n"
+"    ret = avcodec_encode_audio2 (p->audio_st->codec, &pkt, NULL, &got_packet);\n"
+"    if (ret < 0)                                                              \n"
+"      break;                                                                  \n"
+"    if (got_packet)                                                           \n"
+"      {                                                                       \n"
+"        pkt.stream_index = p->audio_st->index;                                \n"
+"        av_packet_rescale_ts (&pkt, p->audio_st->codec->time_base, p->audio_st->time_base);\n"
+"        av_interleaved_write_frame (p->oc, &pkt);                             \n"
+"        av_free_packet (&pkt);                                                \n"
+"      }                                                                       \n"
+"  } while (got_packet);                                                       \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static void flush_video (GeglProperties *o)                                   \n"
+"{                                                                             \n"
+"  Priv *p = (Priv*)o->user_data;                                              \n"
+"  int got_packet = 0;                                                         \n"
+"  do {                                                                        \n"
+"    AVPacket  pkt = { 0 };                                                    \n"
+"    int ret;                                                                  \n"
+"    got_packet = 0;                                                           \n"
+"    av_init_packet (&pkt);                                                    \n"
+"    ret = avcodec_encode_video2 (p->video_st->codec, &pkt, NULL, &got_packet);\n"
+"    if (ret < 0)                                                              \n"
+"      return;                                                                 \n"
+"                                                                              \n"
+"     if (got_packet)                                                          \n"
+"     {                                                                        \n"
+"       pkt.stream_index = p->video_st->index;                                 \n"
+"       av_packet_rescale_ts (&pkt, p->video_st->codec->time_base, p->video_st->time_base);\n"
+"       av_interleaved_write_frame (p->oc, &pkt);                              \n"
+"       av_free_packet (&pkt);                                                 \n"
+"     }                                                                        \n"
+"  } while (got_packet);                                                       \n"
+"}                                                                             \n"
+"                                                                              \n"
 "static void                                                                   \n"
 "finalize (GObject *object)                                                    \n"
 "{                                                                             \n"
@@ -814,18 +872,18 @@ static const char* op_c_source =
 "  if (o->user_data)                                                           \n"
 "    {                                                                         \n"
 "      Priv *p = (Priv*)o->user_data;                                          \n"
+"      flush_audio (o);                                                        \n"
+"      flush_video (o);                                                        \n"
 "                                                                              \n"
-"    if (p->oc)                                                                \n"
-"      {                                                                       \n"
-"        if (p->video_st)                                                      \n"
-"          close_video (p, p->oc, p->video_st);                                \n"
-"        if (p->audio_st)                                                      \n"
-"          close_audio (p, p->oc, p->audio_st);                                \n"
+"      av_write_trailer (p->oc);                                               \n"
 "                                                                              \n"
-"        av_write_trailer (p->oc);                                             \n"
-"        avio_closep (&p->oc->pb);                                             \n"
-"        avformat_free_context (p->oc);                                        \n"
-"      }                                                                       \n"
+"      if (p->video_st)                                                        \n"
+"        close_video (p, p->oc, p->video_st);                                  \n"
+"      if (p->audio_st)                                                        \n"
+"        close_audio (p, p->oc, p->audio_st);                                  \n"
+"                                                                              \n"
+"      avio_closep (&p->oc->pb);                                               \n"
+"      avformat_free_context (p->oc);                                          \n"
 "                                                                              \n"
 "      g_free (o->user_data);                                                  \n"
 "      o->user_data = NULL;                                                    \n"
