@@ -20,16 +20,17 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_file_path (path, _("File"), "", _("Path of file to load."))
+property_file_path (path, _("File"), "")
+  description (_("Path of file to load"))
 
 #else
 
-#define GEGL_CHANT_TYPE_SOURCE
-#define GEGL_CHANT_C_FILE       "jp2-load.c"
+#define GEGL_OP_SOURCE
+#define GEGL_OP_C_SOURCE jp2-load.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <stdio.h>
 #include <jasper/jasper.h>
 
@@ -67,14 +68,14 @@ query_jp2 (const gchar   *path,
       image_fmt = jas_image_getfmt (in);
       if (image_fmt < 0)
         {
-          g_warning (_("Unknown JPEG-2000 image format in '%s'"), path);
+          g_warning (_("Unknown JPEG 2000 image format in '%s'"), path);
           break;
         }
 
       image = jas_image_decode (in, image_fmt, NULL);
       if (!image)
         {
-          g_warning (_("Unable to open JPEG-2000 image in '%s'"), path);
+          g_warning (_("Unable to open JPEG 2000 image in '%s'"), path);
           break;
         }
 
@@ -97,7 +98,7 @@ query_jp2 (const gchar   *path,
       numcmpts = jas_image_numcmpts (cimage);
       if (numcmpts != 3)
         {
-          g_warning (_("Unsupported non-RGB JPEG-2000 file with "
+          g_warning (_("Unsupported non-RGB JPEG 2000 file with "
                        "%d components in '%s'"), numcmpts, path);
           break;
         }
@@ -108,7 +109,7 @@ query_jp2 (const gchar   *path,
 
       if ((*depth != 8) && (*depth != 16))
         {
-          g_warning (_("Unsupported JPEG-2000 file with depth %d in '%s'"),
+          g_warning (_("Unsupported JPEG 2000 file with depth %d in '%s'"),
                      *depth, path);
           break;
         }
@@ -167,9 +168,10 @@ prepare (GeglOperation *operation)
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglRectangle rect = {0,0,0,0};
   jas_image_t *image;
   gint width, height, depth;
@@ -297,12 +299,12 @@ process (GeglOperation       *operation,
           switch (depth)
             {
             case 16:
-              gegl_buffer_set (output, &rect, babl_format ("R'G'B' u16"),
+              gegl_buffer_set (output, &rect, 0, babl_format ("R'G'B' u16"),
                                data_s, GEGL_AUTO_ROWSTRIDE);
               break;
 
             case 8:
-              gegl_buffer_set (output, &rect, babl_format ("R'G'B' u8"),
+              gegl_buffer_set (output, &rect, 0, babl_format ("R'G'B' u8"),
                                data_b, GEGL_AUTO_ROWSTRIDE);
 	      break;
 
@@ -338,7 +340,7 @@ process (GeglOperation       *operation,
 static GeglRectangle
 get_bounding_box (GeglOperation * operation)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   GeglRectangle result = { 0, 0, 0, 0 };
   gint width, height, depth;
 
@@ -377,7 +379,7 @@ get_cached_region (GeglOperation       *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationSourceClass *source_class;
@@ -390,9 +392,12 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_cached_region = get_cached_region;
 
-  operation_class->name        = "gegl:jp2-load";
-  operation_class->categories  = "hidden";
-  operation_class->description = _("JPEG-2000 image loader.");
+  gegl_operation_class_set_keys (operation_class,
+    "name",        "gegl:jp2-load",
+    "title",       _("JPEG 2000 File Loader"),
+    "categories",  "hidden",
+    "description", _("JPEG 2000 image loader using jasper."),
+    NULL);
 
   gegl_extension_handler_register (".jp2", "gegl:jp2-load");
   gegl_extension_handler_register (".jpx", "gegl:jp2-load");
