@@ -21,30 +21,37 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
-gegl_chant_int    (epsilon, _("Epsilon"),
-                  -9, -1, -6,
-                  _("Log of the error weighting"))
-gegl_chant_int    (radius, _("Radius"),
-                  1, 3, 1,
-                  _("Radius of the processing window"))
-gegl_chant_double (threshold, _("Threshold"),
-                  0.0, 0.1, 0.02,
-                  _("Alpha threshold for multilevel processing"))
-gegl_chant_double (lambda, _("Lambda"),
-                  0.0, 100.0, 100.0, _("Trimap influence factor"))
-gegl_chant_int    (levels, _("Levels"),
-                   0, 8, 4,
-                   _("Number of downsampled levels to use"))
-gegl_chant_int    (active_levels, _("Active Levels"),
-                   0, 8, 2,
-                   _("Number of levels to perform solving"))
+#ifdef GEGL_PROPERTIES
+property_int   (epsilon, _("Epsilon"), -6)
+   description (_("Log of the error weighting"))
+   value_range (-9, -1)
+
+property_int    (radius, _("Radius"), 1)
+   description  (_("Radius of the processing window"))
+   value_range  (1, 3)
+
+property_double (threshold, _("Threshold"), 0.02)
+   description  (_("Alpha threshold for multilevel processing"))
+   value_range  (0.0, 0.1)
+
+property_double (lambda, _("Lambda"), 100.0)
+   value_range  (0.0, 100.0)
+   description  (_("Trimap influence factor"))
+
+property_int    (levels, _("Levels"), 4)
+   description  (_("Number of downsampled levels to use"))
+   value_range  (0, 8)
+
+property_int    (active_levels, _("Active levels"), 2)
+   description  (_("Number of levels to perform solving"))
+   value_range  (0, 8)
+
 #else
 
-#define GEGL_CHANT_TYPE_COMPOSER
-#define GEGL_CHANT_C_FILE       "matting-levin.c"
+#define GEGL_OP_COMPOSER
+#define GEGL_OP_C_SOURCE matting-levin.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include "gegl-debug.h"
 
 #include <stdlib.h>
@@ -57,7 +64,6 @@ gegl_chant_int    (active_levels, _("Active Levels"),
  */
 #if defined(HAVE_UMFPACK_H)
 #include <umfpack.h>
-#define SuiteSparse_long UF_long
 #elif defined (HAVE_SUITESPARSE_UMFPACK_H)
 #include <suitesparse/umfpack.h>
 #endif
@@ -763,8 +769,6 @@ matting_upsample (const gdouble       *restrict pixels,
   static const gdouble  UPSAMPLE_KERNEL[] =
     { 0.125, 0.5, 0.75, 0.5, 0.125 };
 
-  gint     x_start, x_end,
-           y_start, y_end;
   gint     x, y;
   guint    c;
   gdouble *newpix = NULL;
@@ -777,16 +781,11 @@ matting_upsample (const gdouble       *restrict pixels,
   g_return_val_if_fail (abs (output->width  - 2 * input->width ) <= 1, NULL);
   g_return_val_if_fail (abs (output->height - 2 * input->height) <= 1, NULL);
 
-  x_start = 1;
-  y_start = 1;
-  x_end   = output->width  - output->width  % 2;
-  y_end   = output->height - output->height % 2;
-
   newpix  = g_new0 (gdouble, output->width * output->height * components);
 
-  for (y = y_start; y < output->height; y += 2)
+  for (y = 1; y < output->height; y += 2)
     {
-      for (x = x_start; x < output->width; x += 2)
+      for (x = 1; x < output->width; x += 2)
         {
           guint newoff = (x     +  y      * output->width) * components,
                 oldoff = (x / 2 + (y / 2) * input->width ) * components;
@@ -888,8 +887,8 @@ matting_sparse_elems (const sparse_t *s)
 static gboolean
 matting_verify (const sparse_t *s)
 {
-  guint    i, j;
-  gboolean rows[s->rows];
+  guint     i, j;
+  gboolean *rows;
 
   /* Must be a square matrix */
   g_return_val_if_fail (s->columns == s->rows, FALSE);
@@ -914,16 +913,30 @@ matting_verify (const sparse_t *s)
    * not a requirement of the UMFPACK format; rather, something we expect of
    * the matrix from the matting algorithm.
    */
+  rows = g_new (gboolean, s->rows);
+
   for (i = 0; i < s->rows; ++i)
     rows [i] = FALSE;
   for (i = 0; i < matting_sparse_elems (s); ++i)
     {
       guint row = s->row_idx[i];
-      g_return_val_if_fail (row < s->rows, FALSE);
+      if (!(row < s->rows))
+        {
+          g_free (rows);
+          g_return_val_if_reached (FALSE);
+        }
       rows[row] = TRUE;
     }
   for (i = 0; i < s->rows; ++i)
-    g_return_val_if_fail (rows[i], FALSE);
+    {
+      if (!rows[i])
+        {
+          g_free (rows);
+          g_return_val_if_reached (FALSE);
+        }
+      }
+
+  g_free (rows);
 
   return TRUE;
 }
@@ -946,18 +959,18 @@ matting_get_laplacian (const gdouble       *restrict image,
                        const gdouble        epsilon,
                        const gdouble        lambda)
 {
-  gint              diameter     = radius * 2 + 1,
-                    window_elems = diameter * diameter,
-                    image_elems  = roi->width * roi->height,
-                    i, j, k, x, y,
-                    status;
+  gint      diameter     = radius * 2 + 1,
+            window_elems = diameter * diameter,
+            image_elems  = roi->width * roi->height,
+            i, j, k, x, y,
+            status;
   SuiteSparse_long *trip_col,
                    *trip_row;
-  glong             trip_nz = 0,
-                    trip_cursor = 0,
-                    trip_masked = 0;
-  gdouble          *trip_val;
-  sparse_t         *laplacian;
+  glong     trip_nz = 0,
+            trip_cursor = 0,
+            trip_masked = 0;
+  gdouble  *trip_val;
+  sparse_t *laplacian;
 
   gdouble       mean[COMPONENTS_INPUT],
          mean_matrix[COMPONENTS_INPUT][COMPONENTS_INPUT],
@@ -1084,7 +1097,7 @@ matting_get_laplacian (const gdouble       *restrict image,
     }
 
   {
-    gdouble row_sum[image_elems];
+    gdouble *row_sum = g_new (gdouble, image_elems);
 
     /* Calculate the sum of all the elements in each row */
     for (i = 0; i < image_elems; ++i)
@@ -1123,6 +1136,8 @@ matting_get_laplacian (const gdouble       *restrict image,
         g_warn_if_fail (float_cmp (row_sum [i], 0.0) ||
                         float_cmp (row_sum [i], lambda));
       }
+
+    g_free (row_sum);
   }
 
   g_warn_if_fail (trip_cursor == trip_nz);
@@ -1379,7 +1394,7 @@ matting_process (GeglOperation       *operation,
                  const GeglRectangle *result,
                  gint                 level)
 {
-  const GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  const GeglProperties *o = GEGL_PROPERTIES (operation);
   gdouble          *input   = NULL,
                    *trimap  = NULL;
   gdouble          *output  = NULL;
@@ -1419,7 +1434,7 @@ matting_process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
@@ -1434,9 +1449,10 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_cached_region       = matting_get_cached_region;
 
   gegl_operation_class_set_keys (operation_class,
-  "name"        , "gegl:matting-levin",
-  "categories"  , "misc",
-  "description" ,
+  "name",         "gegl:matting-levin",
+  "title",        _("Matting Levin"),
+  "categories",   "matting",
+  "description",
         _("Given a sparse user supplied tri-map and an input image, create a "
           "foreground alpha mat. Set white as selected, black as unselected, "
           "for the tri-map."),
