@@ -21,39 +21,43 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
+property_color  (fill, _("Fill Color"), "rgba(0.0,0.0,0.0,0.6)")
+  description(_("Color of paint to use for filling, use 0 opacity to disable filling"))
 
-gegl_chant_color  (fill, _("Fill Color"),  "rgba(0.0,0.0,0.0,0.6)",
-                         _("Color of paint to use for filling, use 0 opacity to disable filling"))
-gegl_chant_color  (stroke,    _("Stroke Color"),      "rgba(0.0,0.0,0.0,0.0)",
-                             _("Color of paint to use for stroking"))
+property_color  (stroke, _("Stroke Color"), "rgba(0.0,0.0,0.0,0.0)")
+  description(_("Color of paint to use for stroking"))
 
-gegl_chant_double (stroke_width,_("Stroke width"),  0.0, 200.0, 2.0,
-                             _("The width of the brush used to stroke the path"))
+property_double (stroke_width,_("Stroke width"), 2.0)
+  description (_("The width of the brush used to stroke the path"))
+  value_range (0.0, 200.0)
 
-gegl_chant_double (stroke_opacity,  _("Stroke opacity"),  -2.0, 2.0, 1.0,
-                             _("Opacity of stroke, note, does not behave like SVG since at the moment stroking is done using an airbrush tool"))
+property_double (stroke_opacity, _("Stroke opacity"), 1.0)
+  description (_("Opacity of stroke, note, does not behave like SVG since at the moment stroking is done using an airbrush tool"))
+  value_range (-2.0, 2.0)
 
-gegl_chant_double (stroke_hardness, _("Hardness"),   0.0, 1.0, 0.6,
-                             _("Hardness of the brush, 0.0 for a soft brush, 1.0 for a hard brush"))
+property_double (stroke_hardness, _("Hardness"), 0.6)
+  description (_("Hardness of the brush, 0.0 for a soft brush, 1.0 for a hard brush"))
+  value_range (0.0, 1.0)
 
-gegl_chant_string (fill_rule,_("Fill rule"), "nonzero",
-                             _("How to determine what to fill (nonzero|evenodd"))
+property_string (fill_rule,_("Fill rule"), "nonzero")
+  description(_("How to determine what to fill (nonzero|evenodd)"))
 
-gegl_chant_string (transform,_("Transform"), "",
-                             _("SVG style description of transform"))
+property_string (transform,_("Transform"), "")
+  description(_("SVG style description of transform"))
 
-gegl_chant_double (fill_opacity, _("Fill opacity"),  -2.0, 2.0, 1.0,
-                             _("The fill opacity to use"))
+property_double (fill_opacity, _("Fill opacity"), 1.0)
+  description(_("The fill opacity to use"))
+  value_range (-2.0, 2.0)
 
-gegl_chant_path   (d,        _("Vector"),
-                             _("A GeglVector representing the path of the stroke"))
+property_path (d, _("Vector"), NULL)
+  description (_("A GeglVector representing the path of the stroke"))
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE "path.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE path.c
 
 #include "gegl-plugin.h"
 #include "gegl-path.h"
@@ -61,8 +65,7 @@ static void path_changed (GeglPath *path,
                           const GeglRectangle *roi,
                           gpointer userdata);
 
-#include "gegl-chant.h"
-#include "gegl-buffer-private.h"
+#include "gegl-op.h"
 #include <cairo.h>
 #include <math.h>
 
@@ -126,10 +129,6 @@ gegl_path_stroke (GeglBuffer *buffer,
    {
      return;
    }
-  if (gegl_buffer_is_shared (buffer))
-    while (!gegl_buffer_try_lock (buffer));
-
-  /*gegl_buffer_clear (buffer, &extent);*/
 
   while (iter)
     {
@@ -204,9 +203,6 @@ gegl_path_stroke (GeglBuffer *buffer,
         }
       iter=iter->next;
     }
-
-  if (gegl_buffer_is_shared (buffer))
-    gegl_buffer_unlock (buffer);
 }
 
 static void
@@ -256,7 +252,7 @@ gegl_path_stamp (GeglBuffer *buffer,
     }
   g_assert (s.buf);
 
-  gegl_buffer_get_unlocked (buffer, 1.0, &roi, s.format, s.buf, 0);
+  gegl_buffer_get (buffer, &roi, 1.0, s.format, s.buf, 0, GEGL_ABYSS_NONE);
 
   {
     gint u, v;
@@ -294,7 +290,7 @@ gegl_path_stamp (GeglBuffer *buffer,
         }
     }
   }
-  gegl_buffer_set_unlocked (buffer, &roi, s.format, s.buf, 0);
+  gegl_buffer_set (buffer, &roi, 0, s.format, s.buf, 0);
   g_free (s.buf);
 }
 
@@ -303,7 +299,7 @@ static void path_changed (GeglPath *path,
                           gpointer userdata)
 {
   GeglRectangle rect = *roi;
-  GeglChantO    *o   = GEGL_CHANT_PROPERTIES (userdata);
+  GeglProperties    *o   = GEGL_PROPERTIES (userdata);
   /* invalidate the incoming rectangle */
 
   rect.x -= o->stroke_width/2;
@@ -317,8 +313,8 @@ static void path_changed (GeglPath *path,
 static void
 prepare (GeglOperation *operation)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  gegl_operation_set_format (operation, "output", babl_format ("RaGaBaA float"));
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  gegl_operation_set_format (operation, "output", babl_format ("R'aG'aB'aA float"));
   if (o->transform && o->transform[0] != '\0')
     {
       GeglMatrix3 matrix;
@@ -330,7 +326,7 @@ prepare (GeglOperation *operation)
 static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO    *o       = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties    *o       = GEGL_PROPERTIES (operation);
   GeglRectangle  defined = { 0, 0, 512, 512 };
   GeglRectangle *in_rect;
   gdouble        x0, x1, y0, y1;
@@ -361,33 +357,33 @@ process (GeglOperation       *operation,
          const GeglRectangle *result,
          gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (input)
     {
-      gegl_buffer_copy (input, result, output, result);
+      gegl_buffer_copy (input, result, GEGL_ABYSS_NONE,
+                        output, result);
     }
   else
     {
       gegl_buffer_clear (output, result);
     }
 
-
-
   if (o->fill_opacity > 0.0001 && o->fill)
     {
-      gdouble r,g,b,a;
-      gegl_color_get_rgba (o->fill, &r,&g,&b,&a);
-      a *= o->fill_opacity;
-      if (a>0.001)
+      gdouble color[4] = {0, 0, 0, 0};
+      gegl_color_get_pixel (o->fill, babl_format ("R'G'B'A double"), color);
+      color[3] *= o->fill_opacity;
+
+      if (color[3] > 0.001)
         {
-          GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+          static GMutex mutex = { 0, };
           cairo_t *cr;
           cairo_surface_t *surface;
           guchar *data;
 
-          g_static_mutex_lock (&mutex);
-          data = (void*)gegl_buffer_linear_open (output, result, NULL, babl_format ("B'aG'aR'aA u8"));
+          g_mutex_lock (&mutex);
+          data = gegl_buffer_linear_open (output, result, NULL, babl_format ("B'aG'aR'aA u8"));
           surface = cairo_image_surface_create_for_data (data,
                                                          CAIRO_FORMAT_ARGB32,
                                                          result->width,
@@ -402,10 +398,10 @@ process (GeglOperation       *operation,
             }
 
           gegl_path_cairo_play (o->d, cr);
-          cairo_set_source_rgba (cr, r,g,b,a);
+          cairo_set_source_rgba (cr, color[0], color[1], color[2], color[3]);
           cairo_fill (cr);
 
-          g_static_mutex_unlock (&mutex);
+          g_mutex_unlock (&mutex);
           gegl_buffer_linear_close (output, data);
         }
     }
@@ -460,7 +456,7 @@ static GeglNode *detect (GeglOperation *operation,
                          gint           x,
                          gint           y)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   cairo_t *cr;
   cairo_surface_t *surface;
   gchar *data = "     ";
@@ -498,10 +494,29 @@ static GeglNode *detect (GeglOperation *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
+  const gchar *composition =
+    "<gegl>"
+    "<node operation='gegl:crop' width='200' height='200'/>"
+    "<node operation='gegl:over'>"
+    "<node operation='gegl:translate' x='40' y='40'/>"
+    "<node operation='gegl:path'>"
+    "  <params>"
+    "    <param name='fill'>rgba(1,1,1,0.2)</param>"
+    "    <param name='stroke'>rgba(0,0,1,0.5)</param>"
+    "    <param name='stroke-hardness'>0.0</param>"
+    "    <param name='stroke-width'>8.0</param>"
+    "    <param name='d'>"
+    "M0,50 C0,78 24,100 50,100 C77,100 100,78 100,50 C100,45 99,40 98,35 C82,35 66,35 50,35 C42,35 35,42 35,50 C35,58 42,65 50,65 C56,65 61,61 64,56 C67,51 75,55 73,60 C69,  69 60,75 50,75 C36,75 25,64 25,50 C25,36 36,25 50,25 L93,25 C83,9 67,0 49,0 C25,0 0,20 0,50   z"
+    "                    </param>"
+    "  </params>"
+    "</node>"
+    "</node>"
+    "<node operation='gegl:checkerboard' color1='rgb(0.25,0.25,0.25)' color2='rgb(0.75,0.75,0.75)'/>"
+    "</gegl>";
 
   operation_class = GEGL_OPERATION_CLASS (klass);
   filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);
@@ -513,8 +528,10 @@ gegl_chant_class_init (GeglChantClass *klass)
   /*operation_class->no_cache = TRUE;*/
 
   gegl_operation_class_set_keys (operation_class,
-    "name"        , "gegl:path",
-    "categories"  , "render",
+    "name",         "gegl:path",
+    "title",        _("Render Path"),
+    "categories",   "render",
+    "reference-composition", composition,
     "description" , _("Renders a brush stroke"),
     NULL);
 }
