@@ -21,21 +21,23 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (path, _("File"), "",
-                   _("Target path and filename, use '-' for stdout."))
-gegl_chant_int    (compression, _("Compression"),
-                   1, 9, 1, _("PNG compression level from 1 to 9"))
-gegl_chant_int    (bitdepth, _("Bitdepth"),
-                   8, 16, 16, _("8 and 16 are amongst the currently accepted values."))
+property_file_path (path, _("File"), "")
+  description (_("Target path and filename, use '-' for stdout."))
+property_int    (compression, _("Compression"), 3)
+  description (_("PNG compression level from 1 to 9"))
+  value_range (1, 9)
+property_int    (bitdepth, _("Bitdepth"), 16)
+  description(_("8 and 16 are the currently accepted values."))
+  value_range (8, 16)
 
 #else
 
-#define GEGL_CHANT_TYPE_SINK
-#define GEGL_CHANT_C_FILE       "png-save.c"
+#define GEGL_OP_SINK
+#define GEGL_OP_C_SOURCE png-save.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <png.h>
 #include <stdio.h>
 
@@ -87,12 +89,7 @@ gegl_buffer_export_png (GeglBuffer  *gegl_buffer,
     }
 
   {
-    const Babl *babl; /*= gegl_buffer->format;*/
-
-    g_object_get (gegl_buffer, "format", &babl, NULL);
-
-    if (babl_format_get_type (babl, 0) != babl_type ("u8"))
-      bit_depth = 16;
+    const Babl *babl = gegl_buffer_get_format (gegl_buffer);
 
     if (bd == 16)
       bit_depth = 16;
@@ -205,7 +202,7 @@ gegl_png_save_process (GeglOperation       *operation,
                        const GeglRectangle *result,
                        gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   gegl_buffer_export_png (input, o->path, o->compression, o->bitdepth,
                           result->x, result->y,
@@ -215,7 +212,7 @@ gegl_png_save_process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass     *operation_class;
   GeglOperationSinkClass *sink_class;
@@ -227,7 +224,8 @@ gegl_chant_class_init (GeglChantClass *klass)
   sink_class->needs_full = TRUE;
 
   gegl_operation_class_set_keys (operation_class,
-  "name"       , "gegl:png-save",
+  "name",        "gegl:png-save",
+  "title",       _("PNG File Saver"),
   "categories" , "output",
   "description",
         _("PNG image saver (passes the buffer through, saves as a side-effect.)"),
