@@ -20,23 +20,24 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string  (path, _("File"), "",
-                    _("Target path and filename, use '-' for stdout."))
-gegl_chant_boolean (rawformat, _("Raw format"), TRUE, _("Raw format"))
-gegl_chant_int     (bitdepth, _("Bitdepth"),
-                    8, 16, 16,
-                    _("8 and 16 are amongst the currently accepted values."))
+property_file_path  (path, _("File"), "")
+    description     (_("Target path and filename, use '-' for stdout."))
+
+property_boolean (rawformat, _("Raw format"), TRUE)
+property_int     (bitdepth, _("Bitdepth"), 16)
+    description (_("8 and 16 are the currently accepted values."))
+    value_range (8, 16)
 
 #else
 
-#define GEGL_CHANT_TYPE_SINK
-#define GEGL_CHANT_C_FILE       "ppm-save.c"
+#define GEGL_OP_SINK
+#define GEGL_OP_C_SOURCE ppm-save.c
 
 #define CHANNEL_COUNT           3
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <stdio.h>
 
 typedef enum {
@@ -54,7 +55,6 @@ ppm_save_write(FILE    *fp,
                map_type type)
 {
   guint i;
-  gint retval;
 
   /* Write the header */
   fprintf (fp, "P%c\n%d %d\n", type, width, height );
@@ -75,7 +75,7 @@ ppm_save_write(FILE    *fp,
             }
         }
 
-      retval = fwrite (data, bpc, numsamples, fp);
+      fwrite (data, bpc, numsamples, fp);
     }
   else
     {
@@ -113,9 +113,10 @@ ppm_save_write(FILE    *fp,
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
-         const GeglRectangle *rect)
+         const GeglRectangle *rect,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   FILE     *fp;
   guchar   *data;
@@ -144,13 +145,13 @@ process (GeglOperation       *operation,
   switch (bpc)
     {
     case 1:
-      gegl_buffer_get (input, 1.0, rect, babl_format ("R'G'B' u8"), data,
-                       GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (input, rect, 1.0, babl_format ("R'G'B' u8"), data,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
       break;
 
     case 2:
-      gegl_buffer_get (input, 1.0, rect, babl_format ("R'G'B' u16"), data,
-                       GEGL_AUTO_ROWSTRIDE);
+      gegl_buffer_get (input, rect, 1.0, babl_format ("R'G'B' u16"), data,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
       break;
 
     default:
@@ -172,7 +173,7 @@ process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass     *operation_class;
   GeglOperationSinkClass *sink_class;
@@ -183,10 +184,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   sink_class->process = process;
   sink_class->needs_full = TRUE;
 
-  operation_class->name        = "gegl:ppm-save";
-  operation_class->categories  = "output";
-  operation_class->description =
-        _("PPM image saver (Portable pixmap saver.)");
+  gegl_operation_class_set_keys (operation_class,
+    "name",        "gegl:ppm-save",
+    "title",       _("PPM File Saver"),
+    "categories",  "output",
+    "description",
+        _("PPM image saver (Portable pixmap saver.)"),
+        NULL);
 
   gegl_extension_handler_register_saver (".ppm", "gegl:ppm-save");
 }
