@@ -21,28 +21,29 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
+property_color  (color, _("Color"),  "rgba(0.0,0.0,0.0,1.0)")
+  description(_("Color of paint to use for filling."))
 
-gegl_chant_color  (color,    _("Color"),  "rgba(0.0,0.0,0.0,0.6)",
-                             _("Color of paint to use for filling."))
+property_double (opacity,  _("Opacity"), 1.0)
+  description(_("The fill opacity to use."))
+  value_range (-2.0, 2.0)
 
-gegl_chant_double (opacity,  _("Opacity"),  -2.0, 2.0, 1.0,
-                             _("The fill opacity to use."))
+  /* XXX: replace with enum? */
+property_string (fill_rule,_("Fill rule."), "nonzero")
+  description(_("how to determine what to fill (nonzero|evenodd)"))
 
-gegl_chant_string (fill_rule,_("Fill rule."), "nonzero",
-                             _("how to determine what to fill (nonzero|evenodd"))
+property_string (transform,_("Transform"), "")
+  description (_("svg style description of transform."))
 
-gegl_chant_string (transform,_("Transform"), "",
-                             _("svg style description of transform."))
-
-gegl_chant_path   (d,        _("Vector"),
-                             _("A GeglVector representing the path of the stroke"))
+property_path (d, _("Vector"), NULL)
+  description (_("A GeglVector representing the path of the stroke"))
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE "vector-fill.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE vector-fill.c
 
 #include "gegl-plugin.h"
 
@@ -52,14 +53,14 @@ static void path_changed (GeglPath *path,
                           const GeglRectangle *roi,
                           gpointer userdata);
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <cairo.h>
 
 static void path_changed (GeglPath *path,
                           const GeglRectangle *roi,
                           gpointer userdata)
 {
-  GeglChantO    *o   = GEGL_CHANT_PROPERTIES (userdata);
+  GeglProperties    *o   = GEGL_PROPERTIES (userdata);
   GeglRectangle rect;
   gdouble        x0, x1, y0, y1;
 
@@ -75,8 +76,9 @@ static void path_changed (GeglPath *path,
 static void
 prepare (GeglOperation *operation)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  gegl_operation_set_format (operation, "output", babl_format ("RaGaBaA float"));
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  gegl_operation_set_format (operation, "output", babl_format ("R'aG'aB'aA float"));
+
   if (o->transform && o->transform[0] != '\0')
     {
       GeglMatrix3 matrix;
@@ -88,7 +90,7 @@ prepare (GeglOperation *operation)
 static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO    *o       = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties    *o       = GEGL_PROPERTIES (operation);
   GeglRectangle  defined = { 0, 0, 512, 512 };
   GeglRectangle *in_rect;
   gdouble        x0, x1, y0, y1;
@@ -116,15 +118,16 @@ static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gboolean need_fill = FALSE;
-  gdouble r,g,b,a;
+  gdouble color[4] = {0, 0, 0, 0};
 
   if (input)
     {
-      gegl_buffer_copy (input, result, output, result);
+      gegl_buffer_copy (input, result, GEGL_ABYSS_NONE, output, result);
     }
   else
     {
@@ -134,22 +137,21 @@ process (GeglOperation       *operation,
 
   if (o->opacity > 0.0001 && o->color)
     {
-      gegl_color_get_rgba (o->color, &r,&g,&b,&a);
-      a *= o->opacity;
-      if (a>0.001)
-          need_fill=TRUE;
+      gegl_color_get_pixel (o->color, babl_format ("R'G'B'A double"), color);
+      color[3] *= o->opacity;
+      if (color[3] > 0.001)
+        need_fill=TRUE;
     }
 
   if (need_fill)
     {
-      GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+      static GMutex mutex = { 0, };
       cairo_t *cr;
       cairo_surface_t *surface;
       guchar *data;
 
-
-      g_static_mutex_lock (&mutex);
-      data = (void*)gegl_buffer_linear_open (output, result, NULL, babl_format ("B'aG'aR'aA u8"));
+      g_mutex_lock (&mutex);
+      data = gegl_buffer_linear_open (output, result, NULL, babl_format ("cairo-ARGB32"));
       surface = cairo_image_surface_create_for_data (data,
                                                      CAIRO_FORMAT_ARGB32,
                                                      result->width,
@@ -162,12 +164,12 @@ process (GeglOperation       *operation,
           cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
 
       gegl_path_cairo_play (o->d, cr);
-      cairo_set_source_rgba (cr, r,g,b,a);
+      cairo_set_source_rgba (cr, color[0], color[1], color[2], color[3]);
       cairo_fill (cr);
       cairo_destroy (cr);
 
       gegl_buffer_linear_close (output, data);
-      g_static_mutex_unlock (&mutex);
+      g_mutex_unlock (&mutex);
     }
   return  TRUE;
 }
@@ -206,7 +208,7 @@ static GeglNode *detect (GeglOperation *operation,
                          gint           x,
                          gint           y)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   cairo_t *cr;
   cairo_surface_t *surface;
   gchar *data = "     ";
@@ -238,10 +240,27 @@ static GeglNode *detect (GeglOperation *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
+  gchar                    *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:crop' width='200' height='200'/>"
+    "<node operation='gegl:over'>"
+    "<node operation='gegl:translate' x='40' y='40'/>"
+    "<node operation='gegl:fill-path'>"
+    "  <params>"
+    "    <param name='color'>rgb(0.0, 0.6, 1.0)</param>"
+    "    <param name='d'>"
+    "M0,50 C0,78 24,100 50,100 C77,100 100,78 100,50 C100,45 99,40 98,35 C82,35 66,35 50,35 C42,35 35,42 35,50 C35,58 42,65 50,65 C56,65 61,61 64,56 C67,51 75,55 73,60 C69,  69 60,75 50,75 C36,75 25,64 25,50 C25,36 36,25 50,25 L93,25 C83,9 67,0 49,0 C25,0 0,20 0,50   z"
+    "                    </param>"
+    "  </params>"
+    "</node>"
+    "</node>"
+    "<node operation='gegl:checkerboard' color1='rgb(0.25,0.25,0.25)' color2='rgb(0.75,0.75,0.75)'/>"
+    "</gegl>";
+
 
   operation_class = GEGL_OPERATION_CLASS (klass);
   filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);
@@ -251,9 +270,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare = prepare;
   operation_class->detect = detect;
 
-  operation_class->name        = "gegl:fill-path";
-  operation_class->categories  = "render";
-  operation_class->description = _("Renders a filled region");
+  gegl_operation_class_set_keys (operation_class,
+    "name",        "gegl:fill-path",
+    "title",       _("Fill Path"),
+    "categories" , "render",
+    "description", _("Renders a filled region"),
+    "reference-composition", composition,
+    NULL);
 }
 
 
