@@ -21,17 +21,18 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_int (pattern, _("Bayer pattern"), 0, 3, 0,
-                _("Bayer pattern used, 0 seems to work for some nikon files, 2 for some Fuji files."))
+property_int (pattern, _("Bayer pattern"), 0)
+   description (_("Bayer pattern used, 0 seems to work for some nikon files, 2 for some Fuji files."))
+   value_range (0, 3)
 
 #else
 
-#define GEGL_CHANT_TYPE_AREA_FILTER
-#define GEGL_CHANT_C_FILE       "demosaic-bimedian.c"
+#define GEGL_OP_AREA_FILTER
+#define GEGL_OP_C_SOURCE demosaic-bimedian.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 
 /* Returns the median of four floats. We define the median as the average of
@@ -79,7 +80,7 @@ m4 (gfloat a, gfloat b, gfloat c, gfloat d)
  * of dst_extent.
  */
 static void
-demosaic (GeglChantO          *op,
+demosaic (GeglProperties          *op,
           GeglBuffer          *src,
           const GeglRectangle *src_rect,
           GeglBuffer          *dst,
@@ -93,8 +94,8 @@ demosaic (GeglChantO          *op,
   src_buf = g_new0 (gfloat, src_rect->width * src_rect->height * 1);
   dst_buf = g_new0 (gfloat, dst_rect->width * dst_rect->height * 3);
 
-  gegl_buffer_get (src, 1.0, src_rect, babl_format ("Y float"), src_buf,
-           GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (src, src_rect, 1.0, babl_format ("Y float"), src_buf,
+                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   offset = ROW + COL;
   doffset = 0;
@@ -167,7 +168,7 @@ demosaic (GeglChantO          *op,
       offset+=2;
     }
 
-  gegl_buffer_set (dst, dst_rect, babl_format ("RGB float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (dst, dst_rect, 0, babl_format ("RGB float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
@@ -189,9 +190,10 @@ static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   GeglRectangle src_rect = gegl_operation_get_required_for_output (operation, "input", result);
 
   demosaic (o, input, &src_rect, output, result);
@@ -201,7 +203,7 @@ process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -212,10 +214,12 @@ gegl_chant_class_init (GeglChantClass *klass)
   filter_class->process    = process;
   operation_class->prepare = prepare;
 
-  operation_class->name        = "gegl:demosaic-bimedian";
-  operation_class->categories  = "blur";
-  operation_class->description =
-        _("Performs a grayscale2color demosaicing of an image, using bimedian interpolation.");
+  gegl_operation_class_set_keys (operation_class,
+    "name"        , "gegl:demosaic-bimedian",
+    "categories"  , "blur",
+    "description" ,
+          _("Performs a grayscale2color demosaicing of an image, using bimedian interpolation."),
+          NULL);
 }
 
 #endif
