@@ -20,7 +20,7 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
 #define THRESHOLD_SCRIPT \
 "level = user_value/2\n"\
@@ -38,18 +38,23 @@
 "  progress (y/height)\n"\
 "end"
 
-gegl_chant_multiline (script, _("Script"), THRESHOLD_SCRIPT,
-         _("The lua script containing the implementation of this operation."))
-gegl_chant_file_path (file, _("File"), "", _("a stored lua script on disk implementing an operation."))
-gegl_chant_double (user_value, _("User value"), -1000.0, 1000.0, 1.0,
-         _("(appears in the global variable 'user_value' in lua."))
+property_string (script, _("Script"), THRESHOLD_SCRIPT)
+    description(_("The lua script containing the implementation of this operation."))
+    ui_meta    ("multiline", "true")
+
+property_file_path (file, _("File"), "")
+    description(_("a stored lua script on disk implementing an operation."))
+
+property_double (user_value, _("User value"), 1.0)
+    description(_("(appears in the global variable 'user_value' in lua."))
+    value_range (-1000.0, 1000.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_COMPOSER
-#define GEGL_CHANT_C_FILE       "gluas.c"
+#define GEGL_OP_COMPOSER
+#define GEGL_OP_C_SOURCE gluas.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
@@ -60,7 +65,7 @@ typedef struct Priv
   GeglBuffer *in_drawable;
   GeglBuffer *aux_drawable;
   GeglBuffer *out_drawable;
-  Babl       *rgba_float;
+  const Babl *rgba_float;
 
   gint        bx1, by1;
   gint        bx2, by2;    /* mask bounds */
@@ -97,7 +102,7 @@ static int l_progress  (lua_State * lua);
 static int l_flush     (lua_State * lua);
 static int l_print     (lua_State * lua);
 
-static const luaL_reg gluas_functions[] =
+static const luaL_Reg gluas_functions[] =
 {
     {"set_rgba",    l_set_rgba},
     {"get_rgba",    l_get_rgba},
@@ -122,7 +127,7 @@ static const luaL_reg gluas_functions[] =
 };
 static void
 register_functions (lua_State      *L,
-                    const luaL_reg *l)
+                    const luaL_Reg *l)
 {
   for (;l->name; l++)
     lua_register (L, l->name, l->func);
@@ -146,7 +151,7 @@ drawable_lua_process (GeglOperation       *op,
     lua_State *L;
     Priv p;
 
-    L = lua_open ();
+    L = luaL_newstate ();
     luaL_openlibs (L);
 
     register_functions (L, gluas_functions);
@@ -244,7 +249,7 @@ get_rgba_pixel (void       *data,
         return;
       gegl_buffer_sample (p->in_drawable, x, y, NULL, buf,
                           p->rgba_float,
-                          GEGL_SAMPLER_NEAREST);
+                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -255,7 +260,7 @@ get_rgba_pixel (void       *data,
         return;
       gegl_buffer_sample (p->aux_drawable, x, y, NULL, buf,
                           p->rgba_float,
-                          GEGL_SAMPLER_NEAREST);
+                          GEGL_SAMPLER_NEAREST, GEGL_ABYSS_NONE);
       for (i = 0; i < 4; i++)
         pixel[i] = buf[i];
     }
@@ -287,7 +292,7 @@ set_rgba_pixel (void       *data,
     }
 
  gegl_buffer_set (p->out_drawable, &roi,
-                  p->rgba_float, buf,
+                  0, p->rgba_float, buf,
                   GEGL_AUTO_ROWSTRIDE);
 }
 
@@ -937,9 +942,10 @@ process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *aux,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
 
   if (o->file && g_file_test (o->file, G_FILE_TEST_IS_REGULAR))
     {
@@ -963,7 +969,7 @@ get_required_for_output (GeglOperation        *operation,
 }
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
@@ -976,11 +982,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_bounding_box = get_bounding_box;
   operation_class->get_required_for_output = get_required_for_output;
 
-  operation_class->name        = "gegl:gluas";
-  operation_class->categories  = "script:misc";
-  operation_class->description =
+  gegl_operation_class_set_keys (operation_class,
+  "name"        , "gegl:gluas",
+  "categories"  , "script:misc",
+  "description" ,
         _("A general purpose filter/composer implementation proxy for the"
-          " lua programming language.");
+          " lua programming language."),
+        NULL);
 }
 
 #endif
