@@ -20,17 +20,18 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (radius, _("Radius"), 0.0, 50.0, 10.0,
-  _("Radius of square pixel region (width and height will be radius*2+1)"))
+property_double (radius, _("Radius"), 10.0)
+   value_range (0.0, 50.0)
+   description (_("Radius of square pixel region (width and height will be radius*2+1)"))
 
 #else
 
-#define GEGL_CHANT_TYPE_AREA_FILTER
-#define GEGL_CHANT_C_FILE       "kuwahara.c"
+#define GEGL_OP_AREA_FILTER
+#define GEGL_OP_C_SOURCE kuwahara.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 
 static inline void
@@ -96,7 +97,8 @@ kuwahara (GeglBuffer *src,
   src_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (src) * 4);
   dst_buf = g_new0 (gfloat, gegl_buffer_get_pixel_count (dst) * 4);
 
-  gegl_buffer_get (src, 1.0, NULL, babl_format ("RGBA float"), src_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (src, NULL, 1.0, babl_format ("RGBA float"), src_buf,
+                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   offset = 0;
   for (v=0; v<gegl_buffer_get_height (dst); v++)
@@ -190,7 +192,7 @@ kuwahara (GeglBuffer *src,
           offset++;
       }
 
-  gegl_buffer_set (dst, NULL, babl_format ("RGBA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (dst, NULL, 0, babl_format ("RGBA float"), dst_buf, GEGL_AUTO_ROWSTRIDE);
   g_free (src_buf);
   g_free (dst_buf);
 }
@@ -200,7 +202,7 @@ static void prepare (GeglOperation *operation)
   GeglOperationAreaFilter *area = GEGL_OPERATION_AREA_FILTER (operation);
 
   area->left = area->right = area->top = area->bottom =
-      ceil (GEGL_CHANT_PROPERTIES (operation)->radius);
+      ceil (GEGL_PROPERTIES (operation)->radius);
   gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
 }
 
@@ -208,9 +210,10 @@ static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties   *o = GEGL_PROPERTIES (operation);
   GeglBuffer   *temp_in;
   GeglRectangle compute = gegl_operation_get_required_for_output (operation, "input", result);
 
@@ -224,7 +227,7 @@ process (GeglOperation       *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -235,9 +238,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   filter_class->process = process;
   operation_class->prepare = prepare;
 
-  operation_class->name        = "gegl:kuwahara";
-  operation_class->categories  = "misc";
-  operation_class->description = _("Edge preserving blur");
+  gegl_operation_class_set_keys (operation_class,
+    "name"    , "gegl:kuwahara",
+    "categories"  , "misc",
+    "description" , _("Edge preserving blur"),
+    NULL);
 }
 
 #endif
