@@ -4,7 +4,7 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <glib/gmacros.h>
+#include <glib.h>
 #include <glib/gstdio.h>
 
 /* This file consists of a testing suite for the GeglBuffer API. For every
@@ -61,6 +61,7 @@ static void fill                  (GeglBuffer *buffer,
                                    gfloat      value);
 
 static void vgrad                 (GeglBuffer *buffer);
+static void hgrad                 (GeglBuffer *buffer);
 
 static void rectangle             (GeglBuffer *buffer,
                                    gint        x,
@@ -149,7 +150,8 @@ print_buffer_internal (GString    *gstring,
                         "height", &height,
                         NULL);
   buf = g_malloc (width*height*sizeof(gfloat));
-  gegl_buffer_get (buffer, 1.0, NULL, babl_format ("Y float"), buf, 0);
+  gegl_buffer_get (buffer, NULL, 1.0, babl_format ("Y float"), buf, 0,
+                   GEGL_ABYSS_NONE);
   print_linear_buffer_internal_float (gstring, width, height, buf);
   g_free (buf);
 }
@@ -168,7 +170,8 @@ fill (GeglBuffer *buffer,
                         "height", &height,
                         NULL);
   buf = g_malloc (width*height*sizeof(gfloat));
-  gegl_buffer_get (buffer, 1.0, NULL, babl_format ("Y float"), buf, 0);
+  gegl_buffer_get (buffer, NULL, 1.0, babl_format ("Y float"), buf, 0,
+                   GEGL_ABYSS_NONE);
 
   i=0;
   for (y=0;y<height;y++)
@@ -178,7 +181,7 @@ fill (GeglBuffer *buffer,
           buf[i++]=value;
         }
     }
-  gegl_buffer_set (buffer, NULL, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (buffer, NULL, 0, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
   g_free (buf);
 }
 
@@ -197,7 +200,8 @@ static void checkerboard          (GeglBuffer *buffer,
                         "height", &height,
                         NULL);
   buf = g_malloc (width*height*sizeof(gfloat));
-  gegl_buffer_get (buffer, 1.0, NULL, babl_format ("Y float"), buf, 0);
+  gegl_buffer_get (buffer, NULL, 1.0, babl_format ("Y float"), buf, 0,
+                   GEGL_ABYSS_NONE);
 
   i=0;
   for (y=0;y<height;y++)
@@ -223,7 +227,7 @@ static void checkerboard          (GeglBuffer *buffer,
         }
     }
 
-  gegl_buffer_set (buffer, NULL, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (buffer, NULL, 0, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
 
   g_free (buf);
 }
@@ -240,7 +244,8 @@ static void vgrad (GeglBuffer *buffer)
                         "height", &height,
                         NULL);
   buf = g_malloc (width*height*sizeof(gfloat));
-  gegl_buffer_get (buffer, 1.0, NULL, babl_format ("Y float"), buf, 0);
+  gegl_buffer_get (buffer, NULL, 1.0, babl_format ("Y float"), buf, 0,
+                   GEGL_ABYSS_NONE);
 
   i=0;
   for (y=0;y<height;y++)
@@ -250,7 +255,34 @@ static void vgrad (GeglBuffer *buffer)
           buf[i++]= (1.0*y)/height;
         }
     }
-  gegl_buffer_set (buffer, NULL, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_set (buffer, NULL, 0, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
+  g_free (buf);
+}
+
+static void hgrad (GeglBuffer *buffer)
+{
+  gfloat *buf;
+  gint x,y;
+  gint i;
+  gint width, height, x0, y0;
+  g_object_get (buffer, "x", &x0,
+                        "y", &y0,
+                        "width", &width,
+                        "height", &height,
+                        NULL);
+  buf = g_malloc (width*height*sizeof(gfloat));
+  gegl_buffer_get (buffer, NULL, 1.0, babl_format ("Y float"), buf, 0,
+                   GEGL_ABYSS_NONE);
+
+  i=0;
+  for (y=0;y<height;y++)
+    {
+      for (x=0;x<width;x++)
+        {
+          buf[i++]= (1.0*x)/width;
+        }
+    }
+  gegl_buffer_set (buffer, NULL, 0, babl_format ("Y float"), buf, GEGL_AUTO_ROWSTRIDE);
   g_free (buf);
 }
 
@@ -260,7 +292,8 @@ static void fill_rect (GeglBuffer          *buffer,
                        )
 {
   GeglBufferIterator *gi;
-  gi = gegl_buffer_iterator_new (buffer, roi, NULL, GEGL_BUFFER_WRITE);
+  gi = gegl_buffer_iterator_new (buffer, roi, 0, NULL,
+                                 GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);
   while (gegl_buffer_iterator_next (gi))
     {
       gfloat *buf = gi->data[0];
@@ -284,7 +317,7 @@ void rectangle (GeglBuffer *buffer,
 
   sub_buf = gegl_buffer_create_sub_buffer (buffer, &rect);
   fill (sub_buf, value);
-  gegl_buffer_destroy (sub_buf);
+  g_object_unref (sub_buf);
 }
 
 #include "buffer-tests.inc"
@@ -292,7 +325,6 @@ void rectangle (GeglBuffer *buffer,
 gint main (gint argc, gchar **argv)
 {
   gint i;
-  g_thread_init (NULL);
   gegl_init (&argc, &argv);
 
   for (i=0; i < G_N_ELEMENTS (tests); i++)
